package com.piacere.selfhealing.backend.config;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.web.servlet.ServletContextInitializer;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.net.InetAddress;
import java.util.Map;

@Configuration
@AutoConfigureBefore(value = {WebConfigurer.class, AsyncConfiguration.class})
public class HostsConfigurer implements ServletContextInitializer {

    private final Logger log = LoggerFactory.getLogger(WebConfigurer.class);

    private final Environment env;

    private final ApplicationProperties applicationProperties;

    public HostsConfigurer(Environment env, ApplicationProperties applicationProperties) {
        this.env = env;
        this.applicationProperties = applicationProperties;
    }

    @Override
    public void onStartup(ServletContext servletContext) throws ServletException {
        processApplicationAddHosts();
        log.info("Hosts fully configured");
    }

    private void processApplicationAddHosts() {
        if (applicationProperties.getAddHosts() == null) return;

        // locate system /etc/hosts file either in linux or windows
        String etcHostsFilePath = null;
        if (System.getProperty("os.name").toLowerCase().contains("win")) {
            etcHostsFilePath = "C:\\Windows\\System32\\drivers\\etc\\hosts";
        } else {
            etcHostsFilePath = "/etc/hosts";
        }

        // create a temporary file and copy the content of /etc/hosts file
        File tmpFile = null;
        try {
            tmpFile = File.createTempFile("hosts", ".tmp");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        if (etcHostsFilePath != null) {
            try {
                org.apache.commons.io.FileUtils.copyFile(new File(etcHostsFilePath), tmpFile);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        } else {
            // create empty file
            try {
                tmpFile.createNewFile();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        for (Map.Entry<String, String> entry : applicationProperties.getAddHosts().entrySet()) {
            log.info("Setting system property: " + entry.getKey() + " = " + entry.getValue());
            // append to temporary file
            try {
                org.apache.commons.io.FileUtils.writeStringToFile(tmpFile, entry.getValue() + " " + entry.getKey() + "\n", "UTF-8", true);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        String hostsFileName = tmpFile.getAbsolutePath();
        System.setProperty("jdk.net.hosts.file", hostsFileName);
        try {
            Field fieldHostsFileName = InetAddress.class.getDeclaredField("nameService");
            fieldHostsFileName.setAccessible(true);
            Method methodCreateNameService = InetAddress.class.getDeclaredMethod("createNameService");
            methodCreateNameService.setAccessible(true);
            Object nameService = methodCreateNameService.invoke(null);
            fieldHostsFileName.set(null, nameService);
        } catch (Exception e) {
            e.printStackTrace();
            // exit application
            System.exit(1);
        }
    }

}
