package com.piacere.selfhealing.backend.consumer;

import io.vavr.control.Either;
import java.time.Duration;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.errors.WakeupException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.piacere.selfhealing.backend.serde.DeserializationError;


public abstract class GenericConsumer<T> implements Runnable {

	private final Logger log = LoggerFactory.getLogger(GenericConsumer.class);

    private final AtomicBoolean closed = new AtomicBoolean(false);

    private final String topicName;
    private final int pollingTimeout;

    private KafkaConsumer<String, Either<DeserializationError, T>> consumer;

    private final Map <String, Object> properties;

    public GenericConsumer(final String topicName, final Map<String, Object> properties, final int pollingTimeout) {
        this.topicName = topicName;
        this.pollingTimeout = pollingTimeout;
        this.properties = properties;
    }

    @PostConstruct
    public void init() {
        Runtime.getRuntime().addShutdownHook(new Thread(this::shutdown));
    }

    @PreDestroy
    public void destroy() {
        shutdown();
    }

    @Override
    public void run() {
        while (true) {
            consumer = new KafkaConsumer<>(properties);
            log.debug("Polling records from topic {}", topicName);
            try {
                consumer.subscribe(Collections.singleton(topicName));
                while (!closed.get()) {
                    final ConsumerRecords<String, Either<DeserializationError, T>> records = consumer.poll(Duration.ofMillis(pollingTimeout));
                    for (final ConsumerRecord<String, Either<DeserializationError, T>> record : records) {
                        handleMessage(record);
                    }
                    consumer.commitSync();
                }
            } catch (final WakeupException e) {
                // Ignore exception if closing
                if (!closed.get()) throw e;
            } catch (final Exception e) {
                log.error("An error occurred while trying to poll records from topic!", e);
            } finally {
                // Close consumer gracefully and destroy the instance
                consumer.close();
            }
        }
    }

    // Shutdown hook which can be called from a separate thread
    public void shutdown() {
        closed.set(true);
        if (consumer != null) consumer.wakeup();
    }

    protected abstract void handleMessage(ConsumerRecord<String, Either<DeserializationError, T>> record);


}
