package com.piacere.selfhealing.backend.service;

import com.piacere.selfhealing.backend.domain.Deployment;
import com.piacere.selfhealing.backend.domain.StrategyApplication;
import net.lingala.zip4j.ZipFile;
import net.lingala.zip4j.exception.ZipException;
import net.lingala.zip4j.model.ZipParameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

@Service
public class BundleService {
    private final Logger log = LoggerFactory.getLogger(BundleService.class);

    public Path unpackBundle(Deployment deployment) {
        Path tmpDir = createTmpDir();

        // save base64 encoded deployment bundle to tmp directory
        Path deploymentBundlePath = tmpDir.resolve("bundle.zip");
        log.info("Writing deployment bundle to {}", deploymentBundlePath);
        try {
            Files.write(deploymentBundlePath, deployment.getBundle());
        } catch (IOException e) {
            log.error("Error writing deployment bundle to tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        // unpack deployment bundle into tmp directory with ZipFile().extractAll()
        log.info("Unpacking deployment bundle");
        try {
            new ZipFile(deploymentBundlePath.toString()).extractAll(tmpDir.toString());
        } catch (ZipException e) {
            log.error("Error unpacking deployment bundle");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        return tmpDir;
    }

    public byte[] getByteArrayFromFile(Path domlPath) {
        // get byte[] from domlxPath file
        byte[] domlBytes = null;
        try {
            domlBytes = Files.readAllBytes(domlPath);
        } catch (IOException e) {
            log.error("Error reading doml file", e);
            throw new RuntimeException(e);
        }
        return domlBytes;
    }

    public Path createTmpDir() {
        // create a random folder name in tmp directory
        Path tmpDir = null;
        try {
            tmpDir = Files.createTempDirectory("piacere-");
        } catch (IOException e) {
            log.error("Error creating tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        return tmpDir;
    }

    public void packBundle(StrategyApplication strategyApplication, Path deploymentUnpackTmpDir) {
        Path tempDir = createTmpDir();
        Path bundlePath = tempDir.resolve("bundle.zip");
        packBundle(deploymentUnpackTmpDir, bundlePath);
        byte[] bundleBytes = getByteArrayFromFile(bundlePath);
        strategyApplication.setBundle(bundleBytes);
        strategyApplication.bundleContentType("application/zip");
    }

    public void packBundle(Path deploymentUnpackTmpDir, Path bundlePath) {
        log.info("Packing bundle to {}", bundlePath);
        ZipParameters zipParameters = new ZipParameters();
        zipParameters.setIncludeRootFolder(false);
        try {
            new ZipFile(bundlePath.toString()).addFolder(deploymentUnpackTmpDir.toFile(), zipParameters);
        } catch (ZipException e) {
            log.error("Error packing bundle");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }
}
