package com.piacere.selfhealing.backend.service;

import com.piacere.selfhealing.backend.domain.Deployment;
import com.piacere.selfhealing.backend.domain.StrategyApplication;
import eu.piacere.doml.DomlStandaloneSetup;
import eu.piacere.doml.doml.application.ApplicationPackage;
import eu.piacere.doml.doml.commons.CommonsPackage;
import eu.piacere.doml.doml.commons.DOMLModel;
import eu.piacere.doml.doml.concrete.ConcretePackage;
import eu.piacere.doml.doml.infrastructure.InfrastructurePackage;
import eu.piacere.doml.doml.optimization.OptimizationPackage;
import net.lingala.zip4j.ZipFile;
import net.lingala.zip4j.exception.ZipException;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;

@Service

public class DOMLService {
    private final Logger log = LoggerFactory.getLogger(DOMLService.class);
    private final BundleService bundleService;

    public DOMLService(BundleService bundleService) {
        this.bundleService = bundleService;

        DomlStandaloneSetup.doSetup();
        if (!EPackage.Registry.INSTANCE.containsKey(CommonsPackage.eNS_URI)) {
            EPackage.Registry.INSTANCE.put(CommonsPackage.eNS_URI, CommonsPackage.eINSTANCE);
        }
        if (!EPackage.Registry.INSTANCE.containsKey(ApplicationPackage.eNS_URI)) {
            EPackage.Registry.INSTANCE.put(ApplicationPackage.eNS_URI, ApplicationPackage.eINSTANCE);
        }
        if (!EPackage.Registry.INSTANCE.containsKey(InfrastructurePackage.eNS_URI)) {
            EPackage.Registry.INSTANCE.put(InfrastructurePackage.eNS_URI, InfrastructurePackage.eINSTANCE);
        }
        if (!EPackage.Registry.INSTANCE.containsKey(ConcretePackage.eNS_URI)) {
            EPackage.Registry.INSTANCE.put(ConcretePackage.eNS_URI, ConcretePackage.eINSTANCE);
        }
        if (!EPackage.Registry.INSTANCE.containsKey(OptimizationPackage.eNS_URI)) {
            EPackage.Registry.INSTANCE.put(OptimizationPackage.eNS_URI, OptimizationPackage.eINSTANCE);
        }
        if (!Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().containsKey("domlx")) {
            Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("domlx", new XMIResourceFactoryImpl());
        }
        if (!Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().containsKey("doml")) {
            Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("doml", new XMIResourceFactoryImpl());
        }
    }
    @NotNull
    public DOMLModel getDomlxModel(Deployment deployment) {
        log.info("unpacking deployment bundle for deployment {}", deployment.getDeploymentId());
        Path tmpDir = bundleService.unpackBundle(deployment);

        // find the domlx file in the tmp directory
        Path domlxPath = null;
        try {
            domlxPath = Files.list(tmpDir).filter(path -> path.toString().endsWith(".domlx")).findFirst().orElseThrow(() -> new RuntimeException("domlx file not found"));
        } catch (IOException e) {
            log.error("Error finding domlx file");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        log.info("Found domlx file {}", domlxPath);

        return getDomlxModel(domlxPath);
    }
    public DOMLModel getDomlxModel(Path domlxPath) {
        // create inputStream from domlxPath file
        InputStream inputStream = null;
        try {
            inputStream = new FileInputStream(domlxPath.toFile());
        } catch (FileNotFoundException e) {
            log.error("Error creating input stream from domlx file", e);
            throw new RuntimeException(e);
        }

        ResourceSet resourceSet = new ResourceSetImpl();

        Resource inputResource = resourceSet.createResource(URI.createFileURI(domlxPath.toString()));

        try{
            inputResource.load(inputStream, null);
        } catch (Exception e) {
            log.error("Error while loading domlx into ecore resource");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        //get Doml model
        DOMLModel doml = (DOMLModel) inputResource.getContents().get(0);
        if (doml == null) {
            log.error("Error while loading domlx into ecore resource");
            throw new RuntimeException("Error while loading domlx into ecore resource");
        }
        return doml;
    }
    public void saveDomlModel(StrategyApplication strategyApplication, DOMLModel doml) {

        Path tmpDir = bundleService.createTmpDir();

        Path domlPath = tmpDir.resolve("tmp.doml");

        saveDomlModel(domlPath, doml);

        byte[] domlBytes = bundleService.getByteArrayFromFile(domlPath);

        strategyApplication.setBundle(domlBytes);
    }

    public void saveDomlModel(Path domlPath, DOMLModel doml) {
        ResourceSet resourceSet = new ResourceSetImpl();
        Resource targetResource = resourceSet.createResource(URI.createFileURI(domlPath.toString()));
        try{
            targetResource.getContents().add(EcoreUtil.copy(doml));
            targetResource.save(null);
        } catch (Exception e) {
            log.error("Error while saving domlx into ecore resource");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    public void saveDomlxModel(Path domlxPath, DOMLModel doml) {
        ResourceSet resourceSet = new ResourceSetImpl();
        Resource targetResource = resourceSet.createResource(URI.createFileURI(domlxPath.toString()));
        try{
            targetResource.getContents().add(EcoreUtil.copy(doml));
            targetResource.save(null);
        } catch (Exception e) {
            log.error("Error while saving domlx into ecore resource");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    public DOMLModel getDomlModel(Path domlPath) {
        // create inputStream from domlPath file
        InputStream inputStream = null;
        try {
            inputStream = new FileInputStream(domlPath.toFile());
        } catch (FileNotFoundException e) {
            log.error("Error creating input stream from doml file", e);
            throw new RuntimeException(e);
        }

        ResourceSet resourceSet = new ResourceSetImpl();

        Resource inputResource = resourceSet.createResource(URI.createFileURI(domlPath.toString()));

        try{
            inputResource.load(inputStream, null);
        } catch (Exception e) {
            log.error("Error while loading doml into ecore resource");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        //get Doml model
        DOMLModel doml = (DOMLModel) inputResource.getContents().get(0);
        if (doml == null) {
            log.error("Error while loading doml into ecore resource");
            throw new RuntimeException("Error while loading doml into ecore resource");
        }
        return doml;
    }
}
