package com.piacere.selfhealing.backend.service;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.piacere.selfhealing.backend.config.ApplicationProperties;
import com.piacere.selfhealing.backend.domain.*;
import com.piacere.selfhealing.backend.repository.*;
import com.piacere.selfhealing.backend.web.rest.EventTypeResource;
import com.piacere.selfhealing.backend.web.rest.StrategyResource;
import eu.piacere.doml.doml.commons.DOMLModel;
import eu.piacere.doml.doml.infrastructure.InfrastructureLayer;
import eu.piacere.doml.doml.infrastructure.MonitoringRule;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.net.URISyntaxException;

@Service
public class DeploymentService {
    private final Logger log = LoggerFactory.getLogger(DeploymentService.class);
    private final EventTypeRepository eventTypeRepository;
    private final StrategyRepository strategyRepository;
    private final StrategyResource strategyResource;
    private final EventTypeResource eventTypeResource;
    private final ApplicationProperties applicationProperties;
    private final DeploymentRepository deploymentRepository;
    private final StrategyApplicationRepository strategyApplicationRepository;
    private final SelfHealingMessageRepository selfHealingMessageRepository;
    private final DOMLService domlService;

    public DeploymentService(EventTypeRepository eventTypeRepository, StrategyRepository strategyRepository, StrategyResource strategyResource, EventTypeResource eventTypeResource, ApplicationProperties applicationProperties, DeploymentRepository deploymentRepository, StrategyApplicationRepository strategyApplicationRepository, DOMLService domlService, SelfHealingMessageRepository selfHealingMessageRepository) {
        this.eventTypeRepository = eventTypeRepository;
        this.strategyRepository = strategyRepository;
        this.strategyResource = strategyResource;
        this.eventTypeResource = eventTypeResource;
        this.applicationProperties = applicationProperties;
        this.deploymentRepository = deploymentRepository;
        this.strategyApplicationRepository = strategyApplicationRepository;
        this.domlService = domlService;
        this.selfHealingMessageRepository = selfHealingMessageRepository;
    }

    @Async
    public void configure(Deployment deployment) {

        DOMLModel doml = domlService.getDomlxModel(deployment);

        //get the infrastructure model
        InfrastructureLayer infrastructureLayer = doml.getInfrastructure();
        if (infrastructureLayer == null) {
            log.error("Error while loading domlx into ecore resource");
            throw new RuntimeException("Error while loading domlx into ecore resource");
        }

        // iterate monitoring rules and create event types
        for (MonitoringRule monitoringRule : infrastructureLayer.getRules()) {
            log.info("Creating event type for monitoring rule {}", monitoringRule.getName());
            String eventType_Id = deployment.getDeploymentId() + "-" + monitoringRule.getName();
            // if the eventType_Id already exists then skip
            EventType eventType = eventTypeRepository.findByCode(eventType_Id);
            if (eventType != null ) {
                log.info("Event type {} already exists", eventType_Id);
                continue;
            }

            // if the strategy is not ansible then find the strategy and create the event type
            String strategy_id = monitoringRule.getStrategy();
            // if strategy_id is ansible log
            if (strategy_id.equals("ansible")) {
                log.info("ansible strategy need to create custom strategy with the configuration");
                String strategyConfigJsonString = monitoringRule.getStrategyConfigurationString();

                JsonObject strategyConfigJson = new JsonObject();
                try {
                    strategyConfigJson = JsonParser.parseString(strategyConfigJsonString).getAsJsonObject();
                } catch (Exception e) {
                    log.debug("Error loading strategy configuration");
                    continue;
                }

                /**
                 * "{\"repeat_not_before\": \"PT1M\",\"ansible\": \"{% raw %}---\n- hosts: servers_for_self_healing_monitoring\n  tasks:\n  - name: reboot\n    lineinfile:\n      line: \\\"{{ ansible_date_time.iso8601}}\\\"\n      path: \\\"{{ ansible_env.HOME}}/fake_shutdown\\\"\n      create: yes{% endraw %}\n    {{ sh_filter }}\n\"}"
                 * {
                 *     "strategy_type": "reboot",
                 *     "repeat_not_before": "PT1M",
                 *     "ansible_j2": "{% raw %}---\n- hosts: servers_for_self_healing_monitoring\n  tasks:\n  - name: reboot\n    lineinfile:\n      line: \"{{ ansible_date_time.iso8601}}\"\n      path: \"{{ ansible_env.HOME}}/fake_reboot\"\n      create: yes{% endraw %}\n    {{ sh_filter }}\n"
                 * }
                 *
                 * "{\"repeat_not_before\": \"PT1M\",\"ansible_j2_template\": \"reboot\",\"key_1_for_template\": \"value_1_for_template\"}"
                 * {
                 *     "strategy_type": "ansible",
                 *     "repeat_not_before": "PT1M",
                 *     "ansible_j2_template": "reboot",
                 *     "key_1_for_template": "value_1_for_template"
                 * }
                 *
                 */

                if (!strategyConfigJson.has("strategy_type")) {
                    log.debug("Adding strategy_type to strategy configuration");
                    strategyConfigJson.addProperty("strategy_type", "ansible");
                } else if (!strategyConfigJson.get("strategy_type").getAsString().equals("ansible")) {
                    log.debug("Skipping strategy configuration with incorrect strategy_type {}", strategyConfigJson.get("strategy_type").getAsString());
                    continue;
                }

                // create strategy with the strategyConfigurationJson
                strategy_id = deployment.getDeploymentId() + "-" + monitoringRule.getName() ;
                Strategy strategy = new Strategy();
                strategy.setCode(strategy_id);
                strategy.setDeployment(deployment);
                strategy.setDescription("Strategy for deployment " + deployment.getDeploymentId() + " and monitoring rule " + monitoringRule.getName());
                strategy.setConfiguration(strategyConfigJson.toString());

                try {
                    strategyResource.createStrategy(strategy);
                } catch (URISyntaxException e) {
                    throw new RuntimeException(e);
                }
            }

            // find the strategy with the strategy_id
            Strategy strategy = strategyRepository.findByCode(strategy_id);

            // create event type with the strategy_id
            eventType = new EventType();
            eventType.setCode(eventType_Id);
            eventType.setStrategy(strategy);
            eventType.setDeployment(deployment);
            eventType.setDescription("Event type for deployment " + deployment.getDeploymentId() + " and monitoring rule " + monitoringRule.getName());

            // save the event type
            try {
                eventTypeResource.createEventType(eventType);
            } catch (URISyntaxException e) {
                throw new RuntimeException(e);
            }
        }
    }

    public void unBound(Long id) {
        log.info("Unconfiguring deployment {}", id);

        // find the event types for the deployment
        for (EventType eventType : eventTypeRepository.findAllByDeploymentId(id)) {
            // unbound the event types from self-healing messages
            for (SelfHealingMessage selfHealingMessage : selfHealingMessageRepository.findAllByEventType(eventType)) {
                log.info("Unbound event type {} from self healing message {}", eventType.getCode(), selfHealingMessage.getTimestamp());
                selfHealingMessage.setEventType(null);
                selfHealingMessageRepository.save(selfHealingMessage);
            }
            log.info("Deleting event type {}", eventType.getCode());
            eventTypeResource.deleteEventType(eventType.getId());
        }

        // find the strategies for the deployment
        for (Strategy strategy : strategyRepository.findAllByDeploymentId(id)) {
            // we need to unbound strategies from strategy applications
            for (StrategyApplication strategyApplication : strategyApplicationRepository.findAllByStrategy(strategy)) {
                log.info("Unbound strategy {} from strategy application {}", strategy.getCode(), strategyApplication.getTimestamp());
                strategyApplication.setStrategy(null);
                strategyApplicationRepository.save(strategyApplication);
            }
            log.info("Deleting strategy {}", strategy.getCode());
            strategyResource.deleteStrategy(strategy.getId());
        }

        // unbound the deployment from strategy applications
        for (StrategyApplication strategyApplication : strategyApplicationRepository.findAllByDeploymentId(id)) {
            log.info("Unbound deployment {} from strategy application {}", id, strategyApplication.getTimestamp());
            strategyApplication.setDeployment(null);
            strategyApplicationRepository.save(strategyApplication);
        }
    }
}
