package com.piacere.selfhealing.backend.service;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.piacere.selfhealing.backend.config.IsrProperties;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.apache.http.auth.Credentials;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.nio.file.Path;
import java.util.HashMap;

@Service
public class IsrService {
    //PiacereIDEIntegration.KR06And07\src\eu\piacere\ide\toolIntegration\kr06And07\commandHandlers\v2
    private final Logger log = LoggerFactory.getLogger(IopService.class);

    private IsrProperties isrProperties;
    private final HashMap<String, String> projectUuids = new HashMap<String, String>();


    public IsrService(IsrProperties isrProperties) {
        this.isrProperties = isrProperties;
    }

    private CloseableHttpClient getHttpClientWithUsernameAndPassAuth() {
        CloseableHttpClient httpclient = null;
        try {
            CredentialsProvider credentialsPovider = new BasicCredentialsProvider();
            Credentials credentials = new UsernamePasswordCredentials(isrProperties.getUsername(), isrProperties.getPassword());
            //Credentials credentials = new UsernamePasswordCredentials("user", "PIACERE-services-21!");
            credentialsPovider.setCredentials(AuthScope.ANY, credentials);

            //Creating the HttpClientBuilder
            HttpClientBuilder clientbuilder = HttpClients.custom();
            //Setting the credentials
            clientbuilder = clientbuilder.setDefaultCredentialsProvider(credentialsPovider);

            //Building the CloseableHttpClient object
            httpclient = clientbuilder.build();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return httpclient;
    }

    public String getProjectUuid(String deploymentId) {
        if (projectUuids.containsKey(deploymentId)) {
            return projectUuids.get(deploymentId);
        }

        log.info("Getting project uuid for {}", deploymentId);
        deploymentId = deploymentId.replace(" ", "_");
        String url = isrProperties.getUrl() + "/iac-scan-runner/projects?creator_id=" + deploymentId;

        try (CloseableHttpClient httpClient = getHttpClientWithUsernameAndPassAuth();) {
            HttpPost httpPost = new HttpPost(url);
            try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
                if (response.getStatusLine().getStatusCode() == 200) {
                    String content =  EntityUtils.toString(response.getEntity());
                    String projectUuid = content.replace("\"", "");
                    projectUuids.put(deploymentId, projectUuid);
                    return projectUuid;
                } else {
                    throw new RuntimeException("Error getting project uuid for deployment id " + deploymentId + ". Status: " + response.getStatusLine().getStatusCode() + "\n\n" + response.getEntity().toString());
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public JsonObject getIacChecks(String deploymentId, Path resultBundlePath) {
        log.info("Getting IAC checks for {}", deploymentId);
        String projectUuid = getProjectUuid(deploymentId);

        JsonArray availableChecks = getAvailableChecks(projectUuid);
        // enable all checks
        for (JsonElement check : availableChecks) {
            // {"name":"opera-tosca-parser","description":"xOpera TOSCA parser can validate TOSCA YAML templates and CSARs","enabled":true,"configured":true,"target_entity_type":"IaC"}
            JsonObject checkObject = check.getAsJsonObject();
            // if not enabled, enable it
            if (!checkObject.get("enabled").getAsBoolean()) {
                enableCheck(projectUuid, checkObject.get("name").getAsString());
            }
        }

        String url = isrProperties.getUrl() + "/iac-scan-runner/projects/" + projectUuid + "/scan?scan_response_type=json";

        MultipartEntityBuilder builder = MultipartEntityBuilder.create();
        builder.addPart("iac", new FileBody(resultBundlePath.toFile(), ContentType.DEFAULT_BINARY));

        try (CloseableHttpClient httpClient = getHttpClientWithUsernameAndPassAuth();) {
            HttpPost httpPost = new HttpPost(url);
            httpPost.setEntity(builder.build());
            try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
                if (response.getStatusLine().getStatusCode() == 200) {
                    String content = EntityUtils.toString(response.getEntity());
                    return JsonParser.parseString(content).getAsJsonObject();
                } else {
                    throw new RuntimeException("Error getting project uuid for deployment id " + deploymentId + ". Status: " + response.getStatusLine().getStatusCode() + "\n\n" + response.getEntity().toString());
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private JsonArray getAvailableChecks(String projectUuid) {
        log.info("Getting available checks for {}", projectUuid);
        String url = isrProperties.getUrl() + "/iac-scan-runner/projects/" + projectUuid + "/checks";

        try (CloseableHttpClient httpClient = getHttpClientWithUsernameAndPassAuth();) {
            HttpGet httpGet = new HttpGet(url);
            try (CloseableHttpResponse response = httpClient.execute(httpGet)) {
                if (response.getStatusLine().getStatusCode() == 200) {
                    String content = EntityUtils.toString(response.getEntity());
                    return JsonParser.parseString(content).getAsJsonArray();
                } else {
                    throw new RuntimeException("Error getting project uuid for deployment id " + projectUuid + ". Status: " + response.getStatusLine().getStatusCode() + "\n\n" + response.getEntity().toString());
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private void enableCheck(String projectUuid, String id) {
        log.info("Enabling check {} for {}", id, projectUuid);
        String url = isrProperties.getUrl() + "/iac-scan-runner/projects/" + projectUuid + "/checks/" + id + "/enable";

        try (CloseableHttpClient httpClient = getHttpClientWithUsernameAndPassAuth();) {
            HttpPut httpPut = new HttpPut(url);
            try (CloseableHttpResponse response = httpClient.execute(httpPut)) {
                if (response.getStatusLine().getStatusCode() != 200) {
                    throw new RuntimeException("Error enabling check " + id + " for project " + projectUuid + ". Status: " + response.getStatusLine().getStatusCode() + "\n\n" + response.getEntity().toString());
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
