package com.piacere.selfhealing.backend.service;

import com.piacere.selfhealing.backend.config.ApplicationProperties;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.Comparator;

@Service
public class ObjectKeepService {
    private final Logger log = LoggerFactory.getLogger(ObjectKeepService.class);
    private final ApplicationProperties applicationProperties;
    private File tmpDir;

    public ObjectKeepService(ApplicationProperties applicationProperties) {
        this.applicationProperties = applicationProperties;
        tmpDir = null;

        // if applicationProperties has keepLastRequests > 0 then create a temporal folder to store the last requests
        if (applicationProperties.getWebhook() != null) {
            if (applicationProperties.getWebhook().getKeepLastRequests() > 0) {
                log.info("Creating temporal folder to store the last {} requests", applicationProperties.getWebhook().getKeepLastRequests());
                try {
                    tmpDir = Files.createTempDirectory("self-healing-").toFile();
                } catch (IOException e) {
                    throw new RuntimeException("Error creating temporal folder", e);
                }
            }
        }
    }

    public void store(Object object) {
        if (tmpDir == null) return;

        // if more than keepLastRequests files then delete the oldest ones
        File[] files = tmpDir.listFiles();

        // https://stackoverflow.com/q/203030
        Arrays.sort(files, new Comparator<File>(){
            public int compare(File f1, File f2)
            {
                return Long.valueOf(f1.lastModified()).compareTo(f2.lastModified());
            } });

        if (files != null && files.length >= applicationProperties.getWebhook().getKeepLastRequests()) {
            // sort the files by last modified
            for (int i = 0; i < files.length - applicationProperties.getWebhook().getKeepLastRequests(); i++) {
                // ignore errors
                try {
                    files[i].delete();
                    log.debug("File {} deleted", files[i].getAbsolutePath());
                } catch (Exception e) {
                    log.error("Error deleting file {}", files[i].getAbsolutePath(), e);
                }
            }
        }
        // store the object in a temporal file
        File tmpFile = null;
        try {
            tmpFile = File.createTempFile("request-", ".tmp", tmpDir);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        // write the object to the file
        try {
            Files.write(tmpFile.toPath(), object.toString().getBytes());
            log.debug("Object stored in {}", tmpFile.getAbsolutePath());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
