package com.piacere.selfhealing.backend.service.impl;

import java.io.*;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.Duration;
import java.time.Instant;
import java.util.*;

import com.google.gson.*;
import com.google.gson.stream.JsonWriter;
import com.hubspot.jinjava.Jinjava;
import com.piacere.selfhealing.backend.config.ApplicationProperties;
import com.piacere.selfhealing.backend.domain.*;
import com.piacere.selfhealing.backend.repository.DeploymentRepository;
import com.piacere.selfhealing.backend.service.*;
import com.piacere.selfhealing.backend.web.rest.StrategyApplicationResource;
import eu.piacere.doml.doml.commons.DOMLModel;
import eu.piacere.doml.doml.infrastructure.ComputingGroup;
import eu.piacere.doml.doml.infrastructure.AutoScalingGroup;
import eu.piacere.doml.doml.infrastructure.ComputingNode;
import eu.piacere.doml.doml.infrastructure.InfrastructureLayer;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.piacere.selfhealing.backend.domain.enumeration.Status;
import com.piacere.selfhealing.backend.repository.EventTypeRepository;
import com.piacere.selfhealing.backend.repository.SelfHealingMessageRepository;
import com.piacere.selfhealing.backend.web.rest.errors.BadRequestAlertException;

@Service
@Transactional
public class SelfHealingMessageServiceImpl implements SelfHealingMessageService {
	private final Logger log = LoggerFactory.getLogger(SelfHealingMessageServiceImpl.class);
    private static final String ENTITY_NAME = "shBackendSelfHealingMessage";

	private final SelfHealingMessageRepository selfHealingMessageRepository;
	private final EventTypeRepository eventTypeRepository;
    private final ApplicationProperties applicationProperties;
    private final PrcService prcService;
    private final DeploymentRepository deploymentRepository;
    private final StrategyApplicationResource strategyApplicationResource;
    private final DOMLService domlService;
    private final BundleService bundleService;
    private final IopService iopService;
    private final DmcService dmcService;
    private final IcgService icgService;
    private final IsrService isrService;

    // dictionary to store the strategy not repeat before
    private final HashMap<String, Instant> strategyNotRepeatBeforeDictionary = new HashMap<String, Instant>();
    final Gson gsonPrettyPrinting = new GsonBuilder().setPrettyPrinting().create();

	public SelfHealingMessageServiceImpl(SelfHealingMessageRepository selfHealingMessageRepository,
			EventTypeRepository eventTypeRepository, ApplicationProperties applicationProperties,
            PrcService prcService, DeploymentRepository deploymentRepository,
            StrategyApplicationResource strategyApplicationResource, DOMLService domlService,
            BundleService bundleService, IopService iopService, DmcService dmcService,
            IcgService icgService, IsrService isrService) {
		super();
		this.selfHealingMessageRepository = selfHealingMessageRepository;
		this.eventTypeRepository = eventTypeRepository;
        this.applicationProperties = applicationProperties;
        this.prcService = prcService;
        this.deploymentRepository = deploymentRepository;
        this.strategyApplicationResource = strategyApplicationResource;
        this.domlService = domlService;
        this.bundleService = bundleService;
        this.iopService = iopService;
        this.dmcService = dmcService;
        this.icgService = icgService;
        this.isrService = isrService;
	}

	@Override
	public SelfHealingMessage create(SelfHealingMessage selfHealingMessage) {
		if (selfHealingMessage.getId() != null) {
            throw new BadRequestAlertException("A new selfHealingMessage cannot already have an ID", ENTITY_NAME, "idexists");
        }
        return selfHealingMessageRepository.save(selfHealingMessage);
	}

	@Override
	public SelfHealingMessage update(Long id, SelfHealingMessage selfHealingMessage) {
		if (selfHealingMessage.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, selfHealingMessage.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!selfHealingMessageRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        return selfHealingMessageRepository.save(selfHealingMessage);
	}

	@Override
	public Optional<SelfHealingMessage> patch(Long id, SelfHealingMessage selfHealingMessage)
			throws BadRequestAlertException {
		if (selfHealingMessage.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, selfHealingMessage.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!selfHealingMessageRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<SelfHealingMessage> result = selfHealingMessageRepository
            .findById(selfHealingMessage.getId())
            .map(existingSelfHealingMessage -> {
                if (selfHealingMessage.getOrigin() != null) {
                    existingSelfHealingMessage.setOrigin(selfHealingMessage.getOrigin());
                }
                if (selfHealingMessage.getDeploymentId() != null) {
                    existingSelfHealingMessage.setDeploymentId(selfHealingMessage.getDeploymentId());
                }
                if (selfHealingMessage.getTimestamp() != null) {
                    existingSelfHealingMessage.setTimestamp(selfHealingMessage.getTimestamp());
                }
                if (selfHealingMessage.getStatus() != null) {
                    existingSelfHealingMessage.setStatus(selfHealingMessage.getStatus());
                }
                if (selfHealingMessage.getError() != null) {
                    existingSelfHealingMessage.setError(selfHealingMessage.getError());
                }

                return existingSelfHealingMessage;
            })
            .map(selfHealingMessageRepository::save);
        return result;
	}

	@Override
	public List<SelfHealingMessage> findAll() {
		return selfHealingMessageRepository.findAll();
	}

	@Override
	public Optional<SelfHealingMessage> find(Long id) {
		return selfHealingMessageRepository.findById(id);
	}

	@Override
	public void delete(Long id) {
		selfHealingMessageRepository.deleteById(id);
	}

	@Override
	public void processMessage(SelfHealingMessage message) throws Exception {
        SelfHealingMessage selfHealingMessage = this.selfHealingMessageRepository.getById(message.getId());
        log.debug("--------Handling SelfHealingMessage: {}", selfHealingMessage);
        log.debug("--------Classify the event");
        log.debug("------------Find appropiate strategy for type " + selfHealingMessage.getEventType().getCode());

        Strategy strategy=null;
        try {
			strategy = this.eventTypeRepository.findByCode(selfHealingMessage.getEventType().getCode()).getStrategy();
			if (strategy == null) {
				throw new Exception("Strategy not found");
			}
		} catch (Exception e) {
			selfHealingMessage.setStatus(Status.ERROR);
			selfHealingMessage.setError(e.getMessage());
		}
        log.debug("----------------Strategy found: {}", strategy);
        log.debug("------------Propose self-healing strategy started");

        String strategyString = strategy.getConfiguration();
        JsonObject strategyConfiguration = new JsonObject();
        try {
            strategyConfiguration = JsonParser.parseString(strategyString).getAsJsonObject();
        } catch (Exception e) {
            log.debug("Error loading strategy configuration");
        }

        //https://www.geeksforgeeks.org/duration-parsecharsequence-method-in-java-with-examples/
        String repeatNotBefore = applicationProperties.getStrategy().getRepeatNotBefore();
        if (strategyConfiguration.has("repeat_not_before")) {
            repeatNotBefore = strategyConfiguration.get("repeat_not_before").getAsString();
        }

        Duration repeatNotBeforeDuration;
        try {
            repeatNotBeforeDuration = Duration.parse(repeatNotBefore);
        } catch (Exception e) {
            log.error("Error parsing repeat_not_before");
            throw new Exception("Error parsing repeat_not_before", e);
        }

        JsonObject eventError = new JsonObject();
        String eventErrorString = selfHealingMessage.getError();
        try {
            eventError = JsonParser.parseString(eventErrorString).getAsJsonObject();
        } catch (Exception e) {
            log.error("Event error is not a valid json, defaulting to { \"labels\": [ \"deployment_id\": \"" + selfHealingMessage.getDeploymentId() + "\" ] }");
            JsonObject eventErrorLabelsDefault = new JsonObject();
            eventErrorLabelsDefault.addProperty("deployment_id", selfHealingMessage.getDeploymentId());
            eventError.add("labels", eventErrorLabelsDefault);
        }

        JsonObject eventErrorLabels = new JsonObject();
        try {
            eventErrorLabels = eventError.getAsJsonObject("labels");
        } catch (Exception e) {
            log.error("eventError does not have labels, defaulting to { \"labels\": [ \"deployment_id\": \"" + selfHealingMessage.getDeploymentId() + "\" ] }");
            JsonObject eventErrorLabelsDefault = new JsonObject();
            eventErrorLabelsDefault.addProperty("deployment_id", selfHealingMessage.getDeploymentId());
            eventError.add("labels", eventErrorLabelsDefault);
        }

        JsonElement deploymentIdElement = eventErrorLabels.get("deployment_id");
        if (deploymentIdElement == null) {
            log.error("Error labels does not have deployment_id, adding it");
            eventErrorLabels.addProperty("deployment_id", selfHealingMessage.getDeploymentId());
            deploymentIdElement = eventErrorLabels.get("deployment_id");
        }
        String deploymentId = deploymentIdElement.getAsString();

        // ensure that event_type is in the labels
        JsonElement eventCodeElement = eventErrorLabels.get("event_type");
        if (eventCodeElement == null) {
            log.error("Error labels does not have event_type, adding it");
            eventErrorLabels.addProperty("event_type", selfHealingMessage.getEventType().getCode());
        }

        // if doml_element_name is in the labels with empty value then remove it
        JsonElement domlElementNameElement = eventErrorLabels.get("doml_element_name");
        if (domlElementNameElement != null) {
            String domlElementName = domlElementNameElement.getAsString();
            if (domlElementName.equals("")) {
                log.error("Error labels has doml_element_name with empty value, removing it");
                eventErrorLabels.remove("doml_element_name");
            }
        }

        // if host is in the labels with empty value then remove it
        JsonElement hostElement = eventErrorLabels.get("host");
        if (hostElement != null) {
            String host = hostElement.getAsString();
            if (host.equals("")) {
                log.error("Error labels has host with empty value, removing it");
                eventErrorLabels.remove("host");
            }
            if (host.equals("no value")) {
                log.error("Error labels has host with empty value, removing it");
                eventErrorLabels.remove("host");
            }
        }

        String strategyKey = strategy.getCode() + "_" + deploymentId;
        try {
            String domlElementName = eventErrorLabels.getAsJsonPrimitive("doml_element_name").getAsString();
            strategyKey = strategyKey + "_" + domlElementName;
        } catch (Exception e) {}

        try {
            String host = eventErrorLabels.getAsJsonPrimitive("host").getAsString();
            strategyKey = strategyKey + "_" + host;
        } catch (Exception e) {}

        Instant repeatNotBeforeTimestamp = strategyNotRepeatBeforeDictionary.get(strategyKey);

        if (repeatNotBeforeTimestamp != null) {
            Instant now = Instant.now();
            if (repeatNotBeforeTimestamp.isAfter(now)) {
                log.debug("------------Strategy not applied because of repeat_not_before");
                selfHealingMessage.setStatus(Status.IGNORED);
                this.selfHealingMessageRepository.saveAndFlush(selfHealingMessage);
                return;
            } else {
                strategyNotRepeatBeforeDictionary.remove(strategyKey);
            }
        }

        String strategyType = "notify";
        if (strategyConfiguration.has("strategy_type")) {
            strategyType = strategyConfiguration.get("strategy_type").getAsString();
        }

        // save into iec database

        switch (strategyType){
            case "notify":
                log.debug("------------Strategy notify");
                strategyNotify(deploymentId, strategy, eventErrorLabels, strategyConfiguration);
                break;
            case "redeploy":
                log.debug("------------Strategy redeploy");
                strategyRedeploy(deploymentId, strategy, eventErrorLabels, strategyConfiguration);
                break;
            case "ansible":
                log.debug("------------Strategy ansible");
                strategyAnsible(deploymentId, strategy, eventErrorLabels, strategyConfiguration);
                break;
            case "v_scale":
                log.debug("------------Strategy vertical_scale");
                strategyVerticalScale(deploymentId, strategy, eventErrorLabels, strategyConfiguration);
                break;
            case "h_scale":
                log.debug("------------Strategy horizontal_scale");
                strategyHorizontalScale(deploymentId, strategy, eventErrorLabels, strategyConfiguration);
                break;
            case "reoptimize":
                log.debug("------------Strategy reoptimize");
                strategyReoptimize(deploymentId, strategy, eventErrorLabels, strategyConfiguration);
                break;
            default:
                log.debug("------------Strategy {} not found", strategyType);
                throw new Exception("Strategy not found");
        }

        selfHealingMessage.setStatus(Status.PROCESSED);
        this.selfHealingMessageRepository.saveAndFlush(selfHealingMessage);

        Instant eventTimeStamp = Instant.now();

        repeatNotBeforeTimestamp = eventTimeStamp.plus(repeatNotBeforeDuration);
        strategyNotRepeatBeforeDictionary.put(strategyKey, repeatNotBeforeTimestamp);

        //clean dictionary
        Instant now = Instant.now();
        Set<String> keys = strategyNotRepeatBeforeDictionary.keySet();
        Iterator<String> iterator = keys.iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            Instant value = strategyNotRepeatBeforeDictionary.get(key);
            if (value.isBefore(now)) {
                iterator.remove();
            }
        }
	}

    private void strategyReoptimize(String deploymentId, Strategy strategy, JsonObject eventErrorLabels, JsonObject strategyConfiguration) {
        Deployment deployment = deploymentRepository.getByDeploymentId(deploymentId);
        String eventCode = eventErrorLabels.get("event_type").getAsString();

        Path deploymentUnpackTmpDir = bundleService.unpackBundle(deployment);
        Path resultTmpDir = bundleService.createTmpDir();

        // copy ecore doml.ecore
        Path domlEcorePath = deploymentUnpackTmpDir.resolve("doml.ecore");
        // if domlEcorePath does not exist error
        if (!domlEcorePath.toFile().exists()) {
            log.error("Error doml.ecore does not exist");
            throw new RuntimeException("doml.ecore does not exist");
        }

        Path resultDomlEcorePath = resultTmpDir.resolve("doml.ecore");
        log.info("Writing doml ecore to {}", resultDomlEcorePath);
        try {
            Files.copy(domlEcorePath, resultDomlEcorePath);
        } catch (IOException e) {
            log.error("Error writing doml ecore to tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        // copy *.domlx file from deploymentUnpackTmpDir to resultTmpDir/base.domlx
        // we will try to find doml file in the tmp directory, if no we will try to find domlx file and convert it to doml
        Path domlxPath = null;
        try {
            domlxPath = Files.list(deploymentUnpackTmpDir).filter(path -> path.toString().endsWith(".domlx")).findFirst().orElseThrow(() -> new RuntimeException("doml file not found"));
        } catch (Exception e) {
            log.error("Error finding domlx file");
            throw new RuntimeException("domlx file not found");
        }

        // copy domlx file to resultTmpDir/base.domlx
        Path baseDomlxPath = resultTmpDir.resolve("base.domlx");
        log.info("Writing base domlx to {}", baseDomlxPath);
        try {
            Files.copy(domlxPath, baseDomlxPath);
        } catch (IOException e) {
            log.error("Error writing base domlx to tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        Path domlPath = null;
        try {
            domlPath = Files.list(deploymentUnpackTmpDir).filter(path -> path.toString().endsWith(".doml")).findFirst().orElseThrow(() -> new RuntimeException("doml file not found"));
        } catch (Exception e) {
        }

        if (domlPath == null) {
            // load doml and save it to tmp directory
            DOMLModel doml = domlService.getDomlxModel(baseDomlxPath);
            domlPath = resultTmpDir.resolve("base.doml");
            domlService.saveDomlModel(domlPath, doml);
        }

        // copy
        Path baseDomlPath = resultTmpDir.resolve("base.doml");
        log.info("Writing base doml to {}", baseDomlPath);
        try {
            Files.write(baseDomlPath, Files.readAllBytes(domlPath));
        } catch (IOException e) {
            log.error("Error writing base doml to tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        String iopResponse = iopService.optimize(domlPath);

        // create optimizedDomlPath file in tmp directory
        Path optimizedDomlPath = resultTmpDir.resolve("optimized.doml");
        log.info("Writing optimized doml to {}", optimizedDomlPath);

        try {
            Files.write(optimizedDomlPath, iopResponse.getBytes(StandardCharsets.UTF_8));
        } catch (IOException e) {
            log.error("Error writing optimized doml to tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        DOMLModel doml = domlService.getDomlModel(optimizedDomlPath);

        Path optimizedDomlxPath = resultTmpDir.resolve("optimized.domlx");
        log.info("Writing optimized domlx to {}", optimizedDomlxPath);
        domlService.saveDomlxModel(optimizedDomlxPath, doml);

        validateDoml(optimizedDomlxPath, resultTmpDir);

        verifyCsp(optimizedDomlxPath, resultTmpDir);

        // copy deploymentUnpackTmpDir/asset into resultTmpDir/asset
        Path assetsPath = resultTmpDir.resolve("asset");
        log.info("Writing assets to {}", assetsPath);
        try {
            org.apache.commons.io.FileUtils.copyDirectory(deploymentUnpackTmpDir.resolve("asset").toFile(), assetsPath.toFile());
        } catch (IOException e) {
            log.error("Error writing assets to tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        log.info("Deleting the optimized domlx file as it cannot be used for the ICG it will used the base.domlx");
        // delete resultTmpDir/optimized.domlx optimizedDomlxPath
        try {
            Files.delete(optimizedDomlxPath);
        } catch (IOException e) {
            log.error("Error deleting optimized domlx");
            e.printStackTrace();
            throw new RuntimeException(e);
        }

        // generate icg and save to iac.zip
        Path tempDir = bundleService.createTmpDir();
        Path bundlePath = tempDir.resolve("bundle.zip");
        bundleService.packBundle(resultTmpDir, bundlePath);

        Path resultBundlePath = resultTmpDir.resolve("iac.zip");
        try {
            icgService.generateIac(bundlePath, resultBundlePath);
        } catch (Exception e) {
            log.error("Error generating iac");
        }

        // check iac scan runner
        JsonObject iacChecks = isrService.getIacChecks(deploymentId, resultBundlePath);
        Path iacScanResultPath = resultTmpDir.resolve("iac_scan_result.json");
        log.info("Writing iac scan result to {}", iacScanResultPath);
        // pretty print json indent 4

        try (final PrintWriter pWriter = new PrintWriter(iacScanResultPath.toFile())) {
            final JsonWriter jWriter = gsonPrettyPrinting.newJsonWriter(pWriter);
            jWriter.setIndent("\t");
            gsonPrettyPrinting.toJson(iacChecks, jWriter);
            pWriter.println();
        } catch (final IOException e) {
        }

        StrategyApplication strategyApplication = new StrategyApplication();
        strategyApplication.setStrategy(strategy);
        strategyApplication.setDeployment(deployment);
        strategyApplication.setTimestamp(Instant.now());
        strategyApplication.setSummary("Reoptimize executed with strategy (" + strategy.getCode() + ") for the deployment (" + deploymentId + ") and the event (" + eventCode + ")");
        bundleService.packBundle(strategyApplication, resultTmpDir);

        try {
            strategyApplicationResource.createStrategyApplication(strategyApplication);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private void verifyCsp(Path optimizedDomlxPath, Path resultTmpDir) {
        String dmcResponse = dmcService.verifyCsp(optimizedDomlxPath);

        Path dmcResponsePath = resultTmpDir.resolve("dmc_verify_csp.html");
        log.info("Writing dmc response to {}", dmcResponsePath);

        try {
            Files.write(dmcResponsePath, dmcResponse.getBytes(StandardCharsets.UTF_8));
        } catch (IOException e) {
            log.error("Error writing dmc verify_csp response to tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    private void validateDoml(Path optimizedDomlxPath, Path resultTmpDir) {
        String dmcResponse = dmcService.validate(optimizedDomlxPath);

        Path dmcResponsePath = resultTmpDir.resolve("dmc_validation.html");
        log.info("Writing dmc response to {}", dmcResponsePath);

        try {
            Files.write(dmcResponsePath, dmcResponse.getBytes(StandardCharsets.UTF_8));
        } catch (IOException e) {
            log.error("Error writing dmc validate response to tmp directory");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    private void strategyHorizontalScale(String deploymentId, Strategy strategy, JsonObject eventErrorLabels, JsonObject strategyConfiguration) {
        Deployment deployment = deploymentRepository.getByDeploymentId(deploymentId);
        String eventCode = eventErrorLabels.get("event_type").getAsString();

        // We constrain to doml_element_name
        JsonElement domlElementNameObject = eventErrorLabels.get("doml_element_name");
        if (domlElementNameObject == null) {
            log.error("Error labels does not have doml_element_name, vertical scale cannot be applied");
            return;
        }
        String domlElementName = domlElementNameObject.getAsString();

        DOMLModel doml = domlService.getDomlxModel(deployment);
        if (doml == null) {
            log.error("Error while loading domlx into ecore resource");
            throw new RuntimeException("Error while loading domlx into ecore resource");
        }

        //get the infrastructure model
        InfrastructureLayer infrastructureLayer = doml.getInfrastructure();
        if (infrastructureLayer == null) {
            log.error("Error while loading domlx into ecore resource");
            throw new RuntimeException("Error while loading domlx into ecore resource");
        }

        // iterate doml elements and find the one with the same name as doml_element_name
        for (ComputingGroup computingGroup: infrastructureLayer.getGroups()) {
            if (computingGroup.getName().equals(domlElementName)) {
                log.debug("computingGroup.getName() {}", computingGroup.getName());
                // if computingGroup is AutoScalingGroup then add 1 to max_size
                if (computingGroup instanceof AutoScalingGroup) {
                    log.debug("------------Strategy horizontal_scale auto_scaling_group");
                    AutoScalingGroup autoScalingGroup = (AutoScalingGroup) computingGroup;
                    autoScalingGroup.setMax(autoScalingGroup.getMax() + 1);
                }
            }
        }

        StrategyApplication strategyApplication = new StrategyApplication();
        strategyApplication.setStrategy(strategy);
        strategyApplication.setDeployment(deployment);
        strategyApplication.setTimestamp(Instant.now());
        strategyApplication.setSummary("Horizontal scale executed with strategy (" + strategy.getCode() + ") for the deployment (" + deploymentId + ") and the event (" + eventCode + ")");
        // save the doml model to the strategy application
        domlService.saveDomlModel(strategyApplication, doml);

        try {
            strategyApplicationResource.createStrategyApplication(strategyApplication);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private void strategyVerticalScale(String deploymentId, Strategy strategy, JsonObject eventErrorLabels, JsonObject strategyConfiguration) {
        Deployment deployment = deploymentRepository.getByDeploymentId(deploymentId);
        String eventCode = eventErrorLabels.get("event_type").getAsString();

        // check if dimension is in the eventerrorlabels
        JsonElement dimensionObject = eventErrorLabels.get("dimension");
        if (dimensionObject == null) {
            log.error("Error labels does not have dimension, horizontal scale cannot be applied");
            return;
        }
        String dimension = dimensionObject.getAsString();

        // We constrain to doml_element_name
        JsonElement domlElementNameObject = eventErrorLabels.get("doml_element_name");
        if (domlElementNameObject == null) {
            log.error("Error labels does not have doml_element_name, vertical scale cannot be applied");
            return;
        }
        String domlElementName = domlElementNameObject.getAsString();

        DOMLModel doml = domlService.getDomlxModel(deployment);
        if (doml == null) {
            log.error("Error while loading domlx into ecore resource");
            throw new RuntimeException("Error while loading domlx into ecore resource");
        }

        //get the infrastructure model
        InfrastructureLayer infrastructureLayer = doml.getInfrastructure();
        if (infrastructureLayer == null) {
            log.error("Error while loading domlx into ecore resource");
            throw new RuntimeException("Error while loading domlx into ecore resource");
        }

        // iterate doml elements and find the one with the same name as doml_element_name
        for (ComputingNode computingNode: infrastructureLayer.getNodes()) {
            if (computingNode.getName().equals(domlElementName)) {
                switch (dimension) {
                    case "cpu":
                        log.debug("------------Strategy vertical_scale cpu");
                        if (computingNode.getCpu_count() == null) {
                            log.error("Error computingNode {} does not have cpuCount", computingNode.getName());
                            return;
                        }
                        Integer cpuCount = computingNode.getCpu_count();
                        Integer newCpuCount = cpuCount + 1;
                        computingNode.setCpu_count(newCpuCount);
                        break;
                    case "memory":
                        log.debug("------------Strategy vertical_scale mem");
                        if (computingNode.getMemory_mb() == null) {
                            log.error("Error computingNode {} does not have mem_mb", computingNode.getName());
                            return;
                        }
                        Float memoryMb = computingNode.getMemory_mb();
                        Float newMemoryMb = memoryMb + 1024;
                        computingNode.setMemory_mb(newMemoryMb);
                        break;
                    case "storage":
                        log.debug("------------Strategy vertical_scale sto");
                        if (computingNode.getStorage() == null) {
                            log.error("Error computingNode {} does not have storage_gb", computingNode.getName());
                            return;
                        }
                        Float storageGb = computingNode.getStorage();
                        Float newStorageGb = storageGb + 10;
                        computingNode.setStorage(newStorageGb);
                        break;
                    default:
                        log.debug("------------Strategy vertical_scale {} not found", eventCode);
                        throw new RuntimeException("Strategy not found");
                }
            }
        }

        StrategyApplication strategyApplication = new StrategyApplication();
        strategyApplication.setStrategy(strategy);
        strategyApplication.setDeployment(deployment);
        strategyApplication.setTimestamp(Instant.now());
        strategyApplication.setSummary("Vertical scale executed with strategy (" + strategy.getCode() + ") for the deployment (" + deploymentId + ") and the event (" + eventCode + ")");
        // save the doml model to the strategy application
        domlService.saveDomlModel(strategyApplication, doml);

        try {
            strategyApplicationResource.createStrategyApplication(strategyApplication);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private void strategyRedeploy(String deploymentId, Strategy strategy, JsonObject eventErrorLabels, JsonObject strategyConfiguration) {
        Deployment deployment = deploymentRepository.getByDeploymentId(deploymentId);
        String eventCode = eventErrorLabels.get("event_type").getAsString();

        prcService.executeRedeploy(deployment);
        StrategyApplication strategyApplication = new StrategyApplication();
        strategyApplication.setStrategy(strategy);
        strategyApplication.setDeployment(deployment);
        strategyApplication.setTimestamp(Instant.now());
        strategyApplication.setSummary("Redeploy executed with strategy (" + strategy.getCode() + ") for the deployment (" + deploymentId + ") and the event (" + eventCode + ")");

        try {
            strategyApplicationResource.createStrategyApplication(strategyApplication);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private void strategyNotify(String deploymentId, Strategy strategy, JsonObject eventErrorLabels, JsonObject strategyConfiguration) {
        StrategyApplication strategyApplication = new StrategyApplication();
        String eventCode = eventErrorLabels.get("event_type").getAsString();

        strategyApplication.setStrategy(strategy);
        Deployment deployment = deploymentRepository.getByDeploymentId(deploymentId);
        strategyApplication.setDeployment(deployment);
        strategyApplication.setTimestamp(Instant.now());
        strategyApplication.setSummary("Notification registered with strategy (" + strategy.getCode() + ") for the deployment (" + deploymentId + ") and the event (" + eventCode + ")");

        try {
            strategyApplicationResource.createStrategyApplication(strategyApplication);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private void strategyAnsible(String deploymentId, Strategy strategy, JsonObject eventErrorLabels, JsonObject strategyConfiguration) {
        String eventCode = eventErrorLabels.get("event_type").getAsString();

        // build when condition with doml_element_name and host
        String whenCondition = "";
        // when: doml_element_name is defined and doml_element_name == 'mysql_vm' and ansible_hostname is defined and ansible_hostname == 'abb3ef8c3cd0'
        boolean add_and = false;
        if (eventErrorLabels.has("doml_element_name")) {
            if (add_and) whenCondition = whenCondition + " and ";
            else {
                whenCondition = whenCondition + "when: ";
                add_and = true;
            }
            //
            String domlElementName = eventErrorLabels.get("doml_element_name").getAsString();
            whenCondition = whenCondition + "doml_element_name is defined and ";
            whenCondition = whenCondition + "doml_element_name == '" + domlElementName + "'";
        }
        if (eventErrorLabels.has("host")) {
            if (add_and) whenCondition = whenCondition + " and ";
            else {
                whenCondition = whenCondition + "when: ";
                add_and = true;
            }
            String host = eventErrorLabels.get("host").getAsString();
            whenCondition = whenCondition + "ansible_hostname is defined and ";
            whenCondition = whenCondition + "ansible_hostname == '" + host + "'";
        }

        Jinjava jinjava = new Jinjava();

        Map<String, Object> context = new HashMap<String, Object>();
        context.put("sh_filter", whenCondition);

        /**
         * {"strategy_type": "ansible", "repeat_not_before": "PT1M", "ansible_j2_template": "shutdown" }
         * {"strategy_type": "ansible", "repeat_not_before": "PT1M", "ansible_j2_template": "run_playbook_at_assets","assets_relative_path": "sh/strategy1.yml"}
         * {"strategy_type": "ansible", "repeat_not_before": "PT1M", "ansible_j2": "{% raw %}---\n- hosts: servers_for_self_healing_monitoring\n  tasks:\n  - name: reboot\n    lineinfile:\n      line: \\\"{{ ansible_date_time.iso8601}}\\\"\n      path: \\\"{{ ansible_env.HOME}}/fake_shutdown\\\"\n      create: yes{% endraw %}\n    {{ sh_filter }}\n"}
         */

        String ansible_j2 = null;

        try {
            ansible_j2 = strategyConfiguration.get("ansible_j2").getAsString();
        } catch (Exception e) {
            log.debug("No ansible_j2 found in strategy configuration");
        }

        // if has ansible_j2_template
        if (strategyConfiguration.has("ansible_j2_template")) {
            String ansible_j2_template = strategyConfiguration.get("ansible_j2_template").getAsString();
            String ansible_j2_template_path = "ansible_j2_templates/" + ansible_j2_template + ".yaml.j2";

            try {
                ansible_j2 = IOUtils.toString(this.getClass().getClassLoader().getResourceAsStream(ansible_j2_template_path), StandardCharsets.UTF_8);
            } catch (Exception e) {
                log.error("Error while loading template {}", ansible_j2_template_path, e);
                return;
            }

            // extend context with all elements of cond
            for (Map.Entry<String, JsonElement> entry : strategyConfiguration.entrySet()) {
                context.put(entry.getKey(), entry.getValue().getAsString());
            }
        }

        // if ansible_j2 is null error
        if (ansible_j2 == null) {
            log.error("no ansible_j2 or ansible_j2_template found in strategy configuration");
            return;
        }

        String strategyPlaybook = null;
        try {
            strategyPlaybook = jinjava.render(ansible_j2, context);
        } catch (Exception e) {
            log.error("Error while rendering cond.json.j2", e);
            return;
        }

        prcService.executeAnsiblePlaybook(deploymentId, strategyPlaybook);

        StrategyApplication strategyApplication = new StrategyApplication();
        strategyApplication.setStrategy(strategy);
        Deployment deployment = deploymentRepository.getByDeploymentId(deploymentId);
        strategyApplication.setDeployment(deployment);
        strategyApplication.setTimestamp(Instant.now());
        strategyApplication.setSummary("Ansible playbook executed with strategy (" + strategy.getCode() + ") for the deployment (" + deploymentId + ") and the event (" + eventCode + ")");

        try {
            strategyApplicationResource.createStrategyApplication(strategyApplication);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }
}
