package com.piacere.selfhealing.backend.web.rest;

import com.piacere.selfhealing.backend.domain.Deployment;
import com.piacere.selfhealing.backend.repository.DeploymentRepository;
import com.piacere.selfhealing.backend.service.DeploymentService;
import com.piacere.selfhealing.backend.web.rest.errors.BadRequestAlertException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.piacere.selfhealing.backend.domain.Deployment}.
 */
@RestController
@RequestMapping("/api")
@Transactional
public class DeploymentResource {

    private final Logger log = LoggerFactory.getLogger(DeploymentResource.class);

    private static final String ENTITY_NAME = "shBackendDeployment";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final DeploymentRepository deploymentRepository;

    private final DeploymentService deploymentService;

    public DeploymentResource(DeploymentRepository deploymentRepository, DeploymentService deploymentService) {
        this.deploymentRepository = deploymentRepository;
        this.deploymentService = deploymentService;
    }

    /**
     * {@code POST  /deployments} : Create a new deployment.
     *
     * @param deployment the deployment to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new deployment, or with status {@code 400 (Bad Request)} if the deployment has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("/deployments")
    public ResponseEntity<Deployment> createDeployment(@Valid @RequestBody Deployment deployment) throws URISyntaxException {
        log.debug("REST request to save Deployment : {}", deployment);
        if (deployment.getId() != null) {
            throw new BadRequestAlertException("A new deployment cannot already have an ID", ENTITY_NAME, "idexists");
        }
        Deployment result = deploymentRepository.save(deployment);
        deploymentService.configure(result);
        return ResponseEntity
            .created(new URI("/api/deployments/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * {@code PUT  /deployments/:id} : Updates an existing deployment.
     *
     * @param id the id of the deployment to save.
     * @param deployment the deployment to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated deployment,
     * or with status {@code 400 (Bad Request)} if the deployment is not valid,
     * or with status {@code 500 (Internal Server Error)} if the deployment couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/deployments/{id}")
    public ResponseEntity<Deployment> updateDeployment(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody Deployment deployment
    ) throws URISyntaxException {
        log.debug("REST request to update Deployment : {}, {}", id, deployment);
        if (deployment.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, deployment.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!deploymentRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Deployment result = deploymentRepository.save(deployment);
        return ResponseEntity
            .ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, deployment.getId().toString()))
            .body(result);
    }

    /**
     * {@code PATCH  /deployments/:id} : Partial updates given fields of an existing deployment, field will ignore if it is null
     *
     * @param id the id of the deployment to save.
     * @param deployment the deployment to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated deployment,
     * or with status {@code 400 (Bad Request)} if the deployment is not valid,
     * or with status {@code 404 (Not Found)} if the deployment is not found,
     * or with status {@code 500 (Internal Server Error)} if the deployment couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/deployments/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<Deployment> partialUpdateDeployment(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody Deployment deployment
    ) throws URISyntaxException {
        log.debug("REST request to partial update Deployment partially : {}, {}", id, deployment);
        if (deployment.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, deployment.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!deploymentRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<Deployment> result = deploymentRepository
            .findById(deployment.getId())
            .map(existingDeployment -> {
                if (deployment.getDeploymentId() != null) {
                    existingDeployment.setDeploymentId(deployment.getDeploymentId());
                }
                if (deployment.getBundle() != null) {
                    existingDeployment.setBundle(deployment.getBundle());
                }
                if (deployment.getBundleContentType() != null) {
                    existingDeployment.setBundleContentType(deployment.getBundleContentType());
                }

                return existingDeployment;
            })
            .map(deploymentRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, deployment.getId().toString())
        );
    }

    /**
     * {@code GET  /deployments} : get all the deployments.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of deployments in body.
     */
    @GetMapping("/deployments")
    public List<Deployment> getAllDeployments() {
        log.debug("REST request to get all Deployments");
        return deploymentRepository.findAll();
    }

    /**
     * {@code GET  /deployments/:id} : get the "id" deployment.
     *
     * @param id the id of the deployment to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the deployment, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/deployments/{id}")
    public ResponseEntity<Deployment> getDeployment(@PathVariable Long id) {
        log.debug("REST request to get Deployment : {}", id);
        Optional<Deployment> deployment = deploymentRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(deployment);
    }

    /**
     * {@code DELETE  /deployments/:id} : delete the "id" deployment.
     *
     * @param id the id of the deployment to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/deployments/{id}")
    public ResponseEntity<Void> deleteDeployment(@PathVariable Long id) {
        log.debug("REST request to delete Deployment : {}", id);
        deploymentService.unBound(id);
        deploymentRepository.deleteById(id);
        return ResponseEntity
            .noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
