package com.piacere.selfhealing.backend.web.rest;

import com.piacere.selfhealing.backend.domain.SelfHealingMessage;
import com.piacere.selfhealing.backend.domain.enumeration.Status;
import com.piacere.selfhealing.backend.producer.SelfHealingMessageProducer;
import com.piacere.selfhealing.backend.repository.SelfHealingMessageRepository;
import com.piacere.selfhealing.backend.web.rest.errors.BadRequestAlertException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.piacere.selfhealing.backend.domain.SelfHealingMessage}.
 */
@RestController
@RequestMapping("/api")
// @Transactional it produces race conditions with the self healing message producer
public class SelfHealingMessageResource {

    private final Logger log = LoggerFactory.getLogger(SelfHealingMessageResource.class);
    private final SelfHealingMessageProducer selfHealingMessageProducer;

    private static final String ENTITY_NAME = "shBackendSelfHealingMessage";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SelfHealingMessageRepository selfHealingMessageRepository;

    public SelfHealingMessageResource(SelfHealingMessageRepository selfHealingMessageRepository, SelfHealingMessageProducer selfHealingMessageProducer) {
        this.selfHealingMessageRepository = selfHealingMessageRepository;
        this.selfHealingMessageProducer = selfHealingMessageProducer;
    }

    /**
     * {@code POST  /self-healing-messages} : Create a new selfHealingMessage.
     *
     * @param selfHealingMessage the selfHealingMessage to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new selfHealingMessage, or with status {@code 400 (Bad Request)} if the selfHealingMessage has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("/self-healing-messages")
    public ResponseEntity<SelfHealingMessage> createSelfHealingMessage(@Valid @RequestBody SelfHealingMessage selfHealingMessage)
        throws URISyntaxException {
        log.debug("REST request to save SelfHealingMessage : {}", selfHealingMessage);
        if (selfHealingMessage.getId() != null) {
            throw new BadRequestAlertException("A new selfHealingMessage cannot already have an ID", ENTITY_NAME, "idexists");
        }

        SelfHealingMessage result = selfHealingMessageRepository.saveAndFlush(selfHealingMessage);

        if (selfHealingMessage.getStatus()!= Status.ERROR) {
            selfHealingMessageProducer.notify(selfHealingMessage);
        }
        return ResponseEntity
            .created(new URI("/api/self-healing-messages/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * {@code PUT  /self-healing-messages/:id} : Updates an existing selfHealingMessage.
     *
     * @param id the id of the selfHealingMessage to save.
     * @param selfHealingMessage the selfHealingMessage to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated selfHealingMessage,
     * or with status {@code 400 (Bad Request)} if the selfHealingMessage is not valid,
     * or with status {@code 500 (Internal Server Error)} if the selfHealingMessage couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/self-healing-messages/{id}")
    public ResponseEntity<SelfHealingMessage> updateSelfHealingMessage(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody SelfHealingMessage selfHealingMessage
    ) throws URISyntaxException {
        log.debug("REST request to update SelfHealingMessage : {}, {}", id, selfHealingMessage);
        if (selfHealingMessage.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, selfHealingMessage.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!selfHealingMessageRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        SelfHealingMessage result = selfHealingMessageRepository.save(selfHealingMessage);
        return ResponseEntity
            .ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, selfHealingMessage.getId().toString()))
            .body(result);
    }

    /**
     * {@code PATCH  /self-healing-messages/:id} : Partial updates given fields of an existing selfHealingMessage, field will ignore if it is null
     *
     * @param id the id of the selfHealingMessage to save.
     * @param selfHealingMessage the selfHealingMessage to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated selfHealingMessage,
     * or with status {@code 400 (Bad Request)} if the selfHealingMessage is not valid,
     * or with status {@code 404 (Not Found)} if the selfHealingMessage is not found,
     * or with status {@code 500 (Internal Server Error)} if the selfHealingMessage couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/self-healing-messages/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<SelfHealingMessage> partialUpdateSelfHealingMessage(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody SelfHealingMessage selfHealingMessage
    ) throws URISyntaxException {
        log.debug("REST request to partial update SelfHealingMessage partially : {}, {}", id, selfHealingMessage);
        if (selfHealingMessage.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, selfHealingMessage.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!selfHealingMessageRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<SelfHealingMessage> result = selfHealingMessageRepository
            .findById(selfHealingMessage.getId())
            .map(existingSelfHealingMessage -> {
                if (selfHealingMessage.getTimestamp() != null) {
                    existingSelfHealingMessage.setTimestamp(selfHealingMessage.getTimestamp());
                }
                if (selfHealingMessage.getOrigin() != null) {
                    existingSelfHealingMessage.setOrigin(selfHealingMessage.getOrigin());
                }
                if (selfHealingMessage.getDeploymentId() != null) {
                    existingSelfHealingMessage.setDeploymentId(selfHealingMessage.getDeploymentId());
                }
                if (selfHealingMessage.getStatus() != null) {
                    existingSelfHealingMessage.setStatus(selfHealingMessage.getStatus());
                }
                if (selfHealingMessage.getError() != null) {
                    existingSelfHealingMessage.setError(selfHealingMessage.getError());
                }

                return existingSelfHealingMessage;
            })
            .map(selfHealingMessageRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, selfHealingMessage.getId().toString())
        );
    }

    /**
     * {@code GET  /self-healing-messages} : get all the selfHealingMessages.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of selfHealingMessages in body.
     */
    @GetMapping("/self-healing-messages")
    public List<SelfHealingMessage> getAllSelfHealingMessages() {
        log.debug("REST request to get all SelfHealingMessages");
        return selfHealingMessageRepository.findAll();
    }

    /**
     * {@code GET  /self-healing-messages/:id} : get the "id" selfHealingMessage.
     *
     * @param id the id of the selfHealingMessage to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the selfHealingMessage, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/self-healing-messages/{id}")
    public ResponseEntity<SelfHealingMessage> getSelfHealingMessage(@PathVariable Long id) {
        log.debug("REST request to get SelfHealingMessage : {}", id);
        Optional<SelfHealingMessage> selfHealingMessage = selfHealingMessageRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(selfHealingMessage);
    }

    /**
     * {@code DELETE  /self-healing-messages/:id} : delete the "id" selfHealingMessage.
     *
     * @param id the id of the selfHealingMessage to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/self-healing-messages/{id}")
    public ResponseEntity<Void> deleteSelfHealingMessage(@PathVariable Long id) {
        log.debug("REST request to delete SelfHealingMessage : {}", id);
        selfHealingMessageRepository.deleteById(id);
        return ResponseEntity
            .noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
