package com.piacere.selfhealing.backend.web.rest;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.piacere.selfhealing.backend.domain.Deployment;
import com.piacere.selfhealing.backend.domain.EventType;
import com.piacere.selfhealing.backend.domain.SelfHealingMessage;
import com.piacere.selfhealing.backend.domain.enumeration.Status;
import com.piacere.selfhealing.backend.producer.SelfHealingMessageProducer;
import com.piacere.selfhealing.backend.repository.DeploymentRepository;
import com.piacere.selfhealing.backend.repository.EventTypeRepository;
import com.piacere.selfhealing.backend.repository.SelfHealingMessageRepository;
import com.piacere.selfhealing.backend.service.ObjectKeepService;
import com.piacere.selfhealing.backend.service.dto.DeploymentDTO;
import com.piacere.selfhealing.backend.service.dto.SelfHealingMessageDTO;
import com.piacere.selfhealing.backend.service.dto.SelfHealingResponseDTO;
import com.piacere.selfhealing.backend.service.mapper.SelfHealingMessageMapper;
import com.piacere.selfhealing.backend.service.mapper.SelfHealingResponseMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.yaml.snakeyaml.Yaml;

import javax.xml.bind.DatatypeConverter;
import java.net.URISyntaxException;
import java.time.Instant;
import java.util.LinkedHashMap;
import java.util.Map;

@RestController
@RequestMapping("/api/self-healing")
public class SelfHealingResource {

    private final Logger log = LoggerFactory.getLogger(SelfHealingResource.class);

    private final SelfHealingMessageProducer selfHealingMessageProducer;
    private final SelfHealingMessageRepository selfHealingMessageRepository;
    private final EventTypeRepository typeRepository;
    private final SelfHealingMessageMapper selfHealingMessageMapper;
    private final SelfHealingResponseMapper selfHealingResponseMapper;
    private final ObjectKeepService objectKeepService;
    private final DeploymentResource deploymentResource;
    private final DeploymentRepository deploymentRepository;
    private final SelfHealingMessageResource selfHealingMessageResource;

    public SelfHealingResource(SelfHealingMessageProducer selfHealingMessageProducer, SelfHealingMessageRepository selfHealingMessageRepository,
    		EventTypeRepository typeRepository, SelfHealingMessageMapper selfHealingMessageMapper,
    		SelfHealingResponseMapper selfHealingResponseMapper, ObjectKeepService objectKeepService, DeploymentResource deploymentResource, DeploymentRepository deploymentRepository, SelfHealingMessageResource selfHealingMessageResource) {
        this.selfHealingMessageProducer = selfHealingMessageProducer;
        this.selfHealingMessageRepository = selfHealingMessageRepository;
        this.typeRepository = typeRepository;
        this.selfHealingMessageMapper = selfHealingMessageMapper;
        this.selfHealingResponseMapper = selfHealingResponseMapper;
        this.objectKeepService = objectKeepService;
        this.deploymentResource = deploymentResource;
        this.deploymentRepository = deploymentRepository;
        this.selfHealingMessageResource = selfHealingMessageResource;
    }

    /**
     * Event notify to self healing component
     *
     * @param SelfHealingMessageDTO the self healing message to send
     * @return self healing response
     */
    @PostMapping("/notify")
    public SelfHealingResponseDTO notify(@RequestBody SelfHealingMessageDTO selfHealingMessageDto) {
        log.info("NOTIFY SELF-HEALING REST SERVICE START----------------------------------------------------------");
        log.info("SelfHealingMessage: {}", selfHealingMessageDto);
        SelfHealingMessage selfHealingMessage = selfHealingMessageMapper.toEntity(selfHealingMessageDto);
        selfHealingMessage.setTimestamp(Instant.now());
        selfHealingMessage.setStatus(Status.PENDING);
        EventType eventType = null;
        try {
			eventType = this.typeRepository.findByCode(selfHealingMessage.getEventType().getCode());
			if (eventType==null) {
				throw new Exception("Type not found");
			}
		} catch (Exception e) {
			selfHealingMessage.setStatus(Status.ERROR);
			selfHealingMessage.setError(e.getMessage());
		}
		selfHealingMessage.setEventType(eventType);

        try {
            selfHealingMessageResource.createSelfHealingMessage(selfHealingMessage);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }

        SelfHealingResponseDTO selfHealingResponseDTO = this.selfHealingResponseMapper.toDto(selfHealingMessage);
        log.info("Self-healing rest service response (ack): {}", selfHealingResponseDTO);
        log.info("NOTIFY SELF-HEALING REST SERVICE FINISHED------------------------------------------------------");
        return selfHealingResponseDTO;
    }

    /**
     * Event webhook to self healing component
     *
     * @param JsonObject the webhook payload received from grafana
     * @return self healing response
     */
    @PostMapping("/grafana-webhooks")
    public ResponseEntity webhook(@RequestBody LinkedHashMap alertMessage) {
        log.info("WEBHOOK SELF-HEALING REST SERVICE START----------------------------------------------------------");
        Gson gson = new Gson();
        JsonObject jsonObject = gson.toJsonTree(alertMessage).getAsJsonObject();
        String prettyJson = gson.toJson(jsonObject);
        objectKeepService.store(prettyJson);

        // if status is resolved we skip the alert
        String status = jsonObject.get("status").getAsString();
        if (status.equals("resolved")) {
            log.debug("Skipping alert with status resolved");
            return new ResponseEntity<>(HttpStatus.CREATED);
        }

        // we iterate over the alerts in the alertMessage
        JsonArray alerts = jsonObject.getAsJsonArray("alerts");
        if ( alerts != null ) {
            for (JsonElement alertElement : alerts) {
                JsonObject alert = alertElement.getAsJsonObject();
                String fingerprint = alert.getAsJsonObject().get("fingerprint").getAsString();

                JsonElement jsonElementEventType = alert.getAsJsonObject().get("labels").getAsJsonObject().get("event_type");
                if (jsonElementEventType == null) {
                    log.debug("Skipping alert with fingerprint {} because event_type is null", fingerprint);
                    log.debug("Alert details: {}", alert);
                    continue;
                }

                String eventTypeString = alert.getAsJsonObject().get("labels").getAsJsonObject().get("event_type").getAsString();
                EventType eventType = this.typeRepository.findByCode(eventTypeString);
                if (eventType == null) {
                    log.debug("Skipping alert with fingerprint {} because event_type {} is not found", fingerprint, eventTypeString);
                    log.debug("Alert details: {}", alert);
                    continue;
                }

                JsonObject inputsLabels = alert.getAsJsonObject().get("labels").getAsJsonObject();
                if (inputsLabels == null) {
                    log.debug("Skipping alert with fingerprint {} because inputs_labels is null", fingerprint);
                    log.debug("Alert details: {}", alert);
                    continue;
                }

                // create labels tuples array minus event_type
                JsonObject labels = new JsonObject();
                for (Map.Entry<String, JsonElement> entry : inputsLabels.entrySet()) {
                    if (!entry.getKey().equals("event_type") &&
                        !entry.getKey().equals("alertname") &&
                        !entry.getKey().equals("category") &&
                        !entry.getKey().equals("deployment_id") &&
                        !entry.getKey().equals("grafana_folder")) {
                        labels.add(entry.getKey(), entry.getValue());
                    }
                }

                JsonObject error = new JsonObject();
                error.add("labels", labels);

                JsonObject annotations = alert.getAsJsonObject().get("annotations").getAsJsonObject();
                JsonElement jsonElementAlertValue = annotations.get("alert_values");
                // check if alert_values is present if present process it
                if (jsonElementAlertValue == null) {
                    JsonElement deploymentIdElement = inputsLabels.get("deployment_id");
                    if (inputsLabels == null) {
                        log.debug("Skipping alert with fingerprint {} because deployment_id is null", fingerprint);
                        log.debug("Alert details: {}", alert);
                        continue;
                    }
                    labels.add("deployment_id", deploymentIdElement);
                    SelfHealingMessage selfHealingMessage = new SelfHealingMessage();
                    selfHealingMessage.setEventType(eventType);
                    //selfHealingMessage.setId(12121212L);
                    selfHealingMessage.setStatus(Status.PENDING);
                    selfHealingMessage.setOrigin("pm-grafana");
                    selfHealingMessage.setError(error.toString());
                    selfHealingMessage.setDeploymentId(deploymentIdElement.getAsString());
                    selfHealingMessage.setTimestamp(Instant.now());
                    try {
                        selfHealingMessageResource.createSelfHealingMessage(selfHealingMessage);
                    } catch (URISyntaxException e) {
                        throw new RuntimeException(e);
                    }
                } else {
                    // process alert_value
                    String alertValueYamlString = jsonElementAlertValue.getAsString();
                    Yaml yaml = new Yaml();
                    LinkedHashMap alertValueYaml = null;
                    try {
                        alertValueYaml = yaml.load(alertValueYamlString);
                    } catch (Exception e) {
                        log.debug("Skipping alert with fingerprint {} because alert_value is not a valid yaml", fingerprint);
                        log.debug("Alert details: {}", alert);
                        continue;
                    }
                    JsonObject internalAlertsJson = gson.toJsonTree(alertValueYaml).getAsJsonObject();
                    JsonElement internalAlerts = internalAlertsJson.get("alerts");
                    if (internalAlerts == null) {
                        log.debug("Skipping alert with fingerprint {} because alerts is null", fingerprint);
                        log.debug("Alert details: {}", alert);
                        continue;
                    }
                    JsonArray alertsArray = internalAlerts.getAsJsonArray();
                    for( JsonElement internalAlertElement : alertsArray ) {
                        JsonObject internalAlert = internalAlertElement.getAsJsonObject();
                        JsonElement deploymentIdElement = internalAlert.get("deployment_id");
                        if (deploymentIdElement == null) {
                            log.debug("Skipping alert with fingerprint {} because deployment_id is null", fingerprint);
                            log.debug("Alert details: {}", alert);
                            continue;
                        }
                        for (Map.Entry<String, JsonElement> entry : internalAlert.entrySet()) {
                                labels.add(entry.getKey(), entry.getValue());
                        }
                        SelfHealingMessage selfHealingMessage = new SelfHealingMessage();
                        selfHealingMessage.setEventType(eventType);
                        selfHealingMessage.setStatus(Status.PENDING);
                        selfHealingMessage.setOrigin("pm-grafana");
                        selfHealingMessage.setError(error.toString());
                        selfHealingMessage.setDeploymentId(deploymentIdElement.getAsString());
                        selfHealingMessage.setTimestamp(Instant.now());

                        try {
                            selfHealingMessageResource.createSelfHealingMessage(selfHealingMessage);
                        } catch (URISyntaxException e) {
                            throw new RuntimeException(e);
                        }
                    }
                    log.debug("Alert value: {}", internalAlerts);
                }
           }
        }

        log.info("WEBHOOK SELF-HEALING REST SERVICE FINISHED------------------------------------------------------");
        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    @PostMapping("/deployments")
    public ResponseEntity deployment(@RequestBody DeploymentDTO deploymentDTO) {
        log.info("Creating Deployment Resource: {}", deploymentDTO.getDeploymentId());

        Deployment deployment = new Deployment();
        deployment.setDeploymentId(deploymentDTO.getDeploymentId());
        byte[] deploymentBundle = DatatypeConverter.parseBase64Binary(deploymentDTO.getDeploymentBundle().getBase64());
        deployment.bundle(deploymentBundle);
        deployment.bundleContentType("application/zip");

        try {
            deploymentResource.createDeployment(deployment);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }

        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    @DeleteMapping("/deployments/{deploymentId}")
    public ResponseEntity deleteDeployment(@PathVariable String deploymentId) {
        log.info("Deleting Deployment Resource: {}", deploymentId);
        Deployment deployment = deploymentRepository.getByDeploymentId(deploymentId);
        if (deployment == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        deploymentResource.deleteDeployment(deployment.getId());

        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }
}
