package com.piacere.selfhealing.backend.web.rest;

import com.piacere.selfhealing.backend.domain.StrategyApplication;
import com.piacere.selfhealing.backend.repository.StrategyApplicationRepository;
import com.piacere.selfhealing.backend.web.rest.errors.BadRequestAlertException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.piacere.selfhealing.backend.domain.StrategyApplication}.
 */
@RestController
@RequestMapping("/api")
@Transactional
public class StrategyApplicationResource {

    private final Logger log = LoggerFactory.getLogger(StrategyApplicationResource.class);

    private static final String ENTITY_NAME = "shBackendStrategyApplication";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final StrategyApplicationRepository strategyApplicationRepository;

    public StrategyApplicationResource(StrategyApplicationRepository strategyApplicationRepository) {
        this.strategyApplicationRepository = strategyApplicationRepository;
    }

    /**
     * {@code POST  /strategy-applications} : Create a new strategyApplication.
     *
     * @param strategyApplication the strategyApplication to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new strategyApplication, or with status {@code 400 (Bad Request)} if the strategyApplication has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("/strategy-applications")
    public ResponseEntity<StrategyApplication> createStrategyApplication(@RequestBody StrategyApplication strategyApplication)
        throws URISyntaxException {
        log.debug("REST request to save StrategyApplication : {}", strategyApplication);
        if (strategyApplication.getId() != null) {
            throw new BadRequestAlertException("A new strategyApplication cannot already have an ID", ENTITY_NAME, "idexists");
        }
        StrategyApplication result = strategyApplicationRepository.save(strategyApplication);
        return ResponseEntity
            .created(new URI("/api/strategy-applications/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * {@code PUT  /strategy-applications/:id} : Updates an existing strategyApplication.
     *
     * @param id the id of the strategyApplication to save.
     * @param strategyApplication the strategyApplication to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated strategyApplication,
     * or with status {@code 400 (Bad Request)} if the strategyApplication is not valid,
     * or with status {@code 500 (Internal Server Error)} if the strategyApplication couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/strategy-applications/{id}")
    public ResponseEntity<StrategyApplication> updateStrategyApplication(
        @PathVariable(value = "id", required = false) final Long id,
        @RequestBody StrategyApplication strategyApplication
    ) throws URISyntaxException {
        log.debug("REST request to update StrategyApplication : {}, {}", id, strategyApplication);
        if (strategyApplication.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, strategyApplication.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!strategyApplicationRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        StrategyApplication result = strategyApplicationRepository.save(strategyApplication);
        return ResponseEntity
            .ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, strategyApplication.getId().toString()))
            .body(result);
    }

    /**
     * {@code PATCH  /strategy-applications/:id} : Partial updates given fields of an existing strategyApplication, field will ignore if it is null
     *
     * @param id the id of the strategyApplication to save.
     * @param strategyApplication the strategyApplication to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated strategyApplication,
     * or with status {@code 400 (Bad Request)} if the strategyApplication is not valid,
     * or with status {@code 404 (Not Found)} if the strategyApplication is not found,
     * or with status {@code 500 (Internal Server Error)} if the strategyApplication couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/strategy-applications/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<StrategyApplication> partialUpdateStrategyApplication(
        @PathVariable(value = "id", required = false) final Long id,
        @RequestBody StrategyApplication strategyApplication
    ) throws URISyntaxException {
        log.debug("REST request to partial update StrategyApplication partially : {}, {}", id, strategyApplication);
        if (strategyApplication.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, strategyApplication.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!strategyApplicationRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<StrategyApplication> result = strategyApplicationRepository
            .findById(strategyApplication.getId())
            .map(existingStrategyApplication -> {
                if (strategyApplication.getTimestamp() != null) {
                    existingStrategyApplication.setTimestamp(strategyApplication.getTimestamp());
                }
                if (strategyApplication.getSummary() != null) {
                    existingStrategyApplication.setSummary(strategyApplication.getSummary());
                }
                if (strategyApplication.getBundle() != null) {
                    existingStrategyApplication.setBundle(strategyApplication.getBundle());
                }
                if (strategyApplication.getBundleContentType() != null) {
                    existingStrategyApplication.setBundleContentType(strategyApplication.getBundleContentType());
                }

                return existingStrategyApplication;
            })
            .map(strategyApplicationRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, strategyApplication.getId().toString())
        );
    }

    /**
     * {@code GET  /strategy-applications} : get all the strategyApplications.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of strategyApplications in body.
     */
    @GetMapping("/strategy-applications")
    public List<StrategyApplication> getAllStrategyApplications() {
        log.debug("REST request to get all StrategyApplications");
        return strategyApplicationRepository.findAll();
    }

    /**
     * {@code GET  /strategy-applications/:id} : get the "id" strategyApplication.
     *
     * @param id the id of the strategyApplication to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the strategyApplication, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/strategy-applications/{id}")
    public ResponseEntity<StrategyApplication> getStrategyApplication(@PathVariable Long id) {
        log.debug("REST request to get StrategyApplication : {}", id);
        Optional<StrategyApplication> strategyApplication = strategyApplicationRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(strategyApplication);
    }

    /**
     * {@code DELETE  /strategy-applications/:id} : delete the "id" strategyApplication.
     *
     * @param id the id of the strategyApplication to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/strategy-applications/{id}")
    public ResponseEntity<Void> deleteStrategyApplication(@PathVariable Long id) {
        log.debug("REST request to delete StrategyApplication : {}", id);
        strategyApplicationRepository.deleteById(id);
        return ResponseEntity
            .noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
