package com.piacere.selfhealing.backend.web.rest;

import com.piacere.selfhealing.backend.domain.Strategy;
import com.piacere.selfhealing.backend.repository.StrategyRepository;
import com.piacere.selfhealing.backend.web.rest.errors.BadRequestAlertException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.piacere.selfhealing.backend.domain.Strategy}.
 */
@RestController
@RequestMapping("/api")
@Transactional
public class StrategyResource {

    private final Logger log = LoggerFactory.getLogger(StrategyResource.class);

    private static final String ENTITY_NAME = "shBackendStrategy";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final StrategyRepository strategyRepository;

    public StrategyResource(StrategyRepository strategyRepository) {
        this.strategyRepository = strategyRepository;
    }

    /**
     * {@code POST  /strategies} : Create a new strategy.
     *
     * @param strategy the strategy to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new strategy, or with status {@code 400 (Bad Request)} if the strategy has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("/strategies")
    public ResponseEntity<Strategy> createStrategy(@Valid @RequestBody Strategy strategy) throws URISyntaxException {
        log.debug("REST request to save Strategy : {}", strategy);
        if (strategy.getId() != null) {
            throw new BadRequestAlertException("A new strategy cannot already have an ID", ENTITY_NAME, "idexists");
        }
        Strategy result = strategyRepository.save(strategy);
        return ResponseEntity
            .created(new URI("/api/strategies/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * {@code PUT  /strategies/:id} : Updates an existing strategy.
     *
     * @param id the id of the strategy to save.
     * @param strategy the strategy to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated strategy,
     * or with status {@code 400 (Bad Request)} if the strategy is not valid,
     * or with status {@code 500 (Internal Server Error)} if the strategy couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/strategies/{id}")
    public ResponseEntity<Strategy> updateStrategy(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody Strategy strategy
    ) throws URISyntaxException {
        log.debug("REST request to update Strategy : {}, {}", id, strategy);
        if (strategy.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, strategy.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!strategyRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Strategy result = strategyRepository.save(strategy);
        return ResponseEntity
            .ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, strategy.getId().toString()))
            .body(result);
    }

    /**
     * {@code PATCH  /strategies/:id} : Partial updates given fields of an existing strategy, field will ignore if it is null
     *
     * @param id the id of the strategy to save.
     * @param strategy the strategy to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated strategy,
     * or with status {@code 400 (Bad Request)} if the strategy is not valid,
     * or with status {@code 404 (Not Found)} if the strategy is not found,
     * or with status {@code 500 (Internal Server Error)} if the strategy couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/strategies/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<Strategy> partialUpdateStrategy(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody Strategy strategy
    ) throws URISyntaxException {
        log.debug("REST request to partial update Strategy partially : {}, {}", id, strategy);
        if (strategy.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, strategy.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!strategyRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<Strategy> result = strategyRepository
            .findById(strategy.getId())
            .map(existingStrategy -> {
                if (strategy.getCode() != null) {
                    existingStrategy.setCode(strategy.getCode());
                }
                if (strategy.getDescription() != null) {
                    existingStrategy.setDescription(strategy.getDescription());
                }
                if (strategy.getConfiguration() != null) {
                    existingStrategy.setConfiguration(strategy.getConfiguration());
                }

                return existingStrategy;
            })
            .map(strategyRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, strategy.getId().toString())
        );
    }

    /**
     * {@code GET  /strategies} : get all the strategies.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of strategies in body.
     */
    @GetMapping("/strategies")
    public List<Strategy> getAllStrategies() {
        log.debug("REST request to get all Strategies");
        return strategyRepository.findAll();
    }

    /**
     * {@code GET  /strategies/:id} : get the "id" strategy.
     *
     * @param id the id of the strategy to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the strategy, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/strategies/{id}")
    public ResponseEntity<Strategy> getStrategy(@PathVariable Long id) {
        log.debug("REST request to get Strategy : {}", id);
        Optional<Strategy> strategy = strategyRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(strategy);
    }

    /**
     * {@code DELETE  /strategies/:id} : delete the "id" strategy.
     *
     * @param id the id of the strategy to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/strategies/{id}")
    public ResponseEntity<Void> deleteStrategy(@PathVariable Long id) {
        log.debug("REST request to delete Strategy : {}", id);
        strategyRepository.deleteById(id);
        return ResponseEntity
            .noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
