package com.piacere.selfhealing.backend.web.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.piacere.selfhealing.backend.IntegrationTest;
import com.piacere.selfhealing.backend.domain.Deployment;
import com.piacere.selfhealing.backend.repository.DeploymentRepository;
import java.util.List;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import javax.persistence.EntityManager;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Base64Utils;

/**
 * Integration tests for the {@link DeploymentResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
class DeploymentResourceIT {

    private static final String DEFAULT_DEPLOYMENT_ID = "AAAAAAAAAA";
    private static final String UPDATED_DEPLOYMENT_ID = "BBBBBBBBBB";

    private static final byte[] DEFAULT_BUNDLE = TestUtil.createByteArray(1, "0");
    private static final byte[] UPDATED_BUNDLE = TestUtil.createByteArray(1, "1");
    private static final String DEFAULT_BUNDLE_CONTENT_TYPE = "image/jpg";
    private static final String UPDATED_BUNDLE_CONTENT_TYPE = "image/png";

    private static final String ENTITY_API_URL = "/api/deployments";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong count = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private DeploymentRepository deploymentRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restDeploymentMockMvc;

    private Deployment deployment;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Deployment createEntity(EntityManager em) {
        Deployment deployment = new Deployment()
            .deploymentId(DEFAULT_DEPLOYMENT_ID)
            .bundle(DEFAULT_BUNDLE)
            .bundleContentType(DEFAULT_BUNDLE_CONTENT_TYPE);
        return deployment;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Deployment createUpdatedEntity(EntityManager em) {
        Deployment deployment = new Deployment()
            .deploymentId(UPDATED_DEPLOYMENT_ID)
            .bundle(UPDATED_BUNDLE)
            .bundleContentType(UPDATED_BUNDLE_CONTENT_TYPE);
        return deployment;
    }

    @BeforeEach
    public void initTest() {
        deployment = createEntity(em);
    }

    @Test
    @Transactional
    void createDeployment() throws Exception {
        int databaseSizeBeforeCreate = deploymentRepository.findAll().size();
        // Create the Deployment
        restDeploymentMockMvc
            .perform(post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(deployment)))
            .andExpect(status().isCreated());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeCreate + 1);
        Deployment testDeployment = deploymentList.get(deploymentList.size() - 1);
        assertThat(testDeployment.getDeploymentId()).isEqualTo(DEFAULT_DEPLOYMENT_ID);
        assertThat(testDeployment.getBundle()).isEqualTo(DEFAULT_BUNDLE);
        assertThat(testDeployment.getBundleContentType()).isEqualTo(DEFAULT_BUNDLE_CONTENT_TYPE);
    }

    @Test
    @Transactional
    void createDeploymentWithExistingId() throws Exception {
        // Create the Deployment with an existing ID
        deployment.setId(1L);

        int databaseSizeBeforeCreate = deploymentRepository.findAll().size();

        // An entity with an existing ID cannot be created, so this API call must fail
        restDeploymentMockMvc
            .perform(post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(deployment)))
            .andExpect(status().isBadRequest());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkDeploymentIdIsRequired() throws Exception {
        int databaseSizeBeforeTest = deploymentRepository.findAll().size();
        // set the field null
        deployment.setDeploymentId(null);

        // Create the Deployment, which fails.

        restDeploymentMockMvc
            .perform(post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(deployment)))
            .andExpect(status().isBadRequest());

        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllDeployments() throws Exception {
        // Initialize the database
        deploymentRepository.saveAndFlush(deployment);

        // Get all the deploymentList
        restDeploymentMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(deployment.getId().intValue())))
            .andExpect(jsonPath("$.[*].deploymentId").value(hasItem(DEFAULT_DEPLOYMENT_ID)))
            .andExpect(jsonPath("$.[*].bundleContentType").value(hasItem(DEFAULT_BUNDLE_CONTENT_TYPE)))
            .andExpect(jsonPath("$.[*].bundle").value(hasItem(Base64Utils.encodeToString(DEFAULT_BUNDLE))));
    }

    @Test
    @Transactional
    void getDeployment() throws Exception {
        // Initialize the database
        deploymentRepository.saveAndFlush(deployment);

        // Get the deployment
        restDeploymentMockMvc
            .perform(get(ENTITY_API_URL_ID, deployment.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(deployment.getId().intValue()))
            .andExpect(jsonPath("$.deploymentId").value(DEFAULT_DEPLOYMENT_ID))
            .andExpect(jsonPath("$.bundleContentType").value(DEFAULT_BUNDLE_CONTENT_TYPE))
            .andExpect(jsonPath("$.bundle").value(Base64Utils.encodeToString(DEFAULT_BUNDLE)));
    }

    @Test
    @Transactional
    void getNonExistingDeployment() throws Exception {
        // Get the deployment
        restDeploymentMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putNewDeployment() throws Exception {
        // Initialize the database
        deploymentRepository.saveAndFlush(deployment);

        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();

        // Update the deployment
        Deployment updatedDeployment = deploymentRepository.findById(deployment.getId()).get();
        // Disconnect from session so that the updates on updatedDeployment are not directly saved in db
        em.detach(updatedDeployment);
        updatedDeployment.deploymentId(UPDATED_DEPLOYMENT_ID).bundle(UPDATED_BUNDLE).bundleContentType(UPDATED_BUNDLE_CONTENT_TYPE);

        restDeploymentMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedDeployment.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(updatedDeployment))
            )
            .andExpect(status().isOk());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
        Deployment testDeployment = deploymentList.get(deploymentList.size() - 1);
        assertThat(testDeployment.getDeploymentId()).isEqualTo(UPDATED_DEPLOYMENT_ID);
        assertThat(testDeployment.getBundle()).isEqualTo(UPDATED_BUNDLE);
        assertThat(testDeployment.getBundleContentType()).isEqualTo(UPDATED_BUNDLE_CONTENT_TYPE);
    }

    @Test
    @Transactional
    void putNonExistingDeployment() throws Exception {
        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();
        deployment.setId(count.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restDeploymentMockMvc
            .perform(
                put(ENTITY_API_URL_ID, deployment.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(deployment))
            )
            .andExpect(status().isBadRequest());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchDeployment() throws Exception {
        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();
        deployment.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restDeploymentMockMvc
            .perform(
                put(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(deployment))
            )
            .andExpect(status().isBadRequest());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamDeployment() throws Exception {
        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();
        deployment.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restDeploymentMockMvc
            .perform(put(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(deployment)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateDeploymentWithPatch() throws Exception {
        // Initialize the database
        deploymentRepository.saveAndFlush(deployment);

        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();

        // Update the deployment using partial update
        Deployment partialUpdatedDeployment = new Deployment();
        partialUpdatedDeployment.setId(deployment.getId());

        restDeploymentMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedDeployment.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedDeployment))
            )
            .andExpect(status().isOk());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
        Deployment testDeployment = deploymentList.get(deploymentList.size() - 1);
        assertThat(testDeployment.getDeploymentId()).isEqualTo(DEFAULT_DEPLOYMENT_ID);
        assertThat(testDeployment.getBundle()).isEqualTo(DEFAULT_BUNDLE);
        assertThat(testDeployment.getBundleContentType()).isEqualTo(DEFAULT_BUNDLE_CONTENT_TYPE);
    }

    @Test
    @Transactional
    void fullUpdateDeploymentWithPatch() throws Exception {
        // Initialize the database
        deploymentRepository.saveAndFlush(deployment);

        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();

        // Update the deployment using partial update
        Deployment partialUpdatedDeployment = new Deployment();
        partialUpdatedDeployment.setId(deployment.getId());

        partialUpdatedDeployment.deploymentId(UPDATED_DEPLOYMENT_ID).bundle(UPDATED_BUNDLE).bundleContentType(UPDATED_BUNDLE_CONTENT_TYPE);

        restDeploymentMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedDeployment.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedDeployment))
            )
            .andExpect(status().isOk());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
        Deployment testDeployment = deploymentList.get(deploymentList.size() - 1);
        assertThat(testDeployment.getDeploymentId()).isEqualTo(UPDATED_DEPLOYMENT_ID);
        assertThat(testDeployment.getBundle()).isEqualTo(UPDATED_BUNDLE);
        assertThat(testDeployment.getBundleContentType()).isEqualTo(UPDATED_BUNDLE_CONTENT_TYPE);
    }

    @Test
    @Transactional
    void patchNonExistingDeployment() throws Exception {
        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();
        deployment.setId(count.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restDeploymentMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, deployment.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(deployment))
            )
            .andExpect(status().isBadRequest());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchDeployment() throws Exception {
        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();
        deployment.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restDeploymentMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(deployment))
            )
            .andExpect(status().isBadRequest());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamDeployment() throws Exception {
        int databaseSizeBeforeUpdate = deploymentRepository.findAll().size();
        deployment.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restDeploymentMockMvc
            .perform(
                patch(ENTITY_API_URL).contentType("application/merge-patch+json").content(TestUtil.convertObjectToJsonBytes(deployment))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the Deployment in the database
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteDeployment() throws Exception {
        // Initialize the database
        deploymentRepository.saveAndFlush(deployment);

        int databaseSizeBeforeDelete = deploymentRepository.findAll().size();

        // Delete the deployment
        restDeploymentMockMvc
            .perform(delete(ENTITY_API_URL_ID, deployment.getId()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<Deployment> deploymentList = deploymentRepository.findAll();
        assertThat(deploymentList).hasSize(databaseSizeBeforeDelete - 1);
    }
}
