package com.piacere.selfhealing.backend.web.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.piacere.selfhealing.backend.IntegrationTest;
import com.piacere.selfhealing.backend.domain.SelfHealingMessage;
import com.piacere.selfhealing.backend.domain.enumeration.Status;
import com.piacere.selfhealing.backend.repository.SelfHealingMessageRepository;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.List;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import javax.persistence.EntityManager;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Base64Utils;

/**
 * Integration tests for the {@link SelfHealingMessageResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
class SelfHealingMessageResourceIT {

    private static final Instant DEFAULT_TIMESTAMP = Instant.ofEpochMilli(0L);
    private static final Instant UPDATED_TIMESTAMP = Instant.now().truncatedTo(ChronoUnit.MILLIS);

    private static final String DEFAULT_ORIGIN = "AAAAAAAAAA";
    private static final String UPDATED_ORIGIN = "BBBBBBBBBB";

    private static final String DEFAULT_DEPLOYMENT_ID = "AAAAAAAAAA";
    private static final String UPDATED_DEPLOYMENT_ID = "BBBBBBBBBB";

    private static final Status DEFAULT_STATUS = Status.PENDING;
    private static final Status UPDATED_STATUS = Status.PROCESSED;

    private static final String DEFAULT_ERROR = "AAAAAAAAAA";
    private static final String UPDATED_ERROR = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/self-healing-messages";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong count = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private SelfHealingMessageRepository selfHealingMessageRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restSelfHealingMessageMockMvc;

    private SelfHealingMessage selfHealingMessage;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SelfHealingMessage createEntity(EntityManager em) {
        SelfHealingMessage selfHealingMessage = new SelfHealingMessage()
            .timestamp(DEFAULT_TIMESTAMP)
            .origin(DEFAULT_ORIGIN)
            .deploymentId(DEFAULT_DEPLOYMENT_ID)
            .status(DEFAULT_STATUS)
            .error(DEFAULT_ERROR);
        return selfHealingMessage;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SelfHealingMessage createUpdatedEntity(EntityManager em) {
        SelfHealingMessage selfHealingMessage = new SelfHealingMessage()
            .timestamp(UPDATED_TIMESTAMP)
            .origin(UPDATED_ORIGIN)
            .deploymentId(UPDATED_DEPLOYMENT_ID)
            .status(UPDATED_STATUS)
            .error(UPDATED_ERROR);
        return selfHealingMessage;
    }

    @BeforeEach
    public void initTest() {
        selfHealingMessage = createEntity(em);
    }

    @Test
    @Transactional
    void createSelfHealingMessage() throws Exception {
        int databaseSizeBeforeCreate = selfHealingMessageRepository.findAll().size();
        // Create the SelfHealingMessage
        restSelfHealingMessageMockMvc
            .perform(
                post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isCreated());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeCreate + 1);
        SelfHealingMessage testSelfHealingMessage = selfHealingMessageList.get(selfHealingMessageList.size() - 1);
        assertThat(testSelfHealingMessage.getTimestamp()).isEqualTo(DEFAULT_TIMESTAMP);
        assertThat(testSelfHealingMessage.getOrigin()).isEqualTo(DEFAULT_ORIGIN);
        assertThat(testSelfHealingMessage.getDeploymentId()).isEqualTo(DEFAULT_DEPLOYMENT_ID);
        assertThat(testSelfHealingMessage.getStatus()).isEqualTo(DEFAULT_STATUS);
        assertThat(testSelfHealingMessage.getError()).isEqualTo(DEFAULT_ERROR);
    }

    @Test
    @Transactional
    void createSelfHealingMessageWithExistingId() throws Exception {
        // Create the SelfHealingMessage with an existing ID
        selfHealingMessage.setId(1L);

        int databaseSizeBeforeCreate = selfHealingMessageRepository.findAll().size();

        // An entity with an existing ID cannot be created, so this API call must fail
        restSelfHealingMessageMockMvc
            .perform(
                post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isBadRequest());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkOriginIsRequired() throws Exception {
        int databaseSizeBeforeTest = selfHealingMessageRepository.findAll().size();
        // set the field null
        selfHealingMessage.setOrigin(null);

        // Create the SelfHealingMessage, which fails.

        restSelfHealingMessageMockMvc
            .perform(
                post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isBadRequest());

        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllSelfHealingMessages() throws Exception {
        // Initialize the database
        selfHealingMessageRepository.saveAndFlush(selfHealingMessage);

        // Get all the selfHealingMessageList
        restSelfHealingMessageMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(selfHealingMessage.getId().intValue())))
            .andExpect(jsonPath("$.[*].timestamp").value(hasItem(DEFAULT_TIMESTAMP.toString())))
            .andExpect(jsonPath("$.[*].origin").value(hasItem(DEFAULT_ORIGIN)))
            .andExpect(jsonPath("$.[*].deploymentId").value(hasItem(DEFAULT_DEPLOYMENT_ID)))
            .andExpect(jsonPath("$.[*].status").value(hasItem(DEFAULT_STATUS.toString())))
            .andExpect(jsonPath("$.[*].error").value(hasItem(DEFAULT_ERROR.toString())));
    }

    @Test
    @Transactional
    void getSelfHealingMessage() throws Exception {
        // Initialize the database
        selfHealingMessageRepository.saveAndFlush(selfHealingMessage);

        // Get the selfHealingMessage
        restSelfHealingMessageMockMvc
            .perform(get(ENTITY_API_URL_ID, selfHealingMessage.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(selfHealingMessage.getId().intValue()))
            .andExpect(jsonPath("$.timestamp").value(DEFAULT_TIMESTAMP.toString()))
            .andExpect(jsonPath("$.origin").value(DEFAULT_ORIGIN))
            .andExpect(jsonPath("$.deploymentId").value(DEFAULT_DEPLOYMENT_ID))
            .andExpect(jsonPath("$.status").value(DEFAULT_STATUS.toString()))
            .andExpect(jsonPath("$.error").value(DEFAULT_ERROR.toString()));
    }

    @Test
    @Transactional
    void getNonExistingSelfHealingMessage() throws Exception {
        // Get the selfHealingMessage
        restSelfHealingMessageMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putNewSelfHealingMessage() throws Exception {
        // Initialize the database
        selfHealingMessageRepository.saveAndFlush(selfHealingMessage);

        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();

        // Update the selfHealingMessage
        SelfHealingMessage updatedSelfHealingMessage = selfHealingMessageRepository.findById(selfHealingMessage.getId()).get();
        // Disconnect from session so that the updates on updatedSelfHealingMessage are not directly saved in db
        em.detach(updatedSelfHealingMessage);
        updatedSelfHealingMessage
            .timestamp(UPDATED_TIMESTAMP)
            .origin(UPDATED_ORIGIN)
            .deploymentId(UPDATED_DEPLOYMENT_ID)
            .status(UPDATED_STATUS)
            .error(UPDATED_ERROR);

        restSelfHealingMessageMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedSelfHealingMessage.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(updatedSelfHealingMessage))
            )
            .andExpect(status().isOk());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
        SelfHealingMessage testSelfHealingMessage = selfHealingMessageList.get(selfHealingMessageList.size() - 1);
        assertThat(testSelfHealingMessage.getTimestamp()).isEqualTo(UPDATED_TIMESTAMP);
        assertThat(testSelfHealingMessage.getOrigin()).isEqualTo(UPDATED_ORIGIN);
        assertThat(testSelfHealingMessage.getDeploymentId()).isEqualTo(UPDATED_DEPLOYMENT_ID);
        assertThat(testSelfHealingMessage.getStatus()).isEqualTo(UPDATED_STATUS);
        assertThat(testSelfHealingMessage.getError()).isEqualTo(UPDATED_ERROR);
    }

    @Test
    @Transactional
    void putNonExistingSelfHealingMessage() throws Exception {
        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();
        selfHealingMessage.setId(count.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSelfHealingMessageMockMvc
            .perform(
                put(ENTITY_API_URL_ID, selfHealingMessage.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isBadRequest());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchSelfHealingMessage() throws Exception {
        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();
        selfHealingMessage.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSelfHealingMessageMockMvc
            .perform(
                put(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isBadRequest());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamSelfHealingMessage() throws Exception {
        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();
        selfHealingMessage.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSelfHealingMessageMockMvc
            .perform(
                put(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateSelfHealingMessageWithPatch() throws Exception {
        // Initialize the database
        selfHealingMessageRepository.saveAndFlush(selfHealingMessage);

        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();

        // Update the selfHealingMessage using partial update
        SelfHealingMessage partialUpdatedSelfHealingMessage = new SelfHealingMessage();
        partialUpdatedSelfHealingMessage.setId(selfHealingMessage.getId());

        partialUpdatedSelfHealingMessage.timestamp(UPDATED_TIMESTAMP).deploymentId(UPDATED_DEPLOYMENT_ID).error(UPDATED_ERROR);

        restSelfHealingMessageMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSelfHealingMessage.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedSelfHealingMessage))
            )
            .andExpect(status().isOk());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
        SelfHealingMessage testSelfHealingMessage = selfHealingMessageList.get(selfHealingMessageList.size() - 1);
        assertThat(testSelfHealingMessage.getTimestamp()).isEqualTo(UPDATED_TIMESTAMP);
        assertThat(testSelfHealingMessage.getOrigin()).isEqualTo(DEFAULT_ORIGIN);
        assertThat(testSelfHealingMessage.getDeploymentId()).isEqualTo(UPDATED_DEPLOYMENT_ID);
        assertThat(testSelfHealingMessage.getStatus()).isEqualTo(DEFAULT_STATUS);
        assertThat(testSelfHealingMessage.getError()).isEqualTo(UPDATED_ERROR);
    }

    @Test
    @Transactional
    void fullUpdateSelfHealingMessageWithPatch() throws Exception {
        // Initialize the database
        selfHealingMessageRepository.saveAndFlush(selfHealingMessage);

        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();

        // Update the selfHealingMessage using partial update
        SelfHealingMessage partialUpdatedSelfHealingMessage = new SelfHealingMessage();
        partialUpdatedSelfHealingMessage.setId(selfHealingMessage.getId());

        partialUpdatedSelfHealingMessage
            .timestamp(UPDATED_TIMESTAMP)
            .origin(UPDATED_ORIGIN)
            .deploymentId(UPDATED_DEPLOYMENT_ID)
            .status(UPDATED_STATUS)
            .error(UPDATED_ERROR);

        restSelfHealingMessageMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSelfHealingMessage.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedSelfHealingMessage))
            )
            .andExpect(status().isOk());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
        SelfHealingMessage testSelfHealingMessage = selfHealingMessageList.get(selfHealingMessageList.size() - 1);
        assertThat(testSelfHealingMessage.getTimestamp()).isEqualTo(UPDATED_TIMESTAMP);
        assertThat(testSelfHealingMessage.getOrigin()).isEqualTo(UPDATED_ORIGIN);
        assertThat(testSelfHealingMessage.getDeploymentId()).isEqualTo(UPDATED_DEPLOYMENT_ID);
        assertThat(testSelfHealingMessage.getStatus()).isEqualTo(UPDATED_STATUS);
        assertThat(testSelfHealingMessage.getError()).isEqualTo(UPDATED_ERROR);
    }

    @Test
    @Transactional
    void patchNonExistingSelfHealingMessage() throws Exception {
        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();
        selfHealingMessage.setId(count.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSelfHealingMessageMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, selfHealingMessage.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isBadRequest());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchSelfHealingMessage() throws Exception {
        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();
        selfHealingMessage.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSelfHealingMessageMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isBadRequest());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamSelfHealingMessage() throws Exception {
        int databaseSizeBeforeUpdate = selfHealingMessageRepository.findAll().size();
        selfHealingMessage.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSelfHealingMessageMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(selfHealingMessage))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SelfHealingMessage in the database
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteSelfHealingMessage() throws Exception {
        // Initialize the database
        selfHealingMessageRepository.saveAndFlush(selfHealingMessage);

        int databaseSizeBeforeDelete = selfHealingMessageRepository.findAll().size();

        // Delete the selfHealingMessage
        restSelfHealingMessageMockMvc
            .perform(delete(ENTITY_API_URL_ID, selfHealingMessage.getId()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<SelfHealingMessage> selfHealingMessageList = selfHealingMessageRepository.findAll();
        assertThat(selfHealingMessageList).hasSize(databaseSizeBeforeDelete - 1);
    }
}
