package com.piacere.selfhealing.backend.web.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.piacere.selfhealing.backend.IntegrationTest;
import com.piacere.selfhealing.backend.domain.StrategyApplication;
import com.piacere.selfhealing.backend.repository.StrategyApplicationRepository;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.List;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import javax.persistence.EntityManager;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Base64Utils;

/**
 * Integration tests for the {@link StrategyApplicationResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
class StrategyApplicationResourceIT {

    private static final Instant DEFAULT_TIMESTAMP = Instant.ofEpochMilli(0L);
    private static final Instant UPDATED_TIMESTAMP = Instant.now().truncatedTo(ChronoUnit.MILLIS);

    private static final String DEFAULT_SUMMARY = "AAAAAAAAAA";
    private static final String UPDATED_SUMMARY = "BBBBBBBBBB";

    private static final byte[] DEFAULT_BUNDLE = TestUtil.createByteArray(1, "0");
    private static final byte[] UPDATED_BUNDLE = TestUtil.createByteArray(1, "1");
    private static final String DEFAULT_BUNDLE_CONTENT_TYPE = "image/jpg";
    private static final String UPDATED_BUNDLE_CONTENT_TYPE = "image/png";

    private static final String ENTITY_API_URL = "/api/strategy-applications";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong count = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private StrategyApplicationRepository strategyApplicationRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restStrategyApplicationMockMvc;

    private StrategyApplication strategyApplication;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static StrategyApplication createEntity(EntityManager em) {
        StrategyApplication strategyApplication = new StrategyApplication()
            .timestamp(DEFAULT_TIMESTAMP)
            .summary(DEFAULT_SUMMARY)
            .bundle(DEFAULT_BUNDLE)
            .bundleContentType(DEFAULT_BUNDLE_CONTENT_TYPE);
        return strategyApplication;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static StrategyApplication createUpdatedEntity(EntityManager em) {
        StrategyApplication strategyApplication = new StrategyApplication()
            .timestamp(UPDATED_TIMESTAMP)
            .summary(UPDATED_SUMMARY)
            .bundle(UPDATED_BUNDLE)
            .bundleContentType(UPDATED_BUNDLE_CONTENT_TYPE);
        return strategyApplication;
    }

    @BeforeEach
    public void initTest() {
        strategyApplication = createEntity(em);
    }

    @Test
    @Transactional
    void createStrategyApplication() throws Exception {
        int databaseSizeBeforeCreate = strategyApplicationRepository.findAll().size();
        // Create the StrategyApplication
        restStrategyApplicationMockMvc
            .perform(
                post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(strategyApplication))
            )
            .andExpect(status().isCreated());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeCreate + 1);
        StrategyApplication testStrategyApplication = strategyApplicationList.get(strategyApplicationList.size() - 1);
        assertThat(testStrategyApplication.getTimestamp()).isEqualTo(DEFAULT_TIMESTAMP);
        assertThat(testStrategyApplication.getSummary()).isEqualTo(DEFAULT_SUMMARY);
        assertThat(testStrategyApplication.getBundle()).isEqualTo(DEFAULT_BUNDLE);
        assertThat(testStrategyApplication.getBundleContentType()).isEqualTo(DEFAULT_BUNDLE_CONTENT_TYPE);
    }

    @Test
    @Transactional
    void createStrategyApplicationWithExistingId() throws Exception {
        // Create the StrategyApplication with an existing ID
        strategyApplication.setId(1L);

        int databaseSizeBeforeCreate = strategyApplicationRepository.findAll().size();

        // An entity with an existing ID cannot be created, so this API call must fail
        restStrategyApplicationMockMvc
            .perform(
                post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(strategyApplication))
            )
            .andExpect(status().isBadRequest());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void getAllStrategyApplications() throws Exception {
        // Initialize the database
        strategyApplicationRepository.saveAndFlush(strategyApplication);

        // Get all the strategyApplicationList
        restStrategyApplicationMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(strategyApplication.getId().intValue())))
            .andExpect(jsonPath("$.[*].timestamp").value(hasItem(DEFAULT_TIMESTAMP.toString())))
            .andExpect(jsonPath("$.[*].summary").value(hasItem(DEFAULT_SUMMARY.toString())))
            .andExpect(jsonPath("$.[*].bundleContentType").value(hasItem(DEFAULT_BUNDLE_CONTENT_TYPE)))
            .andExpect(jsonPath("$.[*].bundle").value(hasItem(Base64Utils.encodeToString(DEFAULT_BUNDLE))));
    }

    @Test
    @Transactional
    void getStrategyApplication() throws Exception {
        // Initialize the database
        strategyApplicationRepository.saveAndFlush(strategyApplication);

        // Get the strategyApplication
        restStrategyApplicationMockMvc
            .perform(get(ENTITY_API_URL_ID, strategyApplication.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(strategyApplication.getId().intValue()))
            .andExpect(jsonPath("$.timestamp").value(DEFAULT_TIMESTAMP.toString()))
            .andExpect(jsonPath("$.summary").value(DEFAULT_SUMMARY.toString()))
            .andExpect(jsonPath("$.bundleContentType").value(DEFAULT_BUNDLE_CONTENT_TYPE))
            .andExpect(jsonPath("$.bundle").value(Base64Utils.encodeToString(DEFAULT_BUNDLE)));
    }

    @Test
    @Transactional
    void getNonExistingStrategyApplication() throws Exception {
        // Get the strategyApplication
        restStrategyApplicationMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putNewStrategyApplication() throws Exception {
        // Initialize the database
        strategyApplicationRepository.saveAndFlush(strategyApplication);

        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();

        // Update the strategyApplication
        StrategyApplication updatedStrategyApplication = strategyApplicationRepository.findById(strategyApplication.getId()).get();
        // Disconnect from session so that the updates on updatedStrategyApplication are not directly saved in db
        em.detach(updatedStrategyApplication);
        updatedStrategyApplication
            .timestamp(UPDATED_TIMESTAMP)
            .summary(UPDATED_SUMMARY)
            .bundle(UPDATED_BUNDLE)
            .bundleContentType(UPDATED_BUNDLE_CONTENT_TYPE);

        restStrategyApplicationMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedStrategyApplication.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(updatedStrategyApplication))
            )
            .andExpect(status().isOk());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
        StrategyApplication testStrategyApplication = strategyApplicationList.get(strategyApplicationList.size() - 1);
        assertThat(testStrategyApplication.getTimestamp()).isEqualTo(UPDATED_TIMESTAMP);
        assertThat(testStrategyApplication.getSummary()).isEqualTo(UPDATED_SUMMARY);
        assertThat(testStrategyApplication.getBundle()).isEqualTo(UPDATED_BUNDLE);
        assertThat(testStrategyApplication.getBundleContentType()).isEqualTo(UPDATED_BUNDLE_CONTENT_TYPE);
    }

    @Test
    @Transactional
    void putNonExistingStrategyApplication() throws Exception {
        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();
        strategyApplication.setId(count.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restStrategyApplicationMockMvc
            .perform(
                put(ENTITY_API_URL_ID, strategyApplication.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(strategyApplication))
            )
            .andExpect(status().isBadRequest());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchStrategyApplication() throws Exception {
        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();
        strategyApplication.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restStrategyApplicationMockMvc
            .perform(
                put(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(strategyApplication))
            )
            .andExpect(status().isBadRequest());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamStrategyApplication() throws Exception {
        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();
        strategyApplication.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restStrategyApplicationMockMvc
            .perform(
                put(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(strategyApplication))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateStrategyApplicationWithPatch() throws Exception {
        // Initialize the database
        strategyApplicationRepository.saveAndFlush(strategyApplication);

        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();

        // Update the strategyApplication using partial update
        StrategyApplication partialUpdatedStrategyApplication = new StrategyApplication();
        partialUpdatedStrategyApplication.setId(strategyApplication.getId());

        partialUpdatedStrategyApplication.timestamp(UPDATED_TIMESTAMP).summary(UPDATED_SUMMARY);

        restStrategyApplicationMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedStrategyApplication.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedStrategyApplication))
            )
            .andExpect(status().isOk());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
        StrategyApplication testStrategyApplication = strategyApplicationList.get(strategyApplicationList.size() - 1);
        assertThat(testStrategyApplication.getTimestamp()).isEqualTo(UPDATED_TIMESTAMP);
        assertThat(testStrategyApplication.getSummary()).isEqualTo(UPDATED_SUMMARY);
        assertThat(testStrategyApplication.getBundle()).isEqualTo(DEFAULT_BUNDLE);
        assertThat(testStrategyApplication.getBundleContentType()).isEqualTo(DEFAULT_BUNDLE_CONTENT_TYPE);
    }

    @Test
    @Transactional
    void fullUpdateStrategyApplicationWithPatch() throws Exception {
        // Initialize the database
        strategyApplicationRepository.saveAndFlush(strategyApplication);

        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();

        // Update the strategyApplication using partial update
        StrategyApplication partialUpdatedStrategyApplication = new StrategyApplication();
        partialUpdatedStrategyApplication.setId(strategyApplication.getId());

        partialUpdatedStrategyApplication
            .timestamp(UPDATED_TIMESTAMP)
            .summary(UPDATED_SUMMARY)
            .bundle(UPDATED_BUNDLE)
            .bundleContentType(UPDATED_BUNDLE_CONTENT_TYPE);

        restStrategyApplicationMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedStrategyApplication.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedStrategyApplication))
            )
            .andExpect(status().isOk());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
        StrategyApplication testStrategyApplication = strategyApplicationList.get(strategyApplicationList.size() - 1);
        assertThat(testStrategyApplication.getTimestamp()).isEqualTo(UPDATED_TIMESTAMP);
        assertThat(testStrategyApplication.getSummary()).isEqualTo(UPDATED_SUMMARY);
        assertThat(testStrategyApplication.getBundle()).isEqualTo(UPDATED_BUNDLE);
        assertThat(testStrategyApplication.getBundleContentType()).isEqualTo(UPDATED_BUNDLE_CONTENT_TYPE);
    }

    @Test
    @Transactional
    void patchNonExistingStrategyApplication() throws Exception {
        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();
        strategyApplication.setId(count.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restStrategyApplicationMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, strategyApplication.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(strategyApplication))
            )
            .andExpect(status().isBadRequest());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchStrategyApplication() throws Exception {
        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();
        strategyApplication.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restStrategyApplicationMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(strategyApplication))
            )
            .andExpect(status().isBadRequest());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamStrategyApplication() throws Exception {
        int databaseSizeBeforeUpdate = strategyApplicationRepository.findAll().size();
        strategyApplication.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restStrategyApplicationMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(strategyApplication))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the StrategyApplication in the database
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteStrategyApplication() throws Exception {
        // Initialize the database
        strategyApplicationRepository.saveAndFlush(strategyApplication);

        int databaseSizeBeforeDelete = strategyApplicationRepository.findAll().size();

        // Delete the strategyApplication
        restStrategyApplicationMockMvc
            .perform(delete(ENTITY_API_URL_ID, strategyApplication.getId()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<StrategyApplication> strategyApplicationList = strategyApplicationRepository.findAll();
        assertThat(strategyApplicationList).hasSize(databaseSizeBeforeDelete - 1);
    }
}
