package com.piacere.selfhealing.backend.web.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.piacere.selfhealing.backend.IntegrationTest;
import com.piacere.selfhealing.backend.domain.Strategy;
import com.piacere.selfhealing.backend.repository.StrategyRepository;
import java.util.List;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import javax.persistence.EntityManager;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Base64Utils;

/**
 * Integration tests for the {@link StrategyResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
class StrategyResourceIT {

    private static final String DEFAULT_CODE = "AAAAAAAAAA";
    private static final String UPDATED_CODE = "BBBBBBBBBB";

    private static final String DEFAULT_DESCRIPTION = "AAAAAAAAAA";
    private static final String UPDATED_DESCRIPTION = "BBBBBBBBBB";

    private static final String DEFAULT_CONFIGURATION = "AAAAAAAAAA";
    private static final String UPDATED_CONFIGURATION = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/strategies";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong count = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private StrategyRepository strategyRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restStrategyMockMvc;

    private Strategy strategy;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Strategy createEntity(EntityManager em) {
        Strategy strategy = new Strategy().code(DEFAULT_CODE).description(DEFAULT_DESCRIPTION).configuration(DEFAULT_CONFIGURATION);
        return strategy;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Strategy createUpdatedEntity(EntityManager em) {
        Strategy strategy = new Strategy().code(UPDATED_CODE).description(UPDATED_DESCRIPTION).configuration(UPDATED_CONFIGURATION);
        return strategy;
    }

    @BeforeEach
    public void initTest() {
        strategy = createEntity(em);
    }

    @Test
    @Transactional
    void createStrategy() throws Exception {
        int databaseSizeBeforeCreate = strategyRepository.findAll().size();
        // Create the Strategy
        restStrategyMockMvc
            .perform(post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(strategy)))
            .andExpect(status().isCreated());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeCreate + 1);
        Strategy testStrategy = strategyList.get(strategyList.size() - 1);
        assertThat(testStrategy.getCode()).isEqualTo(DEFAULT_CODE);
        assertThat(testStrategy.getDescription()).isEqualTo(DEFAULT_DESCRIPTION);
        assertThat(testStrategy.getConfiguration()).isEqualTo(DEFAULT_CONFIGURATION);
    }

    @Test
    @Transactional
    void createStrategyWithExistingId() throws Exception {
        // Create the Strategy with an existing ID
        strategy.setId(1L);

        int databaseSizeBeforeCreate = strategyRepository.findAll().size();

        // An entity with an existing ID cannot be created, so this API call must fail
        restStrategyMockMvc
            .perform(post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(strategy)))
            .andExpect(status().isBadRequest());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkCodeIsRequired() throws Exception {
        int databaseSizeBeforeTest = strategyRepository.findAll().size();
        // set the field null
        strategy.setCode(null);

        // Create the Strategy, which fails.

        restStrategyMockMvc
            .perform(post(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(strategy)))
            .andExpect(status().isBadRequest());

        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllStrategies() throws Exception {
        // Initialize the database
        strategyRepository.saveAndFlush(strategy);

        // Get all the strategyList
        restStrategyMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(strategy.getId().intValue())))
            .andExpect(jsonPath("$.[*].code").value(hasItem(DEFAULT_CODE)))
            .andExpect(jsonPath("$.[*].description").value(hasItem(DEFAULT_DESCRIPTION)))
            .andExpect(jsonPath("$.[*].configuration").value(hasItem(DEFAULT_CONFIGURATION.toString())));
    }

    @Test
    @Transactional
    void getStrategy() throws Exception {
        // Initialize the database
        strategyRepository.saveAndFlush(strategy);

        // Get the strategy
        restStrategyMockMvc
            .perform(get(ENTITY_API_URL_ID, strategy.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(strategy.getId().intValue()))
            .andExpect(jsonPath("$.code").value(DEFAULT_CODE))
            .andExpect(jsonPath("$.description").value(DEFAULT_DESCRIPTION))
            .andExpect(jsonPath("$.configuration").value(DEFAULT_CONFIGURATION.toString()));
    }

    @Test
    @Transactional
    void getNonExistingStrategy() throws Exception {
        // Get the strategy
        restStrategyMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putNewStrategy() throws Exception {
        // Initialize the database
        strategyRepository.saveAndFlush(strategy);

        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();

        // Update the strategy
        Strategy updatedStrategy = strategyRepository.findById(strategy.getId()).get();
        // Disconnect from session so that the updates on updatedStrategy are not directly saved in db
        em.detach(updatedStrategy);
        updatedStrategy.code(UPDATED_CODE).description(UPDATED_DESCRIPTION).configuration(UPDATED_CONFIGURATION);

        restStrategyMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedStrategy.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(updatedStrategy))
            )
            .andExpect(status().isOk());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
        Strategy testStrategy = strategyList.get(strategyList.size() - 1);
        assertThat(testStrategy.getCode()).isEqualTo(UPDATED_CODE);
        assertThat(testStrategy.getDescription()).isEqualTo(UPDATED_DESCRIPTION);
        assertThat(testStrategy.getConfiguration()).isEqualTo(UPDATED_CONFIGURATION);
    }

    @Test
    @Transactional
    void putNonExistingStrategy() throws Exception {
        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();
        strategy.setId(count.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restStrategyMockMvc
            .perform(
                put(ENTITY_API_URL_ID, strategy.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(strategy))
            )
            .andExpect(status().isBadRequest());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchStrategy() throws Exception {
        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();
        strategy.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restStrategyMockMvc
            .perform(
                put(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(strategy))
            )
            .andExpect(status().isBadRequest());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamStrategy() throws Exception {
        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();
        strategy.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restStrategyMockMvc
            .perform(put(ENTITY_API_URL).contentType(MediaType.APPLICATION_JSON).content(TestUtil.convertObjectToJsonBytes(strategy)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateStrategyWithPatch() throws Exception {
        // Initialize the database
        strategyRepository.saveAndFlush(strategy);

        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();

        // Update the strategy using partial update
        Strategy partialUpdatedStrategy = new Strategy();
        partialUpdatedStrategy.setId(strategy.getId());

        restStrategyMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedStrategy.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedStrategy))
            )
            .andExpect(status().isOk());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
        Strategy testStrategy = strategyList.get(strategyList.size() - 1);
        assertThat(testStrategy.getCode()).isEqualTo(DEFAULT_CODE);
        assertThat(testStrategy.getDescription()).isEqualTo(DEFAULT_DESCRIPTION);
        assertThat(testStrategy.getConfiguration()).isEqualTo(DEFAULT_CONFIGURATION);
    }

    @Test
    @Transactional
    void fullUpdateStrategyWithPatch() throws Exception {
        // Initialize the database
        strategyRepository.saveAndFlush(strategy);

        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();

        // Update the strategy using partial update
        Strategy partialUpdatedStrategy = new Strategy();
        partialUpdatedStrategy.setId(strategy.getId());

        partialUpdatedStrategy.code(UPDATED_CODE).description(UPDATED_DESCRIPTION).configuration(UPDATED_CONFIGURATION);

        restStrategyMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedStrategy.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedStrategy))
            )
            .andExpect(status().isOk());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
        Strategy testStrategy = strategyList.get(strategyList.size() - 1);
        assertThat(testStrategy.getCode()).isEqualTo(UPDATED_CODE);
        assertThat(testStrategy.getDescription()).isEqualTo(UPDATED_DESCRIPTION);
        assertThat(testStrategy.getConfiguration()).isEqualTo(UPDATED_CONFIGURATION);
    }

    @Test
    @Transactional
    void patchNonExistingStrategy() throws Exception {
        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();
        strategy.setId(count.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restStrategyMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, strategy.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(strategy))
            )
            .andExpect(status().isBadRequest());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchStrategy() throws Exception {
        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();
        strategy.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restStrategyMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(strategy))
            )
            .andExpect(status().isBadRequest());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamStrategy() throws Exception {
        int databaseSizeBeforeUpdate = strategyRepository.findAll().size();
        strategy.setId(count.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restStrategyMockMvc
            .perform(patch(ENTITY_API_URL).contentType("application/merge-patch+json").content(TestUtil.convertObjectToJsonBytes(strategy)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the Strategy in the database
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteStrategy() throws Exception {
        // Initialize the database
        strategyRepository.saveAndFlush(strategy);

        int databaseSizeBeforeDelete = strategyRepository.findAll().size();

        // Delete the strategy
        restStrategyMockMvc
            .perform(delete(ENTITY_API_URL_ID, strategy.getId()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<Strategy> strategyList = strategyRepository.findAll();
        assertThat(strategyList).hasSize(databaseSizeBeforeDelete - 1);
    }
}
