import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IDeployment, Deployment } from '../deployment.model';

import { DeploymentService } from './deployment.service';

describe('Service Tests', () => {
  describe('Deployment Service', () => {
    let service: DeploymentService;
    let httpMock: HttpTestingController;
    let elemDefault: IDeployment;
    let expectedResult: IDeployment | IDeployment[] | boolean | null;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
      });
      expectedResult = null;
      service = TestBed.inject(DeploymentService);
      httpMock = TestBed.inject(HttpTestingController);

      elemDefault = {
        id: 0,
        deploymentId: 'AAAAAAA',
        bundleContentType: 'image/png',
        bundle: 'AAAAAAA',
      };
    });

    describe('Service methods', () => {
      it('should find an element', () => {
        const returnedFromService = Object.assign({}, elemDefault);

        service.find(123).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(elemDefault);
      });

      it('should create a Deployment', () => {
        const returnedFromService = Object.assign(
          {
            id: 0,
          },
          elemDefault
        );

        const expected = Object.assign({}, returnedFromService);

        service.create(new Deployment()).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'POST' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should update a Deployment', () => {
        const returnedFromService = Object.assign(
          {
            id: 1,
            deploymentId: 'BBBBBB',
            bundle: 'BBBBBB',
          },
          elemDefault
        );

        const expected = Object.assign({}, returnedFromService);

        service.update(expected).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PUT' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should partial update a Deployment', () => {
        const patchObject = Object.assign(
          {
            bundle: 'BBBBBB',
          },
          new Deployment()
        );

        const returnedFromService = Object.assign(patchObject, elemDefault);

        const expected = Object.assign({}, returnedFromService);

        service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PATCH' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should return a list of Deployment', () => {
        const returnedFromService = Object.assign(
          {
            id: 1,
            deploymentId: 'BBBBBB',
            bundle: 'BBBBBB',
          },
          elemDefault
        );

        const expected = Object.assign({}, returnedFromService);

        service.query().subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush([returnedFromService]);
        httpMock.verify();
        expect(expectedResult).toContainEqual(expected);
      });

      it('should delete a Deployment', () => {
        service.delete(123).subscribe(resp => (expectedResult = resp.ok));

        const req = httpMock.expectOne({ method: 'DELETE' });
        req.flush({ status: 200 });
        expect(expectedResult);
      });

      describe('addDeploymentToCollectionIfMissing', () => {
        it('should add a Deployment to an empty array', () => {
          const deployment: IDeployment = { id: 123 };
          expectedResult = service.addDeploymentToCollectionIfMissing([], deployment);
          expect(expectedResult).toHaveLength(1);
          expect(expectedResult).toContain(deployment);
        });

        it('should not add a Deployment to an array that contains it', () => {
          const deployment: IDeployment = { id: 123 };
          const deploymentCollection: IDeployment[] = [
            {
              ...deployment,
            },
            { id: 456 },
          ];
          expectedResult = service.addDeploymentToCollectionIfMissing(deploymentCollection, deployment);
          expect(expectedResult).toHaveLength(2);
        });

        it("should add a Deployment to an array that doesn't contain it", () => {
          const deployment: IDeployment = { id: 123 };
          const deploymentCollection: IDeployment[] = [{ id: 456 }];
          expectedResult = service.addDeploymentToCollectionIfMissing(deploymentCollection, deployment);
          expect(expectedResult).toHaveLength(2);
          expect(expectedResult).toContain(deployment);
        });

        it('should add only unique Deployment to an array', () => {
          const deploymentArray: IDeployment[] = [{ id: 123 }, { id: 456 }, { id: 66697 }];
          const deploymentCollection: IDeployment[] = [{ id: 123 }];
          expectedResult = service.addDeploymentToCollectionIfMissing(deploymentCollection, ...deploymentArray);
          expect(expectedResult).toHaveLength(3);
        });

        it('should accept varargs', () => {
          const deployment: IDeployment = { id: 123 };
          const deployment2: IDeployment = { id: 456 };
          expectedResult = service.addDeploymentToCollectionIfMissing([], deployment, deployment2);
          expect(expectedResult).toHaveLength(2);
          expect(expectedResult).toContain(deployment);
          expect(expectedResult).toContain(deployment2);
        });

        it('should accept null and undefined values', () => {
          const deployment: IDeployment = { id: 123 };
          expectedResult = service.addDeploymentToCollectionIfMissing([], null, deployment, undefined);
          expect(expectedResult).toHaveLength(1);
          expect(expectedResult).toContain(deployment);
        });

        it('should return initial array if no Deployment is added', () => {
          const deploymentCollection: IDeployment[] = [{ id: 123 }];
          expectedResult = service.addDeploymentToCollectionIfMissing(deploymentCollection, undefined, null);
          expect(expectedResult).toEqual(deploymentCollection);
        });
      });
    });

    afterEach(() => {
      httpMock.verify();
    });
  });
});
