import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { isPresent } from 'app/core/util/operators';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption } from 'app/core/request/request-util';
import { IDeployment, getDeploymentIdentifier } from '../deployment.model';

export type EntityResponseType = HttpResponse<IDeployment>;
export type EntityArrayResponseType = HttpResponse<IDeployment[]>;

@Injectable({ providedIn: 'root' })
export class DeploymentService {
  protected resourceUrl = this.applicationConfigService.getEndpointFor('api/deployments', 'shbackend');

  constructor(protected http: HttpClient, protected applicationConfigService: ApplicationConfigService) {}

  create(deployment: IDeployment): Observable<EntityResponseType> {
    return this.http.post<IDeployment>(this.resourceUrl, deployment, { observe: 'response' });
  }

  update(deployment: IDeployment): Observable<EntityResponseType> {
    return this.http.put<IDeployment>(`${this.resourceUrl}/${getDeploymentIdentifier(deployment) as number}`, deployment, {
      observe: 'response',
    });
  }

  partialUpdate(deployment: IDeployment): Observable<EntityResponseType> {
    return this.http.patch<IDeployment>(`${this.resourceUrl}/${getDeploymentIdentifier(deployment) as number}`, deployment, {
      observe: 'response',
    });
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http.get<IDeployment>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IDeployment[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  addDeploymentToCollectionIfMissing(
    deploymentCollection: IDeployment[],
    ...deploymentsToCheck: (IDeployment | null | undefined)[]
  ): IDeployment[] {
    const deployments: IDeployment[] = deploymentsToCheck.filter(isPresent);
    if (deployments.length > 0) {
      const deploymentCollectionIdentifiers = deploymentCollection.map(deploymentItem => getDeploymentIdentifier(deploymentItem)!);
      const deploymentsToAdd = deployments.filter(deploymentItem => {
        const deploymentIdentifier = getDeploymentIdentifier(deploymentItem);
        if (deploymentIdentifier == null || deploymentCollectionIdentifiers.includes(deploymentIdentifier)) {
          return false;
        }
        deploymentCollectionIdentifiers.push(deploymentIdentifier);
        return true;
      });
      return [...deploymentsToAdd, ...deploymentCollection];
    }
    return deploymentCollection;
  }
}
