import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';

import { IDeployment, Deployment } from '../deployment.model';
import { DeploymentService } from '../service/deployment.service';
import { AlertError } from 'app/shared/alert/alert-error.model';
import { EventManager, EventWithContent } from 'app/core/util/event-manager.service';
import { DataUtils, FileLoadError } from 'app/core/util/data-util.service';

@Component({
  selector: 'jhi-deployment-update',
  templateUrl: './deployment-update.component.html',
})
export class DeploymentUpdateComponent implements OnInit {
  isSaving = false;

  editForm = this.fb.group({
    id: [],
    deploymentId: [null, [Validators.required]],
    bundle: [null, [Validators.required]],
    bundleContentType: [],
  });

  constructor(
    protected dataUtils: DataUtils,
    protected eventManager: EventManager,
    protected deploymentService: DeploymentService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ deployment }) => {
      this.updateForm(deployment);
    });
  }

  byteSize(base64String: string): string {
    return this.dataUtils.byteSize(base64String);
  }

  openFile(base64String: string, contentType: string | null | undefined): void {
    this.dataUtils.openFile(base64String, contentType);
  }

  setFileData(event: Event, field: string, isImage: boolean): void {
    this.dataUtils.loadFileToForm(event, this.editForm, field, isImage).subscribe({
      error: (err: FileLoadError) =>
        this.eventManager.broadcast(new EventWithContent<AlertError>('shGatewayApp.error', { ...err, key: 'error.file.' + err.key })),
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const deployment = this.createFromForm();
    if (deployment.id !== undefined) {
      this.subscribeToSaveResponse(this.deploymentService.update(deployment));
    } else {
      this.subscribeToSaveResponse(this.deploymentService.create(deployment));
    }
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IDeployment>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(deployment: IDeployment): void {
    this.editForm.patchValue({
      id: deployment.id,
      deploymentId: deployment.deploymentId,
      bundle: deployment.bundle,
      bundleContentType: deployment.bundleContentType,
    });
  }

  protected createFromForm(): IDeployment {
    return {
      ...new Deployment(),
      id: this.editForm.get(['id'])!.value,
      deploymentId: this.editForm.get(['deploymentId'])!.value,
      bundleContentType: this.editForm.get(['bundleContentType'])!.value,
      bundle: this.editForm.get(['bundle'])!.value,
    };
  }
}
