jest.mock('@angular/router');

import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { of, Subject } from 'rxjs';

import { EventTypeService } from '../service/event-type.service';
import { IEventType, EventType } from '../event-type.model';
import { IStrategy } from 'app/entities/shBackend/strategy/strategy.model';
import { StrategyService } from 'app/entities/shBackend/strategy/service/strategy.service';
import { IDeployment } from 'app/entities/shBackend/deployment/deployment.model';
import { DeploymentService } from 'app/entities/shBackend/deployment/service/deployment.service';

import { EventTypeUpdateComponent } from './event-type-update.component';

describe('Component Tests', () => {
  describe('EventType Management Update Component', () => {
    let comp: EventTypeUpdateComponent;
    let fixture: ComponentFixture<EventTypeUpdateComponent>;
    let activatedRoute: ActivatedRoute;
    let eventTypeService: EventTypeService;
    let strategyService: StrategyService;
    let deploymentService: DeploymentService;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        declarations: [EventTypeUpdateComponent],
        providers: [FormBuilder, ActivatedRoute],
      })
        .overrideTemplate(EventTypeUpdateComponent, '')
        .compileComponents();

      fixture = TestBed.createComponent(EventTypeUpdateComponent);
      activatedRoute = TestBed.inject(ActivatedRoute);
      eventTypeService = TestBed.inject(EventTypeService);
      strategyService = TestBed.inject(StrategyService);
      deploymentService = TestBed.inject(DeploymentService);

      comp = fixture.componentInstance;
    });

    describe('ngOnInit', () => {
      it('Should call Strategy query and add missing value', () => {
        const eventType: IEventType = { id: 456 };
        const strategy: IStrategy = { id: 72204 };
        eventType.strategy = strategy;

        const strategyCollection: IStrategy[] = [{ id: 98359 }];
        jest.spyOn(strategyService, 'query').mockReturnValue(of(new HttpResponse({ body: strategyCollection })));
        const additionalStrategies = [strategy];
        const expectedCollection: IStrategy[] = [...additionalStrategies, ...strategyCollection];
        jest.spyOn(strategyService, 'addStrategyToCollectionIfMissing').mockReturnValue(expectedCollection);

        activatedRoute.data = of({ eventType });
        comp.ngOnInit();

        expect(strategyService.query).toHaveBeenCalled();
        expect(strategyService.addStrategyToCollectionIfMissing).toHaveBeenCalledWith(strategyCollection, ...additionalStrategies);
        expect(comp.strategiesSharedCollection).toEqual(expectedCollection);
      });

      it('Should call Deployment query and add missing value', () => {
        const eventType: IEventType = { id: 456 };
        const deployment: IDeployment = { id: 97496 };
        eventType.deployment = deployment;

        const deploymentCollection: IDeployment[] = [{ id: 10014 }];
        jest.spyOn(deploymentService, 'query').mockReturnValue(of(new HttpResponse({ body: deploymentCollection })));
        const additionalDeployments = [deployment];
        const expectedCollection: IDeployment[] = [...additionalDeployments, ...deploymentCollection];
        jest.spyOn(deploymentService, 'addDeploymentToCollectionIfMissing').mockReturnValue(expectedCollection);

        activatedRoute.data = of({ eventType });
        comp.ngOnInit();

        expect(deploymentService.query).toHaveBeenCalled();
        expect(deploymentService.addDeploymentToCollectionIfMissing).toHaveBeenCalledWith(deploymentCollection, ...additionalDeployments);
        expect(comp.deploymentsSharedCollection).toEqual(expectedCollection);
      });

      it('Should update editForm', () => {
        const eventType: IEventType = { id: 456 };
        const strategy: IStrategy = { id: 96706 };
        eventType.strategy = strategy;
        const deployment: IDeployment = { id: 35030 };
        eventType.deployment = deployment;

        activatedRoute.data = of({ eventType });
        comp.ngOnInit();

        expect(comp.editForm.value).toEqual(expect.objectContaining(eventType));
        expect(comp.strategiesSharedCollection).toContain(strategy);
        expect(comp.deploymentsSharedCollection).toContain(deployment);
      });
    });

    describe('save', () => {
      it('Should call update service on save for existing entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<EventType>>();
        const eventType = { id: 123 };
        jest.spyOn(eventTypeService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ eventType });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: eventType }));
        saveSubject.complete();

        // THEN
        expect(comp.previousState).toHaveBeenCalled();
        expect(eventTypeService.update).toHaveBeenCalledWith(eventType);
        expect(comp.isSaving).toEqual(false);
      });

      it('Should call create service on save for new entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<EventType>>();
        const eventType = new EventType();
        jest.spyOn(eventTypeService, 'create').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ eventType });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: eventType }));
        saveSubject.complete();

        // THEN
        expect(eventTypeService.create).toHaveBeenCalledWith(eventType);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).toHaveBeenCalled();
      });

      it('Should set isSaving to false on error', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<EventType>>();
        const eventType = { id: 123 };
        jest.spyOn(eventTypeService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ eventType });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.error('This is an error!');

        // THEN
        expect(eventTypeService.update).toHaveBeenCalledWith(eventType);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).not.toHaveBeenCalled();
      });
    });

    describe('Tracking relationships identifiers', () => {
      describe('trackStrategyById', () => {
        it('Should return tracked Strategy primary key', () => {
          const entity = { id: 123 };
          const trackResult = comp.trackStrategyById(0, entity);
          expect(trackResult).toEqual(entity.id);
        });
      });

      describe('trackDeploymentById', () => {
        it('Should return tracked Deployment primary key', () => {
          const entity = { id: 123 };
          const trackResult = comp.trackDeploymentById(0, entity);
          expect(trackResult).toEqual(entity.id);
        });
      });
    });
  });
});
