import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize, map } from 'rxjs/operators';

import { IEventType, EventType } from '../event-type.model';
import { EventTypeService } from '../service/event-type.service';
import { IStrategy } from 'app/entities/shBackend/strategy/strategy.model';
import { StrategyService } from 'app/entities/shBackend/strategy/service/strategy.service';
import { IDeployment } from 'app/entities/shBackend/deployment/deployment.model';
import { DeploymentService } from 'app/entities/shBackend/deployment/service/deployment.service';

@Component({
  selector: 'jhi-event-type-update',
  templateUrl: './event-type-update.component.html',
})
export class EventTypeUpdateComponent implements OnInit {
  isSaving = false;

  strategiesSharedCollection: IStrategy[] = [];
  deploymentsSharedCollection: IDeployment[] = [];

  editForm = this.fb.group({
    id: [],
    code: [null, [Validators.required]],
    description: [],
    strategy: [],
    deployment: [],
  });

  constructor(
    protected eventTypeService: EventTypeService,
    protected strategyService: StrategyService,
    protected deploymentService: DeploymentService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ eventType }) => {
      this.updateForm(eventType);

      this.loadRelationshipsOptions();
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const eventType = this.createFromForm();
    if (eventType.id !== undefined) {
      this.subscribeToSaveResponse(this.eventTypeService.update(eventType));
    } else {
      this.subscribeToSaveResponse(this.eventTypeService.create(eventType));
    }
  }

  trackStrategyById(index: number, item: IStrategy): number {
    return item.id!;
  }

  trackDeploymentById(index: number, item: IDeployment): number {
    return item.id!;
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IEventType>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(eventType: IEventType): void {
    this.editForm.patchValue({
      id: eventType.id,
      code: eventType.code,
      description: eventType.description,
      strategy: eventType.strategy,
      deployment: eventType.deployment,
    });

    this.strategiesSharedCollection = this.strategyService.addStrategyToCollectionIfMissing(
      this.strategiesSharedCollection,
      eventType.strategy
    );
    this.deploymentsSharedCollection = this.deploymentService.addDeploymentToCollectionIfMissing(
      this.deploymentsSharedCollection,
      eventType.deployment
    );
  }

  protected loadRelationshipsOptions(): void {
    this.strategyService
      .query()
      .pipe(map((res: HttpResponse<IStrategy[]>) => res.body ?? []))
      .pipe(
        map((strategies: IStrategy[]) =>
          this.strategyService.addStrategyToCollectionIfMissing(strategies, this.editForm.get('strategy')!.value)
        )
      )
      .subscribe((strategies: IStrategy[]) => (this.strategiesSharedCollection = strategies));

    this.deploymentService
      .query()
      .pipe(map((res: HttpResponse<IDeployment[]>) => res.body ?? []))
      .pipe(
        map((deployments: IDeployment[]) =>
          this.deploymentService.addDeploymentToCollectionIfMissing(deployments, this.editForm.get('deployment')!.value)
        )
      )
      .subscribe((deployments: IDeployment[]) => (this.deploymentsSharedCollection = deployments));
  }

  protected createFromForm(): IEventType {
    return {
      ...new EventType(),
      id: this.editForm.get(['id'])!.value,
      code: this.editForm.get(['code'])!.value,
      description: this.editForm.get(['description'])!.value,
      strategy: this.editForm.get(['strategy'])!.value,
      deployment: this.editForm.get(['deployment'])!.value,
    };
  }
}
