import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import * as dayjs from 'dayjs';

import { DATE_TIME_FORMAT } from 'app/config/input.constants';
import { Status } from 'app/entities/enumerations/status.model';
import { ISelfHealingMessage, SelfHealingMessage } from '../self-healing-message.model';

import { SelfHealingMessageService } from './self-healing-message.service';

describe('Service Tests', () => {
  describe('SelfHealingMessage Service', () => {
    let service: SelfHealingMessageService;
    let httpMock: HttpTestingController;
    let elemDefault: ISelfHealingMessage;
    let expectedResult: ISelfHealingMessage | ISelfHealingMessage[] | boolean | null;
    let currentDate: dayjs.Dayjs;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
      });
      expectedResult = null;
      service = TestBed.inject(SelfHealingMessageService);
      httpMock = TestBed.inject(HttpTestingController);
      currentDate = dayjs();

      elemDefault = {
        id: 0,
        timestamp: currentDate,
        origin: 'AAAAAAA',
        deploymentId: 'AAAAAAA',
        status: Status.PENDING,
        error: 'AAAAAAA',
      };
    });

    describe('Service methods', () => {
      it('should find an element', () => {
        const returnedFromService = Object.assign(
          {
            timestamp: currentDate.format(DATE_TIME_FORMAT),
          },
          elemDefault
        );

        service.find(123).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(elemDefault);
      });

      it('should create a SelfHealingMessage', () => {
        const returnedFromService = Object.assign(
          {
            id: 0,
            timestamp: currentDate.format(DATE_TIME_FORMAT),
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            timestamp: currentDate,
          },
          returnedFromService
        );

        service.create(new SelfHealingMessage()).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'POST' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should update a SelfHealingMessage', () => {
        const returnedFromService = Object.assign(
          {
            id: 1,
            timestamp: currentDate.format(DATE_TIME_FORMAT),
            origin: 'BBBBBB',
            deploymentId: 'BBBBBB',
            status: 'BBBBBB',
            error: 'BBBBBB',
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            timestamp: currentDate,
          },
          returnedFromService
        );

        service.update(expected).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PUT' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should partial update a SelfHealingMessage', () => {
        const patchObject = Object.assign(
          {
            error: 'BBBBBB',
          },
          new SelfHealingMessage()
        );

        const returnedFromService = Object.assign(patchObject, elemDefault);

        const expected = Object.assign(
          {
            timestamp: currentDate,
          },
          returnedFromService
        );

        service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PATCH' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should return a list of SelfHealingMessage', () => {
        const returnedFromService = Object.assign(
          {
            id: 1,
            timestamp: currentDate.format(DATE_TIME_FORMAT),
            origin: 'BBBBBB',
            deploymentId: 'BBBBBB',
            status: 'BBBBBB',
            error: 'BBBBBB',
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            timestamp: currentDate,
          },
          returnedFromService
        );

        service.query().subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush([returnedFromService]);
        httpMock.verify();
        expect(expectedResult).toContainEqual(expected);
      });

      it('should delete a SelfHealingMessage', () => {
        service.delete(123).subscribe(resp => (expectedResult = resp.ok));

        const req = httpMock.expectOne({ method: 'DELETE' });
        req.flush({ status: 200 });
        expect(expectedResult);
      });

      describe('addSelfHealingMessageToCollectionIfMissing', () => {
        it('should add a SelfHealingMessage to an empty array', () => {
          const selfHealingMessage: ISelfHealingMessage = { id: 123 };
          expectedResult = service.addSelfHealingMessageToCollectionIfMissing([], selfHealingMessage);
          expect(expectedResult).toHaveLength(1);
          expect(expectedResult).toContain(selfHealingMessage);
        });

        it('should not add a SelfHealingMessage to an array that contains it', () => {
          const selfHealingMessage: ISelfHealingMessage = { id: 123 };
          const selfHealingMessageCollection: ISelfHealingMessage[] = [
            {
              ...selfHealingMessage,
            },
            { id: 456 },
          ];
          expectedResult = service.addSelfHealingMessageToCollectionIfMissing(selfHealingMessageCollection, selfHealingMessage);
          expect(expectedResult).toHaveLength(2);
        });

        it("should add a SelfHealingMessage to an array that doesn't contain it", () => {
          const selfHealingMessage: ISelfHealingMessage = { id: 123 };
          const selfHealingMessageCollection: ISelfHealingMessage[] = [{ id: 456 }];
          expectedResult = service.addSelfHealingMessageToCollectionIfMissing(selfHealingMessageCollection, selfHealingMessage);
          expect(expectedResult).toHaveLength(2);
          expect(expectedResult).toContain(selfHealingMessage);
        });

        it('should add only unique SelfHealingMessage to an array', () => {
          const selfHealingMessageArray: ISelfHealingMessage[] = [{ id: 123 }, { id: 456 }, { id: 18838 }];
          const selfHealingMessageCollection: ISelfHealingMessage[] = [{ id: 123 }];
          expectedResult = service.addSelfHealingMessageToCollectionIfMissing(selfHealingMessageCollection, ...selfHealingMessageArray);
          expect(expectedResult).toHaveLength(3);
        });

        it('should accept varargs', () => {
          const selfHealingMessage: ISelfHealingMessage = { id: 123 };
          const selfHealingMessage2: ISelfHealingMessage = { id: 456 };
          expectedResult = service.addSelfHealingMessageToCollectionIfMissing([], selfHealingMessage, selfHealingMessage2);
          expect(expectedResult).toHaveLength(2);
          expect(expectedResult).toContain(selfHealingMessage);
          expect(expectedResult).toContain(selfHealingMessage2);
        });

        it('should accept null and undefined values', () => {
          const selfHealingMessage: ISelfHealingMessage = { id: 123 };
          expectedResult = service.addSelfHealingMessageToCollectionIfMissing([], null, selfHealingMessage, undefined);
          expect(expectedResult).toHaveLength(1);
          expect(expectedResult).toContain(selfHealingMessage);
        });

        it('should return initial array if no SelfHealingMessage is added', () => {
          const selfHealingMessageCollection: ISelfHealingMessage[] = [{ id: 123 }];
          expectedResult = service.addSelfHealingMessageToCollectionIfMissing(selfHealingMessageCollection, undefined, null);
          expect(expectedResult).toEqual(selfHealingMessageCollection);
        });
      });
    });

    afterEach(() => {
      httpMock.verify();
    });
  });
});
