import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import * as dayjs from 'dayjs';

import { isPresent } from 'app/core/util/operators';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption } from 'app/core/request/request-util';
import { ISelfHealingMessage, getSelfHealingMessageIdentifier } from '../self-healing-message.model';

export type EntityResponseType = HttpResponse<ISelfHealingMessage>;
export type EntityArrayResponseType = HttpResponse<ISelfHealingMessage[]>;

@Injectable({ providedIn: 'root' })
export class SelfHealingMessageService {
  protected resourceUrl = this.applicationConfigService.getEndpointFor('api/self-healing-messages', 'shbackend');

  constructor(protected http: HttpClient, protected applicationConfigService: ApplicationConfigService) {}

  create(selfHealingMessage: ISelfHealingMessage): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(selfHealingMessage);
    return this.http
      .post<ISelfHealingMessage>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  update(selfHealingMessage: ISelfHealingMessage): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(selfHealingMessage);
    return this.http
      .put<ISelfHealingMessage>(`${this.resourceUrl}/${getSelfHealingMessageIdentifier(selfHealingMessage) as number}`, copy, {
        observe: 'response',
      })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  partialUpdate(selfHealingMessage: ISelfHealingMessage): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(selfHealingMessage);
    return this.http
      .patch<ISelfHealingMessage>(`${this.resourceUrl}/${getSelfHealingMessageIdentifier(selfHealingMessage) as number}`, copy, {
        observe: 'response',
      })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http
      .get<ISelfHealingMessage>(`${this.resourceUrl}/${id}`, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http
      .get<ISelfHealingMessage[]>(this.resourceUrl, { params: options, observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  addSelfHealingMessageToCollectionIfMissing(
    selfHealingMessageCollection: ISelfHealingMessage[],
    ...selfHealingMessagesToCheck: (ISelfHealingMessage | null | undefined)[]
  ): ISelfHealingMessage[] {
    const selfHealingMessages: ISelfHealingMessage[] = selfHealingMessagesToCheck.filter(isPresent);
    if (selfHealingMessages.length > 0) {
      const selfHealingMessageCollectionIdentifiers = selfHealingMessageCollection.map(
        selfHealingMessageItem => getSelfHealingMessageIdentifier(selfHealingMessageItem)!
      );
      const selfHealingMessagesToAdd = selfHealingMessages.filter(selfHealingMessageItem => {
        const selfHealingMessageIdentifier = getSelfHealingMessageIdentifier(selfHealingMessageItem);
        if (selfHealingMessageIdentifier == null || selfHealingMessageCollectionIdentifiers.includes(selfHealingMessageIdentifier)) {
          return false;
        }
        selfHealingMessageCollectionIdentifiers.push(selfHealingMessageIdentifier);
        return true;
      });
      return [...selfHealingMessagesToAdd, ...selfHealingMessageCollection];
    }
    return selfHealingMessageCollection;
  }

  protected convertDateFromClient(selfHealingMessage: ISelfHealingMessage): ISelfHealingMessage {
    return Object.assign({}, selfHealingMessage, {
      timestamp: selfHealingMessage.timestamp?.isValid() ? selfHealingMessage.timestamp.toJSON() : undefined,
    });
  }

  protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
    if (res.body) {
      res.body.timestamp = res.body.timestamp ? dayjs(res.body.timestamp) : undefined;
    }
    return res;
  }

  protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
    if (res.body) {
      res.body.forEach((selfHealingMessage: ISelfHealingMessage) => {
        selfHealingMessage.timestamp = selfHealingMessage.timestamp ? dayjs(selfHealingMessage.timestamp) : undefined;
      });
    }
    return res;
  }
}
