jest.mock('@angular/router');

import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { of, Subject } from 'rxjs';

import { SelfHealingMessageService } from '../service/self-healing-message.service';
import { ISelfHealingMessage, SelfHealingMessage } from '../self-healing-message.model';
import { IEventType } from 'app/entities/shBackend/event-type/event-type.model';
import { EventTypeService } from 'app/entities/shBackend/event-type/service/event-type.service';

import { SelfHealingMessageUpdateComponent } from './self-healing-message-update.component';

describe('Component Tests', () => {
  describe('SelfHealingMessage Management Update Component', () => {
    let comp: SelfHealingMessageUpdateComponent;
    let fixture: ComponentFixture<SelfHealingMessageUpdateComponent>;
    let activatedRoute: ActivatedRoute;
    let selfHealingMessageService: SelfHealingMessageService;
    let eventTypeService: EventTypeService;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        declarations: [SelfHealingMessageUpdateComponent],
        providers: [FormBuilder, ActivatedRoute],
      })
        .overrideTemplate(SelfHealingMessageUpdateComponent, '')
        .compileComponents();

      fixture = TestBed.createComponent(SelfHealingMessageUpdateComponent);
      activatedRoute = TestBed.inject(ActivatedRoute);
      selfHealingMessageService = TestBed.inject(SelfHealingMessageService);
      eventTypeService = TestBed.inject(EventTypeService);

      comp = fixture.componentInstance;
    });

    describe('ngOnInit', () => {
      it('Should call EventType query and add missing value', () => {
        const selfHealingMessage: ISelfHealingMessage = { id: 456 };
        const eventType: IEventType = { id: 52237 };
        selfHealingMessage.eventType = eventType;

        const eventTypeCollection: IEventType[] = [{ id: 88541 }];
        jest.spyOn(eventTypeService, 'query').mockReturnValue(of(new HttpResponse({ body: eventTypeCollection })));
        const additionalEventTypes = [eventType];
        const expectedCollection: IEventType[] = [...additionalEventTypes, ...eventTypeCollection];
        jest.spyOn(eventTypeService, 'addEventTypeToCollectionIfMissing').mockReturnValue(expectedCollection);

        activatedRoute.data = of({ selfHealingMessage });
        comp.ngOnInit();

        expect(eventTypeService.query).toHaveBeenCalled();
        expect(eventTypeService.addEventTypeToCollectionIfMissing).toHaveBeenCalledWith(eventTypeCollection, ...additionalEventTypes);
        expect(comp.eventTypesSharedCollection).toEqual(expectedCollection);
      });

      it('Should update editForm', () => {
        const selfHealingMessage: ISelfHealingMessage = { id: 456 };
        const eventType: IEventType = { id: 27873 };
        selfHealingMessage.eventType = eventType;

        activatedRoute.data = of({ selfHealingMessage });
        comp.ngOnInit();

        expect(comp.editForm.value).toEqual(expect.objectContaining(selfHealingMessage));
        expect(comp.eventTypesSharedCollection).toContain(eventType);
      });
    });

    describe('save', () => {
      it('Should call update service on save for existing entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<SelfHealingMessage>>();
        const selfHealingMessage = { id: 123 };
        jest.spyOn(selfHealingMessageService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ selfHealingMessage });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: selfHealingMessage }));
        saveSubject.complete();

        // THEN
        expect(comp.previousState).toHaveBeenCalled();
        expect(selfHealingMessageService.update).toHaveBeenCalledWith(selfHealingMessage);
        expect(comp.isSaving).toEqual(false);
      });

      it('Should call create service on save for new entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<SelfHealingMessage>>();
        const selfHealingMessage = new SelfHealingMessage();
        jest.spyOn(selfHealingMessageService, 'create').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ selfHealingMessage });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: selfHealingMessage }));
        saveSubject.complete();

        // THEN
        expect(selfHealingMessageService.create).toHaveBeenCalledWith(selfHealingMessage);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).toHaveBeenCalled();
      });

      it('Should set isSaving to false on error', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<SelfHealingMessage>>();
        const selfHealingMessage = { id: 123 };
        jest.spyOn(selfHealingMessageService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ selfHealingMessage });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.error('This is an error!');

        // THEN
        expect(selfHealingMessageService.update).toHaveBeenCalledWith(selfHealingMessage);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).not.toHaveBeenCalled();
      });
    });

    describe('Tracking relationships identifiers', () => {
      describe('trackEventTypeById', () => {
        it('Should return tracked EventType primary key', () => {
          const entity = { id: 123 };
          const trackResult = comp.trackEventTypeById(0, entity);
          expect(trackResult).toEqual(entity.id);
        });
      });
    });
  });
});
