import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize, map } from 'rxjs/operators';

import * as dayjs from 'dayjs';
import { DATE_TIME_FORMAT } from 'app/config/input.constants';

import { ISelfHealingMessage, SelfHealingMessage } from '../self-healing-message.model';
import { SelfHealingMessageService } from '../service/self-healing-message.service';
import { AlertError } from 'app/shared/alert/alert-error.model';
import { EventManager, EventWithContent } from 'app/core/util/event-manager.service';
import { DataUtils, FileLoadError } from 'app/core/util/data-util.service';
import { IEventType } from 'app/entities/shBackend/event-type/event-type.model';
import { EventTypeService } from 'app/entities/shBackend/event-type/service/event-type.service';

@Component({
  selector: 'jhi-self-healing-message-update',
  templateUrl: './self-healing-message-update.component.html',
})
export class SelfHealingMessageUpdateComponent implements OnInit {
  isSaving = false;

  eventTypesSharedCollection: IEventType[] = [];

  editForm = this.fb.group({
    id: [],
    timestamp: [],
    origin: [null, [Validators.required]],
    deploymentId: [],
    status: [],
    error: [],
    eventType: [],
  });

  constructor(
    protected dataUtils: DataUtils,
    protected eventManager: EventManager,
    protected selfHealingMessageService: SelfHealingMessageService,
    protected eventTypeService: EventTypeService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ selfHealingMessage }) => {
      if (selfHealingMessage.id === undefined) {
        const today = dayjs().startOf('day');
        selfHealingMessage.timestamp = today;
      }

      this.updateForm(selfHealingMessage);

      this.loadRelationshipsOptions();
    });
  }

  byteSize(base64String: string): string {
    return this.dataUtils.byteSize(base64String);
  }

  openFile(base64String: string, contentType: string | null | undefined): void {
    this.dataUtils.openFile(base64String, contentType);
  }

  setFileData(event: Event, field: string, isImage: boolean): void {
    this.dataUtils.loadFileToForm(event, this.editForm, field, isImage).subscribe({
      error: (err: FileLoadError) =>
        this.eventManager.broadcast(new EventWithContent<AlertError>('shGatewayApp.error', { ...err, key: 'error.file.' + err.key })),
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const selfHealingMessage = this.createFromForm();
    if (selfHealingMessage.id !== undefined) {
      this.subscribeToSaveResponse(this.selfHealingMessageService.update(selfHealingMessage));
    } else {
      this.subscribeToSaveResponse(this.selfHealingMessageService.create(selfHealingMessage));
    }
  }

  trackEventTypeById(index: number, item: IEventType): number {
    return item.id!;
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<ISelfHealingMessage>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(selfHealingMessage: ISelfHealingMessage): void {
    this.editForm.patchValue({
      id: selfHealingMessage.id,
      timestamp: selfHealingMessage.timestamp ? selfHealingMessage.timestamp.format(DATE_TIME_FORMAT) : null,
      origin: selfHealingMessage.origin,
      deploymentId: selfHealingMessage.deploymentId,
      status: selfHealingMessage.status,
      error: selfHealingMessage.error,
      eventType: selfHealingMessage.eventType,
    });

    this.eventTypesSharedCollection = this.eventTypeService.addEventTypeToCollectionIfMissing(
      this.eventTypesSharedCollection,
      selfHealingMessage.eventType
    );
  }

  protected loadRelationshipsOptions(): void {
    this.eventTypeService
      .query()
      .pipe(map((res: HttpResponse<IEventType[]>) => res.body ?? []))
      .pipe(
        map((eventTypes: IEventType[]) =>
          this.eventTypeService.addEventTypeToCollectionIfMissing(eventTypes, this.editForm.get('eventType')!.value)
        )
      )
      .subscribe((eventTypes: IEventType[]) => (this.eventTypesSharedCollection = eventTypes));
  }

  protected createFromForm(): ISelfHealingMessage {
    return {
      ...new SelfHealingMessage(),
      id: this.editForm.get(['id'])!.value,
      timestamp: this.editForm.get(['timestamp'])!.value ? dayjs(this.editForm.get(['timestamp'])!.value, DATE_TIME_FORMAT) : undefined,
      origin: this.editForm.get(['origin'])!.value,
      deploymentId: this.editForm.get(['deploymentId'])!.value,
      status: this.editForm.get(['status'])!.value,
      error: this.editForm.get(['error'])!.value,
      eventType: this.editForm.get(['eventType'])!.value,
    };
  }
}
