import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import * as dayjs from 'dayjs';

import { DATE_TIME_FORMAT } from 'app/config/input.constants';
import { IStrategyApplication, StrategyApplication } from '../strategy-application.model';

import { StrategyApplicationService } from './strategy-application.service';

describe('Service Tests', () => {
  describe('StrategyApplication Service', () => {
    let service: StrategyApplicationService;
    let httpMock: HttpTestingController;
    let elemDefault: IStrategyApplication;
    let expectedResult: IStrategyApplication | IStrategyApplication[] | boolean | null;
    let currentDate: dayjs.Dayjs;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
      });
      expectedResult = null;
      service = TestBed.inject(StrategyApplicationService);
      httpMock = TestBed.inject(HttpTestingController);
      currentDate = dayjs();

      elemDefault = {
        id: 0,
        timestamp: currentDate,
        summary: 'AAAAAAA',
        bundleContentType: 'image/png',
        bundle: 'AAAAAAA',
      };
    });

    describe('Service methods', () => {
      it('should find an element', () => {
        const returnedFromService = Object.assign(
          {
            timestamp: currentDate.format(DATE_TIME_FORMAT),
          },
          elemDefault
        );

        service.find(123).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(elemDefault);
      });

      it('should create a StrategyApplication', () => {
        const returnedFromService = Object.assign(
          {
            id: 0,
            timestamp: currentDate.format(DATE_TIME_FORMAT),
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            timestamp: currentDate,
          },
          returnedFromService
        );

        service.create(new StrategyApplication()).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'POST' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should update a StrategyApplication', () => {
        const returnedFromService = Object.assign(
          {
            id: 1,
            timestamp: currentDate.format(DATE_TIME_FORMAT),
            summary: 'BBBBBB',
            bundle: 'BBBBBB',
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            timestamp: currentDate,
          },
          returnedFromService
        );

        service.update(expected).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PUT' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should partial update a StrategyApplication', () => {
        const patchObject = Object.assign(
          {
            bundle: 'BBBBBB',
          },
          new StrategyApplication()
        );

        const returnedFromService = Object.assign(patchObject, elemDefault);

        const expected = Object.assign(
          {
            timestamp: currentDate,
          },
          returnedFromService
        );

        service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PATCH' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should return a list of StrategyApplication', () => {
        const returnedFromService = Object.assign(
          {
            id: 1,
            timestamp: currentDate.format(DATE_TIME_FORMAT),
            summary: 'BBBBBB',
            bundle: 'BBBBBB',
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            timestamp: currentDate,
          },
          returnedFromService
        );

        service.query().subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush([returnedFromService]);
        httpMock.verify();
        expect(expectedResult).toContainEqual(expected);
      });

      it('should delete a StrategyApplication', () => {
        service.delete(123).subscribe(resp => (expectedResult = resp.ok));

        const req = httpMock.expectOne({ method: 'DELETE' });
        req.flush({ status: 200 });
        expect(expectedResult);
      });

      describe('addStrategyApplicationToCollectionIfMissing', () => {
        it('should add a StrategyApplication to an empty array', () => {
          const strategyApplication: IStrategyApplication = { id: 123 };
          expectedResult = service.addStrategyApplicationToCollectionIfMissing([], strategyApplication);
          expect(expectedResult).toHaveLength(1);
          expect(expectedResult).toContain(strategyApplication);
        });

        it('should not add a StrategyApplication to an array that contains it', () => {
          const strategyApplication: IStrategyApplication = { id: 123 };
          const strategyApplicationCollection: IStrategyApplication[] = [
            {
              ...strategyApplication,
            },
            { id: 456 },
          ];
          expectedResult = service.addStrategyApplicationToCollectionIfMissing(strategyApplicationCollection, strategyApplication);
          expect(expectedResult).toHaveLength(2);
        });

        it("should add a StrategyApplication to an array that doesn't contain it", () => {
          const strategyApplication: IStrategyApplication = { id: 123 };
          const strategyApplicationCollection: IStrategyApplication[] = [{ id: 456 }];
          expectedResult = service.addStrategyApplicationToCollectionIfMissing(strategyApplicationCollection, strategyApplication);
          expect(expectedResult).toHaveLength(2);
          expect(expectedResult).toContain(strategyApplication);
        });

        it('should add only unique StrategyApplication to an array', () => {
          const strategyApplicationArray: IStrategyApplication[] = [{ id: 123 }, { id: 456 }, { id: 59740 }];
          const strategyApplicationCollection: IStrategyApplication[] = [{ id: 123 }];
          expectedResult = service.addStrategyApplicationToCollectionIfMissing(strategyApplicationCollection, ...strategyApplicationArray);
          expect(expectedResult).toHaveLength(3);
        });

        it('should accept varargs', () => {
          const strategyApplication: IStrategyApplication = { id: 123 };
          const strategyApplication2: IStrategyApplication = { id: 456 };
          expectedResult = service.addStrategyApplicationToCollectionIfMissing([], strategyApplication, strategyApplication2);
          expect(expectedResult).toHaveLength(2);
          expect(expectedResult).toContain(strategyApplication);
          expect(expectedResult).toContain(strategyApplication2);
        });

        it('should accept null and undefined values', () => {
          const strategyApplication: IStrategyApplication = { id: 123 };
          expectedResult = service.addStrategyApplicationToCollectionIfMissing([], null, strategyApplication, undefined);
          expect(expectedResult).toHaveLength(1);
          expect(expectedResult).toContain(strategyApplication);
        });

        it('should return initial array if no StrategyApplication is added', () => {
          const strategyApplicationCollection: IStrategyApplication[] = [{ id: 123 }];
          expectedResult = service.addStrategyApplicationToCollectionIfMissing(strategyApplicationCollection, undefined, null);
          expect(expectedResult).toEqual(strategyApplicationCollection);
        });
      });
    });

    afterEach(() => {
      httpMock.verify();
    });
  });
});
