import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import * as dayjs from 'dayjs';

import { isPresent } from 'app/core/util/operators';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption } from 'app/core/request/request-util';
import { IStrategyApplication, getStrategyApplicationIdentifier } from '../strategy-application.model';

export type EntityResponseType = HttpResponse<IStrategyApplication>;
export type EntityArrayResponseType = HttpResponse<IStrategyApplication[]>;

@Injectable({ providedIn: 'root' })
export class StrategyApplicationService {
  protected resourceUrl = this.applicationConfigService.getEndpointFor('api/strategy-applications', 'shbackend');

  constructor(protected http: HttpClient, protected applicationConfigService: ApplicationConfigService) {}

  create(strategyApplication: IStrategyApplication): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(strategyApplication);
    return this.http
      .post<IStrategyApplication>(this.resourceUrl, copy, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  update(strategyApplication: IStrategyApplication): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(strategyApplication);
    return this.http
      .put<IStrategyApplication>(`${this.resourceUrl}/${getStrategyApplicationIdentifier(strategyApplication) as number}`, copy, {
        observe: 'response',
      })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  partialUpdate(strategyApplication: IStrategyApplication): Observable<EntityResponseType> {
    const copy = this.convertDateFromClient(strategyApplication);
    return this.http
      .patch<IStrategyApplication>(`${this.resourceUrl}/${getStrategyApplicationIdentifier(strategyApplication) as number}`, copy, {
        observe: 'response',
      })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http
      .get<IStrategyApplication>(`${this.resourceUrl}/${id}`, { observe: 'response' })
      .pipe(map((res: EntityResponseType) => this.convertDateFromServer(res)));
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http
      .get<IStrategyApplication[]>(this.resourceUrl, { params: options, observe: 'response' })
      .pipe(map((res: EntityArrayResponseType) => this.convertDateArrayFromServer(res)));
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  addStrategyApplicationToCollectionIfMissing(
    strategyApplicationCollection: IStrategyApplication[],
    ...strategyApplicationsToCheck: (IStrategyApplication | null | undefined)[]
  ): IStrategyApplication[] {
    const strategyApplications: IStrategyApplication[] = strategyApplicationsToCheck.filter(isPresent);
    if (strategyApplications.length > 0) {
      const strategyApplicationCollectionIdentifiers = strategyApplicationCollection.map(
        strategyApplicationItem => getStrategyApplicationIdentifier(strategyApplicationItem)!
      );
      const strategyApplicationsToAdd = strategyApplications.filter(strategyApplicationItem => {
        const strategyApplicationIdentifier = getStrategyApplicationIdentifier(strategyApplicationItem);
        if (strategyApplicationIdentifier == null || strategyApplicationCollectionIdentifiers.includes(strategyApplicationIdentifier)) {
          return false;
        }
        strategyApplicationCollectionIdentifiers.push(strategyApplicationIdentifier);
        return true;
      });
      return [...strategyApplicationsToAdd, ...strategyApplicationCollection];
    }
    return strategyApplicationCollection;
  }

  protected convertDateFromClient(strategyApplication: IStrategyApplication): IStrategyApplication {
    return Object.assign({}, strategyApplication, {
      timestamp: strategyApplication.timestamp?.isValid() ? strategyApplication.timestamp.toJSON() : undefined,
    });
  }

  protected convertDateFromServer(res: EntityResponseType): EntityResponseType {
    if (res.body) {
      res.body.timestamp = res.body.timestamp ? dayjs(res.body.timestamp) : undefined;
    }
    return res;
  }

  protected convertDateArrayFromServer(res: EntityArrayResponseType): EntityArrayResponseType {
    if (res.body) {
      res.body.forEach((strategyApplication: IStrategyApplication) => {
        strategyApplication.timestamp = strategyApplication.timestamp ? dayjs(strategyApplication.timestamp) : undefined;
      });
    }
    return res;
  }
}
