jest.mock('@angular/router');

import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { of, Subject } from 'rxjs';

import { StrategyApplicationService } from '../service/strategy-application.service';
import { IStrategyApplication, StrategyApplication } from '../strategy-application.model';
import { IStrategy } from 'app/entities/shBackend/strategy/strategy.model';
import { StrategyService } from 'app/entities/shBackend/strategy/service/strategy.service';
import { IDeployment } from 'app/entities/shBackend/deployment/deployment.model';
import { DeploymentService } from 'app/entities/shBackend/deployment/service/deployment.service';

import { StrategyApplicationUpdateComponent } from './strategy-application-update.component';

describe('Component Tests', () => {
  describe('StrategyApplication Management Update Component', () => {
    let comp: StrategyApplicationUpdateComponent;
    let fixture: ComponentFixture<StrategyApplicationUpdateComponent>;
    let activatedRoute: ActivatedRoute;
    let strategyApplicationService: StrategyApplicationService;
    let strategyService: StrategyService;
    let deploymentService: DeploymentService;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        declarations: [StrategyApplicationUpdateComponent],
        providers: [FormBuilder, ActivatedRoute],
      })
        .overrideTemplate(StrategyApplicationUpdateComponent, '')
        .compileComponents();

      fixture = TestBed.createComponent(StrategyApplicationUpdateComponent);
      activatedRoute = TestBed.inject(ActivatedRoute);
      strategyApplicationService = TestBed.inject(StrategyApplicationService);
      strategyService = TestBed.inject(StrategyService);
      deploymentService = TestBed.inject(DeploymentService);

      comp = fixture.componentInstance;
    });

    describe('ngOnInit', () => {
      it('Should call Strategy query and add missing value', () => {
        const strategyApplication: IStrategyApplication = { id: 456 };
        const strategy: IStrategy = { id: 88753 };
        strategyApplication.strategy = strategy;

        const strategyCollection: IStrategy[] = [{ id: 30811 }];
        jest.spyOn(strategyService, 'query').mockReturnValue(of(new HttpResponse({ body: strategyCollection })));
        const additionalStrategies = [strategy];
        const expectedCollection: IStrategy[] = [...additionalStrategies, ...strategyCollection];
        jest.spyOn(strategyService, 'addStrategyToCollectionIfMissing').mockReturnValue(expectedCollection);

        activatedRoute.data = of({ strategyApplication });
        comp.ngOnInit();

        expect(strategyService.query).toHaveBeenCalled();
        expect(strategyService.addStrategyToCollectionIfMissing).toHaveBeenCalledWith(strategyCollection, ...additionalStrategies);
        expect(comp.strategiesSharedCollection).toEqual(expectedCollection);
      });

      it('Should call Deployment query and add missing value', () => {
        const strategyApplication: IStrategyApplication = { id: 456 };
        const deployment: IDeployment = { id: 77820 };
        strategyApplication.deployment = deployment;

        const deploymentCollection: IDeployment[] = [{ id: 14102 }];
        jest.spyOn(deploymentService, 'query').mockReturnValue(of(new HttpResponse({ body: deploymentCollection })));
        const additionalDeployments = [deployment];
        const expectedCollection: IDeployment[] = [...additionalDeployments, ...deploymentCollection];
        jest.spyOn(deploymentService, 'addDeploymentToCollectionIfMissing').mockReturnValue(expectedCollection);

        activatedRoute.data = of({ strategyApplication });
        comp.ngOnInit();

        expect(deploymentService.query).toHaveBeenCalled();
        expect(deploymentService.addDeploymentToCollectionIfMissing).toHaveBeenCalledWith(deploymentCollection, ...additionalDeployments);
        expect(comp.deploymentsSharedCollection).toEqual(expectedCollection);
      });

      it('Should update editForm', () => {
        const strategyApplication: IStrategyApplication = { id: 456 };
        const strategy: IStrategy = { id: 11396 };
        strategyApplication.strategy = strategy;
        const deployment: IDeployment = { id: 76547 };
        strategyApplication.deployment = deployment;

        activatedRoute.data = of({ strategyApplication });
        comp.ngOnInit();

        expect(comp.editForm.value).toEqual(expect.objectContaining(strategyApplication));
        expect(comp.strategiesSharedCollection).toContain(strategy);
        expect(comp.deploymentsSharedCollection).toContain(deployment);
      });
    });

    describe('save', () => {
      it('Should call update service on save for existing entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<StrategyApplication>>();
        const strategyApplication = { id: 123 };
        jest.spyOn(strategyApplicationService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ strategyApplication });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: strategyApplication }));
        saveSubject.complete();

        // THEN
        expect(comp.previousState).toHaveBeenCalled();
        expect(strategyApplicationService.update).toHaveBeenCalledWith(strategyApplication);
        expect(comp.isSaving).toEqual(false);
      });

      it('Should call create service on save for new entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<StrategyApplication>>();
        const strategyApplication = new StrategyApplication();
        jest.spyOn(strategyApplicationService, 'create').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ strategyApplication });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: strategyApplication }));
        saveSubject.complete();

        // THEN
        expect(strategyApplicationService.create).toHaveBeenCalledWith(strategyApplication);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).toHaveBeenCalled();
      });

      it('Should set isSaving to false on error', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<StrategyApplication>>();
        const strategyApplication = { id: 123 };
        jest.spyOn(strategyApplicationService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ strategyApplication });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.error('This is an error!');

        // THEN
        expect(strategyApplicationService.update).toHaveBeenCalledWith(strategyApplication);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).not.toHaveBeenCalled();
      });
    });

    describe('Tracking relationships identifiers', () => {
      describe('trackStrategyById', () => {
        it('Should return tracked Strategy primary key', () => {
          const entity = { id: 123 };
          const trackResult = comp.trackStrategyById(0, entity);
          expect(trackResult).toEqual(entity.id);
        });
      });

      describe('trackDeploymentById', () => {
        it('Should return tracked Deployment primary key', () => {
          const entity = { id: 123 };
          const trackResult = comp.trackDeploymentById(0, entity);
          expect(trackResult).toEqual(entity.id);
        });
      });
    });
  });
});
