import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize, map } from 'rxjs/operators';

import * as dayjs from 'dayjs';
import { DATE_TIME_FORMAT } from 'app/config/input.constants';

import { IStrategyApplication, StrategyApplication } from '../strategy-application.model';
import { StrategyApplicationService } from '../service/strategy-application.service';
import { AlertError } from 'app/shared/alert/alert-error.model';
import { EventManager, EventWithContent } from 'app/core/util/event-manager.service';
import { DataUtils, FileLoadError } from 'app/core/util/data-util.service';
import { IStrategy } from 'app/entities/shBackend/strategy/strategy.model';
import { StrategyService } from 'app/entities/shBackend/strategy/service/strategy.service';
import { IDeployment } from 'app/entities/shBackend/deployment/deployment.model';
import { DeploymentService } from 'app/entities/shBackend/deployment/service/deployment.service';

@Component({
  selector: 'jhi-strategy-application-update',
  templateUrl: './strategy-application-update.component.html',
})
export class StrategyApplicationUpdateComponent implements OnInit {
  isSaving = false;

  strategiesSharedCollection: IStrategy[] = [];
  deploymentsSharedCollection: IDeployment[] = [];

  editForm = this.fb.group({
    id: [],
    timestamp: [],
    summary: [],
    bundle: [],
    bundleContentType: [],
    strategy: [],
    deployment: [],
  });

  constructor(
    protected dataUtils: DataUtils,
    protected eventManager: EventManager,
    protected strategyApplicationService: StrategyApplicationService,
    protected strategyService: StrategyService,
    protected deploymentService: DeploymentService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ strategyApplication }) => {
      if (strategyApplication.id === undefined) {
        const today = dayjs().startOf('day');
        strategyApplication.timestamp = today;
      }

      this.updateForm(strategyApplication);

      this.loadRelationshipsOptions();
    });
  }

  byteSize(base64String: string): string {
    return this.dataUtils.byteSize(base64String);
  }

  openFile(base64String: string, contentType: string | null | undefined): void {
    this.dataUtils.openFile(base64String, contentType);
  }

  setFileData(event: Event, field: string, isImage: boolean): void {
    this.dataUtils.loadFileToForm(event, this.editForm, field, isImage).subscribe({
      error: (err: FileLoadError) =>
        this.eventManager.broadcast(new EventWithContent<AlertError>('shGatewayApp.error', { ...err, key: 'error.file.' + err.key })),
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const strategyApplication = this.createFromForm();
    if (strategyApplication.id !== undefined) {
      this.subscribeToSaveResponse(this.strategyApplicationService.update(strategyApplication));
    } else {
      this.subscribeToSaveResponse(this.strategyApplicationService.create(strategyApplication));
    }
  }

  trackStrategyById(index: number, item: IStrategy): number {
    return item.id!;
  }

  trackDeploymentById(index: number, item: IDeployment): number {
    return item.id!;
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IStrategyApplication>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(strategyApplication: IStrategyApplication): void {
    this.editForm.patchValue({
      id: strategyApplication.id,
      timestamp: strategyApplication.timestamp ? strategyApplication.timestamp.format(DATE_TIME_FORMAT) : null,
      summary: strategyApplication.summary,
      bundle: strategyApplication.bundle,
      bundleContentType: strategyApplication.bundleContentType,
      strategy: strategyApplication.strategy,
      deployment: strategyApplication.deployment,
    });

    this.strategiesSharedCollection = this.strategyService.addStrategyToCollectionIfMissing(
      this.strategiesSharedCollection,
      strategyApplication.strategy
    );
    this.deploymentsSharedCollection = this.deploymentService.addDeploymentToCollectionIfMissing(
      this.deploymentsSharedCollection,
      strategyApplication.deployment
    );
  }

  protected loadRelationshipsOptions(): void {
    this.strategyService
      .query()
      .pipe(map((res: HttpResponse<IStrategy[]>) => res.body ?? []))
      .pipe(
        map((strategies: IStrategy[]) =>
          this.strategyService.addStrategyToCollectionIfMissing(strategies, this.editForm.get('strategy')!.value)
        )
      )
      .subscribe((strategies: IStrategy[]) => (this.strategiesSharedCollection = strategies));

    this.deploymentService
      .query()
      .pipe(map((res: HttpResponse<IDeployment[]>) => res.body ?? []))
      .pipe(
        map((deployments: IDeployment[]) =>
          this.deploymentService.addDeploymentToCollectionIfMissing(deployments, this.editForm.get('deployment')!.value)
        )
      )
      .subscribe((deployments: IDeployment[]) => (this.deploymentsSharedCollection = deployments));
  }

  protected createFromForm(): IStrategyApplication {
    return {
      ...new StrategyApplication(),
      id: this.editForm.get(['id'])!.value,
      timestamp: this.editForm.get(['timestamp'])!.value ? dayjs(this.editForm.get(['timestamp'])!.value, DATE_TIME_FORMAT) : undefined,
      summary: this.editForm.get(['summary'])!.value,
      bundleContentType: this.editForm.get(['bundleContentType'])!.value,
      bundle: this.editForm.get(['bundle'])!.value,
      strategy: this.editForm.get(['strategy'])!.value,
      deployment: this.editForm.get(['deployment'])!.value,
    };
  }
}
