import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IStrategy, Strategy } from '../strategy.model';

import { StrategyService } from './strategy.service';

describe('Service Tests', () => {
  describe('Strategy Service', () => {
    let service: StrategyService;
    let httpMock: HttpTestingController;
    let elemDefault: IStrategy;
    let expectedResult: IStrategy | IStrategy[] | boolean | null;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
      });
      expectedResult = null;
      service = TestBed.inject(StrategyService);
      httpMock = TestBed.inject(HttpTestingController);

      elemDefault = {
        id: 0,
        code: 'AAAAAAA',
        description: 'AAAAAAA',
        configuration: 'AAAAAAA',
      };
    });

    describe('Service methods', () => {
      it('should find an element', () => {
        const returnedFromService = Object.assign({}, elemDefault);

        service.find(123).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(elemDefault);
      });

      it('should create a Strategy', () => {
        const returnedFromService = Object.assign(
          {
            id: 0,
          },
          elemDefault
        );

        const expected = Object.assign({}, returnedFromService);

        service.create(new Strategy()).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'POST' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should update a Strategy', () => {
        const returnedFromService = Object.assign(
          {
            id: 1,
            code: 'BBBBBB',
            description: 'BBBBBB',
            configuration: 'BBBBBB',
          },
          elemDefault
        );

        const expected = Object.assign({}, returnedFromService);

        service.update(expected).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PUT' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should partial update a Strategy', () => {
        const patchObject = Object.assign(
          {
            configuration: 'BBBBBB',
          },
          new Strategy()
        );

        const returnedFromService = Object.assign(patchObject, elemDefault);

        const expected = Object.assign({}, returnedFromService);

        service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PATCH' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should return a list of Strategy', () => {
        const returnedFromService = Object.assign(
          {
            id: 1,
            code: 'BBBBBB',
            description: 'BBBBBB',
            configuration: 'BBBBBB',
          },
          elemDefault
        );

        const expected = Object.assign({}, returnedFromService);

        service.query().subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush([returnedFromService]);
        httpMock.verify();
        expect(expectedResult).toContainEqual(expected);
      });

      it('should delete a Strategy', () => {
        service.delete(123).subscribe(resp => (expectedResult = resp.ok));

        const req = httpMock.expectOne({ method: 'DELETE' });
        req.flush({ status: 200 });
        expect(expectedResult);
      });

      describe('addStrategyToCollectionIfMissing', () => {
        it('should add a Strategy to an empty array', () => {
          const strategy: IStrategy = { id: 123 };
          expectedResult = service.addStrategyToCollectionIfMissing([], strategy);
          expect(expectedResult).toHaveLength(1);
          expect(expectedResult).toContain(strategy);
        });

        it('should not add a Strategy to an array that contains it', () => {
          const strategy: IStrategy = { id: 123 };
          const strategyCollection: IStrategy[] = [
            {
              ...strategy,
            },
            { id: 456 },
          ];
          expectedResult = service.addStrategyToCollectionIfMissing(strategyCollection, strategy);
          expect(expectedResult).toHaveLength(2);
        });

        it("should add a Strategy to an array that doesn't contain it", () => {
          const strategy: IStrategy = { id: 123 };
          const strategyCollection: IStrategy[] = [{ id: 456 }];
          expectedResult = service.addStrategyToCollectionIfMissing(strategyCollection, strategy);
          expect(expectedResult).toHaveLength(2);
          expect(expectedResult).toContain(strategy);
        });

        it('should add only unique Strategy to an array', () => {
          const strategyArray: IStrategy[] = [{ id: 123 }, { id: 456 }, { id: 68701 }];
          const strategyCollection: IStrategy[] = [{ id: 123 }];
          expectedResult = service.addStrategyToCollectionIfMissing(strategyCollection, ...strategyArray);
          expect(expectedResult).toHaveLength(3);
        });

        it('should accept varargs', () => {
          const strategy: IStrategy = { id: 123 };
          const strategy2: IStrategy = { id: 456 };
          expectedResult = service.addStrategyToCollectionIfMissing([], strategy, strategy2);
          expect(expectedResult).toHaveLength(2);
          expect(expectedResult).toContain(strategy);
          expect(expectedResult).toContain(strategy2);
        });

        it('should accept null and undefined values', () => {
          const strategy: IStrategy = { id: 123 };
          expectedResult = service.addStrategyToCollectionIfMissing([], null, strategy, undefined);
          expect(expectedResult).toHaveLength(1);
          expect(expectedResult).toContain(strategy);
        });

        it('should return initial array if no Strategy is added', () => {
          const strategyCollection: IStrategy[] = [{ id: 123 }];
          expectedResult = service.addStrategyToCollectionIfMissing(strategyCollection, undefined, null);
          expect(expectedResult).toEqual(strategyCollection);
        });
      });
    });

    afterEach(() => {
      httpMock.verify();
    });
  });
});
