import { Injectable } from '@angular/core';
import { HttpClient, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { isPresent } from 'app/core/util/operators';
import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { createRequestOption } from 'app/core/request/request-util';
import { IStrategy, getStrategyIdentifier } from '../strategy.model';

export type EntityResponseType = HttpResponse<IStrategy>;
export type EntityArrayResponseType = HttpResponse<IStrategy[]>;

@Injectable({ providedIn: 'root' })
export class StrategyService {
  protected resourceUrl = this.applicationConfigService.getEndpointFor('api/strategies', 'shbackend');

  constructor(protected http: HttpClient, protected applicationConfigService: ApplicationConfigService) {}

  create(strategy: IStrategy): Observable<EntityResponseType> {
    return this.http.post<IStrategy>(this.resourceUrl, strategy, { observe: 'response' });
  }

  update(strategy: IStrategy): Observable<EntityResponseType> {
    return this.http.put<IStrategy>(`${this.resourceUrl}/${getStrategyIdentifier(strategy) as number}`, strategy, { observe: 'response' });
  }

  partialUpdate(strategy: IStrategy): Observable<EntityResponseType> {
    return this.http.patch<IStrategy>(`${this.resourceUrl}/${getStrategyIdentifier(strategy) as number}`, strategy, {
      observe: 'response',
    });
  }

  find(id: number): Observable<EntityResponseType> {
    return this.http.get<IStrategy>(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  query(req?: any): Observable<EntityArrayResponseType> {
    const options = createRequestOption(req);
    return this.http.get<IStrategy[]>(this.resourceUrl, { params: options, observe: 'response' });
  }

  delete(id: number): Observable<HttpResponse<{}>> {
    return this.http.delete(`${this.resourceUrl}/${id}`, { observe: 'response' });
  }

  addStrategyToCollectionIfMissing(strategyCollection: IStrategy[], ...strategiesToCheck: (IStrategy | null | undefined)[]): IStrategy[] {
    const strategies: IStrategy[] = strategiesToCheck.filter(isPresent);
    if (strategies.length > 0) {
      const strategyCollectionIdentifiers = strategyCollection.map(strategyItem => getStrategyIdentifier(strategyItem)!);
      const strategiesToAdd = strategies.filter(strategyItem => {
        const strategyIdentifier = getStrategyIdentifier(strategyItem);
        if (strategyIdentifier == null || strategyCollectionIdentifiers.includes(strategyIdentifier)) {
          return false;
        }
        strategyCollectionIdentifiers.push(strategyIdentifier);
        return true;
      });
      return [...strategiesToAdd, ...strategyCollection];
    }
    return strategyCollection;
  }
}
