jest.mock('@angular/router');

import { ComponentFixture, TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { of, Subject } from 'rxjs';

import { StrategyService } from '../service/strategy.service';
import { IStrategy, Strategy } from '../strategy.model';
import { IDeployment } from 'app/entities/shBackend/deployment/deployment.model';
import { DeploymentService } from 'app/entities/shBackend/deployment/service/deployment.service';

import { StrategyUpdateComponent } from './strategy-update.component';

describe('Component Tests', () => {
  describe('Strategy Management Update Component', () => {
    let comp: StrategyUpdateComponent;
    let fixture: ComponentFixture<StrategyUpdateComponent>;
    let activatedRoute: ActivatedRoute;
    let strategyService: StrategyService;
    let deploymentService: DeploymentService;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
        declarations: [StrategyUpdateComponent],
        providers: [FormBuilder, ActivatedRoute],
      })
        .overrideTemplate(StrategyUpdateComponent, '')
        .compileComponents();

      fixture = TestBed.createComponent(StrategyUpdateComponent);
      activatedRoute = TestBed.inject(ActivatedRoute);
      strategyService = TestBed.inject(StrategyService);
      deploymentService = TestBed.inject(DeploymentService);

      comp = fixture.componentInstance;
    });

    describe('ngOnInit', () => {
      it('Should call Deployment query and add missing value', () => {
        const strategy: IStrategy = { id: 456 };
        const deployment: IDeployment = { id: 49042 };
        strategy.deployment = deployment;

        const deploymentCollection: IDeployment[] = [{ id: 37200 }];
        jest.spyOn(deploymentService, 'query').mockReturnValue(of(new HttpResponse({ body: deploymentCollection })));
        const additionalDeployments = [deployment];
        const expectedCollection: IDeployment[] = [...additionalDeployments, ...deploymentCollection];
        jest.spyOn(deploymentService, 'addDeploymentToCollectionIfMissing').mockReturnValue(expectedCollection);

        activatedRoute.data = of({ strategy });
        comp.ngOnInit();

        expect(deploymentService.query).toHaveBeenCalled();
        expect(deploymentService.addDeploymentToCollectionIfMissing).toHaveBeenCalledWith(deploymentCollection, ...additionalDeployments);
        expect(comp.deploymentsSharedCollection).toEqual(expectedCollection);
      });

      it('Should update editForm', () => {
        const strategy: IStrategy = { id: 456 };
        const deployment: IDeployment = { id: 30841 };
        strategy.deployment = deployment;

        activatedRoute.data = of({ strategy });
        comp.ngOnInit();

        expect(comp.editForm.value).toEqual(expect.objectContaining(strategy));
        expect(comp.deploymentsSharedCollection).toContain(deployment);
      });
    });

    describe('save', () => {
      it('Should call update service on save for existing entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<Strategy>>();
        const strategy = { id: 123 };
        jest.spyOn(strategyService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ strategy });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: strategy }));
        saveSubject.complete();

        // THEN
        expect(comp.previousState).toHaveBeenCalled();
        expect(strategyService.update).toHaveBeenCalledWith(strategy);
        expect(comp.isSaving).toEqual(false);
      });

      it('Should call create service on save for new entity', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<Strategy>>();
        const strategy = new Strategy();
        jest.spyOn(strategyService, 'create').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ strategy });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.next(new HttpResponse({ body: strategy }));
        saveSubject.complete();

        // THEN
        expect(strategyService.create).toHaveBeenCalledWith(strategy);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).toHaveBeenCalled();
      });

      it('Should set isSaving to false on error', () => {
        // GIVEN
        const saveSubject = new Subject<HttpResponse<Strategy>>();
        const strategy = { id: 123 };
        jest.spyOn(strategyService, 'update').mockReturnValue(saveSubject);
        jest.spyOn(comp, 'previousState');
        activatedRoute.data = of({ strategy });
        comp.ngOnInit();

        // WHEN
        comp.save();
        expect(comp.isSaving).toEqual(true);
        saveSubject.error('This is an error!');

        // THEN
        expect(strategyService.update).toHaveBeenCalledWith(strategy);
        expect(comp.isSaving).toEqual(false);
        expect(comp.previousState).not.toHaveBeenCalled();
      });
    });

    describe('Tracking relationships identifiers', () => {
      describe('trackDeploymentById', () => {
        it('Should return tracked Deployment primary key', () => {
          const entity = { id: 123 };
          const trackResult = comp.trackDeploymentById(0, entity);
          expect(trackResult).toEqual(entity.id);
        });
      });
    });
  });
});
