import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { FormBuilder, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize, map } from 'rxjs/operators';

import { IStrategy, Strategy } from '../strategy.model';
import { StrategyService } from '../service/strategy.service';
import { AlertError } from 'app/shared/alert/alert-error.model';
import { EventManager, EventWithContent } from 'app/core/util/event-manager.service';
import { DataUtils, FileLoadError } from 'app/core/util/data-util.service';
import { IDeployment } from 'app/entities/shBackend/deployment/deployment.model';
import { DeploymentService } from 'app/entities/shBackend/deployment/service/deployment.service';

@Component({
  selector: 'jhi-strategy-update',
  templateUrl: './strategy-update.component.html',
})
export class StrategyUpdateComponent implements OnInit {
  isSaving = false;

  deploymentsSharedCollection: IDeployment[] = [];

  editForm = this.fb.group({
    id: [],
    code: [null, [Validators.required]],
    description: [],
    configuration: [],
    deployment: [],
  });

  constructor(
    protected dataUtils: DataUtils,
    protected eventManager: EventManager,
    protected strategyService: StrategyService,
    protected deploymentService: DeploymentService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ strategy }) => {
      this.updateForm(strategy);

      this.loadRelationshipsOptions();
    });
  }

  byteSize(base64String: string): string {
    return this.dataUtils.byteSize(base64String);
  }

  openFile(base64String: string, contentType: string | null | undefined): void {
    this.dataUtils.openFile(base64String, contentType);
  }

  setFileData(event: Event, field: string, isImage: boolean): void {
    this.dataUtils.loadFileToForm(event, this.editForm, field, isImage).subscribe({
      error: (err: FileLoadError) =>
        this.eventManager.broadcast(new EventWithContent<AlertError>('shGatewayApp.error', { ...err, key: 'error.file.' + err.key })),
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const strategy = this.createFromForm();
    if (strategy.id !== undefined) {
      this.subscribeToSaveResponse(this.strategyService.update(strategy));
    } else {
      this.subscribeToSaveResponse(this.strategyService.create(strategy));
    }
  }

  trackDeploymentById(index: number, item: IDeployment): number {
    return item.id!;
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IStrategy>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe(
      () => this.onSaveSuccess(),
      () => this.onSaveError()
    );
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(strategy: IStrategy): void {
    this.editForm.patchValue({
      id: strategy.id,
      code: strategy.code,
      description: strategy.description,
      configuration: strategy.configuration,
      deployment: strategy.deployment,
    });

    this.deploymentsSharedCollection = this.deploymentService.addDeploymentToCollectionIfMissing(
      this.deploymentsSharedCollection,
      strategy.deployment
    );
  }

  protected loadRelationshipsOptions(): void {
    this.deploymentService
      .query()
      .pipe(map((res: HttpResponse<IDeployment[]>) => res.body ?? []))
      .pipe(
        map((deployments: IDeployment[]) =>
          this.deploymentService.addDeploymentToCollectionIfMissing(deployments, this.editForm.get('deployment')!.value)
        )
      )
      .subscribe((deployments: IDeployment[]) => (this.deploymentsSharedCollection = deployments));
  }

  protected createFromForm(): IStrategy {
    return {
      ...new Strategy(),
      id: this.editForm.get(['id'])!.value,
      code: this.editForm.get(['code'])!.value,
      description: this.editForm.get(['description'])!.value,
      configuration: this.editForm.get(['configuration'])!.value,
      deployment: this.editForm.get(['deployment'])!.value,
    };
  }
}
