#!/bin/sh
set -e

if [ -z "$TRAEFIK_CONF_FILE_NAME" ]
then 
  echo TRAEFIK_CONF_FILE_NAME environment variable required 
  exit 1
fi 

mkdir -p etc/traefik > /dev/null 2>&1 || true

TRAEFIK_CONF_PATH=etc/traefik/$TRAEFIK_CONF_FILE_NAME
rm $TRAEFIK_CONF_PATH > /dev/null 2>&1 || true
touch $TRAEFIK_CONF_PATH

if [ "$ENABLE_LOG" = "true" ]
then 
	cat <<- EOF >>$TRAEFIK_CONF_PATH
	log:
	  level: DEBUG

	EOF
fi

# add providers section
cat <<- EOF >>$TRAEFIK_CONF_PATH
providers:
  docker:
    constraints: Label(\`com.docker.compose.project\`, \`$COMPOSE_PROJECT\`)
    exposedByDefault: false # this requires to add 
  file:
    debugLogGeneratedTemplate: true
    directory: /dynamic/

EOF

# add insecureSkip for internal servers
cat <<- EOF >>$TRAEFIK_CONF_PATH
serversTransport:
  insecureSkipVerify: true

EOF

# add ping section for health check
cat <<- EOF >>$TRAEFIK_CONF_PATH
ping: {}

EOF

if [ "$ENABLE_API" = "true" ] || [ "$ENABLE_DASHBOARD" = "true" ]
then 
	cat <<- EOF >>$TRAEFIK_CONF_PATH
	api:
	  dashboard: $ENABLE_DASHBOARD
	  insecure: true # this makes it accessible in the traefik entrypoint or in port 8080 if no traefik otherwise it is accessible in service api@internal
	
	EOF
fi

mkdir dynamic > /dev/null 2>&1 || true

if [ "$ROUTE_API" = "true" ]
then
	sed -e "s/ROOT_DOMAIN/$ROOT_DOMAIN/g" templates/api.yml > dynamic/api.yml
fi

if [ "$ROUTE_DASHBOARD" = "true" ]
then
	sed -e "s/ROOT_DOMAIN/$ROOT_DOMAIN/g" templates/dashboard.yml > dynamic/dashboard.yml
fi

if [ "$DISSABLE_CORS" = "true" ]
then
	cp templates/cors.yml dynamic/cors.yml
fi

# start entryPoints section
cat <<- EOF >>$TRAEFIK_CONF_PATH
entryPoints:
EOF

if [ "$ENABLE_HTTP_TO_HTTPS_REDIRECT" = "true" ] 
then 
	cat <<- EOF >>$TRAEFIK_CONF_PATH
	  web:
	    address: ":$HTTP_PORT"
	EOF
fi

if [ "$ENABLE_HTTP_TO_HTTPS_REDIRECT" = "false" ] || [ "$ADD_SELFSIGNED_CA" = "false" ] && [ "$ENABLE_ACME" = "true" ] && [ "$ACME_CHALLENGE" = "acme-httpChallenge" ]
then 
	if [ "$HTTP_PORT" != "80" ]
	then
		echo WARNING HTTP_PORT has invalid value, when using acme-httpChallenge it must be 80
		exit 1
	fi
	cat <<- EOF >>$TRAEFIK_CONF_PATH
	  web:
	    address: ":80"
	EOF
fi

# add websecure entryPoints section
cat <<- EOF >>$TRAEFIK_CONF_PATH
  websecure:
    address: ":$HTTPS_PORT"
EOF

if [ "$ADD_SELFSIGNED_CA" = "true" ]
then
	cat <<- EOF >>$TRAEFIK_CONF_PATH
	    http: 
	      tls: {}
	EOF
	cp templates/tls.yml dynamic/tls.yml
	if [ ! -f certificate.crt.pem ]
	then
		sh createCertificate.sh
	fi
	if [ "$ENABLE_CA_REDIRECT" = "true" ]
	then
		sed -e "s/ROOT_DOMAIN/$ROOT_DOMAIN/g" -e "s|CA_REDIRECT_TARGET|$CA_REDIRECT_TARGET|g" templates/ca-redirect.yml > dynamic/ca-redirect.yml
	fi
fi

if [ "$ADD_SELFSIGNED_CA" = "false" ] && [ "$ENABLE_ACME" = "true" ]
then
	if [ "$ACME_CHALLENGE" = "acme-tlsChallenge" ] || [ "$ACME_CHALLENGE" = "acme-httpChallenge" ]
	then
		if [ "$HTTPS_PORT" != "443" ]  
		then
			echo WARNING HTTPS_PORT has invalid value, when using acme it must be 443
			exit 1
		fi
	fi
	cat <<- EOF >>$TRAEFIK_CONF_PATH
	    http:
	    tls:
	      certResolver: myresolver
	      domains:
	        - main: $ROOT_DOMAIN
	EOF
	if [ "$ACME_SANS" != "false" ]
	then
		echo "          sans:" >>$TRAEFIK_CONF_PATH

		for i in $(echo $ACME_SANS | sed "s/,/ /g")
		do
			echo "            - $i" >>$TRAEFIK_CONF_PATH
		done
	fi
	echo >> $TRAEFIK_CONF_PATH
	if [ "$ENABLE_CA_REDIRECT" = "true" ]
	then
		sed -e "s/ACME_MAIL/$ACME_MAIL/g" -e "s|ACME_CA_SERVER|$ACME_CA_SERVER|g" templates/$ACME_CHALLENGE.yml >> $TRAEFIK_CONF_PATH		
	fi
fi

cat $TRAEFIK_CONF_PATH
